@@ -51,7 +51,7 @@ struct ar_archive {
 	} currentheader;
 	off_t bytes_left;
 	void *readbuffer;
-	bool_t wasodd;
+	bool wasodd;
 };
 
 static ssize_t readwait(int fd, /*@out@*/void *buf, size_t count) {
@@ -88,7 +88,7 @@ retvalue ar_open(/*@out@*/struct ar_archive **n, const char *filename) {
 	ssize_t bytesread;
 
 	if( interrupted() )
-		return RET_ERROR_INTERUPTED;
+		return RET_ERROR_INTERRUPTED;
 	ar = calloc(1,sizeof(struct ar_archive));
 	if( ar == NULL )
 		return RET_ERROR_OOM;
@@ -161,7 +161,7 @@ retvalue ar_nextmember(struct ar_archive *ar,/*@out@*/char **filename) {
 	/* read the next header from the file */
 
 	if( interrupted() )
-		return RET_ERROR_INTERUPTED;
+		return RET_ERROR_INTERRUPTED;
 
 	bytesread = readwait(ar->fd,&ar->currentheader,sizeof(ar->currentheader));
 	if( bytesread == 0 )
@@ -191,10 +191,10 @@ retvalue ar_nextmember(struct ar_archive *ar,/*@out@*/char **filename) {
 		return RET_ERROR;
 	}
 	if( (ar->bytes_left & 1) != 0 )
-		ar->wasodd = TRUE;
+		ar->wasodd = true;
 
 	/* get the name of the file */
-	if( FALSE ) {
+	if( false ) {
 		/* handle long filenames */
 		// TODO!
 	} else {
