@@ -29,9 +29,9 @@
 #include "error.h"
 #include "ignore.h"
 #include "strlist.h"
-#include "md5sum.h"
-#include "copyfile.h"
+#include "checksums.h"
 #include "names.h"
+#include "filecntl.h"
 #include "dirs.h"
 #include "chunks.h"
 #include "reference.h"
@@ -75,7 +75,7 @@ struct fileentry {
 	struct fileentry *next;
 	char *basename;
 	filetype type;
-	char *md5sum;
+	struct checksums *checksums;
 	char *section;
 	char *priority;
 	char *architecture;
@@ -87,10 +87,12 @@ struct fileentry {
 	/* only set after changes_includefiles */
 	char *filekey;
 	/* was already found in the pool before */
-	bool_t wasalreadythere;
-	bool_t included;
+	bool wasalreadythere;
+	bool included;
 	/* set between checkpkg and includepkg */
-	union { struct dscpackage *dsc; struct debpackage *deb;} pkg;
+	struct strlist needed_filekeys;
+	union { struct dsc_headers dsc;
+		struct debpackage *deb;} pkg;
 };
 
 struct changes {
@@ -107,6 +109,7 @@ struct changes {
 	const char *srccomponent;
 	/* != NULL if changesfile was put into pool/ */
 	/*@null@*/ char *changesfilekey;
+	bool includedchangesfile;
 	/* the directory where source files are put into */
 	char *srcdirectory;
 	/* (only to warn if multiple are used) */
@@ -114,7 +117,7 @@ struct changes {
 	/* the directory the .changes file resides in */
 	char *incomingdirectory;
 	/* the Version: and the version in Source: differ */
-	bool_t isbinnmu;
+	bool isbinnmu;
 };
 
 static void freeentries(/*@only@*/struct fileentry *entry) {
@@ -125,15 +128,17 @@ static void freeentries(/*@only@*/struct fileentry *entry) {
 		free(entry->filekey);
 		free(entry->component);
 		free(entry->basename);
-		free(entry->md5sum);
+		checksums_free(entry->checksums);
 		free(entry->section);
 		free(entry->priority);
 		free(entry->architecture);
 		free(entry->name);
 		if( entry->type == fe_DEB || entry->type == fe_UDEB)
 			deb_free(entry->pkg.deb);
-		 else if( entry->type == fe_DSC )
-			dsc_free(entry->pkg.dsc);
+		 else if( entry->type == fe_DSC ) {
+			 strlist_done(&entry->needed_filekeys);
+			 sources_done(&entry->pkg.dsc);
+		 }
 		free(entry);
 		entry = h;
 	}
@@ -167,7 +172,7 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 	if( e == NULL )
 		return RET_ERROR_OOM;
 
-	r = changes_parsefileline(fileline, &e->type, &e->basename, &e->md5sum,
+	r = changes_parsefileline(fileline, &e->type, &e->basename, &e->checksums,
 			&e->section, &e->priority, &e->architecture, &e->name);
 	if( RET_WAS_ERROR(r) ) {
 		free(e);
@@ -256,7 +261,7 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	retvalue r;
 	struct changes *c;
 	struct strlist filelines;
-	bool_t broken;
+	bool broken;
 	int versioncmp;
 
 #define E(err) { \
@@ -316,7 +321,7 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 			changes_free(c);
 			return RET_ERROR_OOM;
 		}
-		c->isbinnmu = FALSE;
+		c->isbinnmu = false;
 	} else {
 		r = dpkgversions_cmp(c->sourceversion, c->changesversion,
 				&versioncmp);
@@ -329,10 +334,6 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	R;
 	r = check(filename,c,"Maintainer");
 	R;
-	r = check(filename,c,"Description");
-	R;
-	r = check(filename,c,"Changes");
-	R;
 	r = chunk_getextralinelist(c->control,"Files",&filelines);
 	E("Missing 'Files' field!");
 	r = changes_parsefilelines(filename,c,&filelines,packagetypeonly,forcearchitecture);
@@ -495,7 +496,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 	int i;
 	struct fileentry *e;
 	retvalue r = RET_OK;
-	bool_t havedsc=FALSE, haveorig=FALSE, havetar=FALSE, havediff=FALSE;
+	bool havedsc = false, haveorig = false, havetar = false, havediff = false;
 
 	/* First check for each given architecture, if it has files: */
 	if( forcearchitecture != NULL ) {
@@ -550,7 +551,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 				fprintf(stderr,"I don't know what to do with multiple .dsc files in '%s'!\n",filename);
 				return RET_ERROR;
 			}
-			havedsc = TRUE;
+			havedsc = true;
 			calculatedname = calc_source_basename(changes->source,changes->sourceversion);
 			if( calculatedname == NULL )
 				return RET_ERROR_OOM;
@@ -565,19 +566,19 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 				fprintf(stderr,"I don't know what to do with multiple .diff files in '%s'!\n",filename);
 				return RET_ERROR;
 			}
-			havediff = TRUE;
+			havediff = true;
 		} else if( e->type == fe_ORIG ) {
 			if( haveorig ) {
 				fprintf(stderr,"I don't know what to do with multiple .orig.tar.gz files in '%s'!\n",filename);
 				return RET_ERROR;
 			}
-			haveorig = TRUE;
+			haveorig = true;
 		} else if( e->type == fe_TAR ) {
 			if( havetar ) {
 				fprintf(stderr,"I don't know what to do with multiple .tar.gz files in '%s'!\n",filename);
 				return RET_ERROR;
 			}
-			havetar = TRUE;
+			havetar = true;
 		}
 
 		e = e->next;
@@ -608,7 +609,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 	return r;
 }
 
-static retvalue changes_checkfiles(filesdb filesdb,const char *filename,struct changes *changes) {
+static retvalue changes_checkfiles(struct database *database,const char *filename,struct changes *changes) {
 	struct fileentry *e;
 	retvalue r;
 
@@ -635,17 +636,17 @@ static retvalue changes_checkfiles(filesdb filesdb,const char *filename,struct c
 		if( e->filekey == NULL )
 			return RET_ERROR_OOM;
 		/* do not copy yet, but only check if it could be included */
-		r = files_expect(filesdb,e->filekey,e->md5sum);
+		r = files_canadd(database, e->filekey, e->checksums);
 		if( RET_WAS_ERROR(r) )
 			return r;
 		/* If is was already there, remember that */
-		if( RET_IS_OK(r) ) {
-			e->wasalreadythere = TRUE;
+		if( r == RET_NOTHING ) {
+			e->wasalreadythere = true;
 		} else {
 		/* and if it needs inclusion check if there is a file */
 			char *fullfilename;
 
-			assert( r == RET_NOTHING );
+			assert(RET_IS_OK(r));
 			// TODO: add a --paranoid to also check md5sums before copying?
 
 			fullfilename = calc_dirconcat(changes->incomingdirectory,e->basename);
@@ -663,7 +664,7 @@ static retvalue changes_checkfiles(filesdb filesdb,const char *filename,struct c
 	return RET_OK;
 }
 
-static retvalue changes_includefiles(filesdb filesdb,struct changes *changes,int delete) {
+static retvalue changes_includefiles(struct database *database,struct changes *changes) {
 	struct fileentry *e;
 	retvalue r;
 
@@ -672,15 +673,15 @@ static retvalue changes_includefiles(filesdb filesdb,struct changes *changes,int
 	for( e = changes->files; e != NULL ; e = e->next ) {
 		assert( e->filekey != NULL );
 
-		if( e->wasalreadythere )
+		if( e->wasalreadythere && checksums_iscomplete(e->checksums) )
 			continue;
 
-		r = files_includefile(filesdb,changes->incomingdirectory,
-				e->basename,e->filekey,e->md5sum,NULL,
-				/* do not delete, we do that later outself */
-				(delete>=D_MOVE)?D_COPY:delete);
-		if( RET_IS_OK(r) )
-			e->included = TRUE;
+		e->included = false;
+		r = files_checkincludefile(database,
+				changes->incomingdirectory, e->basename,
+				e->filekey, &e->checksums, &e->included);
+		assert( e->included || e->wasalreadythere );
+		assert( !(e->included && e->wasalreadythere) );
 		if( RET_WAS_ERROR(r) )
 			return r;
 	}
@@ -689,7 +690,7 @@ static retvalue changes_includefiles(filesdb filesdb,struct changes *changes,int
 }
 /* run if packages are not all includeable and the stuff put into the
  * pool shall be removed again */
-static void changes_unincludefiles(filesdb filesdb,struct changes *changes) {
+static void changes_unincludefiles(struct database *database,struct changes *changes) {
 	struct fileentry *e;
 
 	for( e = changes->files; e != NULL ; e = e->next ) {
@@ -697,9 +698,40 @@ static void changes_unincludefiles(filesdb filesdb,struct changes *changes) {
 		if( e->filekey == NULL || e->wasalreadythere || !e->included )
 			continue;
 
-		(void)files_deleteandremove(filesdb,e->filekey,TRUE,FALSE);
+		(void)files_deleteandremove(database, e->filekey, true, false);
+	}
+	if( changes->includedchangesfile )
+		(void)files_deleteandremove(database, changes->changesfilekey,
+			true, false);
+}
+
+static void check_all_files_included(struct database *database, struct changes *changes, bool unusedpartsexpected) {
+	struct fileentry *e;
+	retvalue r;
+
+	for( e = changes->files; e != NULL ; e = e->next ) {
+
+		if( e->filekey == NULL || e->wasalreadythere || !e->included )
+			continue;
+
+		if( unusedpartsexpected ) {
+			(void)files_deleteandremove(database, e->filekey, true, false);
+			continue;
+		}
+
+		fprintf(stderr, "Warning: File '%s' was listed in the .changes\n"
+			" but looks like not used. Checking for references...\n",
+			e->filekey);
+		r = references_isused(database, e->filekey);
+		if( r == RET_NOTHING ) {
+			fprintf(stderr, " indeed unused, deleting it...\n");
+			(void)files_deleteandremove(database, e->filekey, true, false);
+		} else if( RET_IS_OK(r) ) {
+			fprintf(stderr, " It looks used. Strange. Please report this bug.\n");
+		}
 	}
 }
+
 /* delete the files included */
 static retvalue changes_deleteleftoverfiles(struct changes *changes,int delete) {
 	struct fileentry *e;
@@ -734,7 +766,148 @@ static retvalue changes_deleteleftoverfiles(struct changes *changes,int delete)
 	return result;
 }
 
-static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribution,struct changes *changes, const char *sourcedirectory) {
+static retvalue changes_check_sourcefile(struct changes *changes, struct fileentry *dsc, struct database *database, const char *basename, const char *filekey, struct checksums **checksums_p) {
+	retvalue r;
+	bool dummy = false;
+
+	r = files_expect(database, filekey, *checksums_p);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	// TODO: get additionals checksum out of database, as future
+	// source file completion code might need them...
+	if( RET_IS_OK(r) )
+		return RET_OK;
+	if( !IGNORABLE(missingfile) ) {
+		fprintf(stderr,
+"Unable to find %s needed by %s!\n"
+"Perhaps you forgot to give dpkg-buildpackage the -sa option,\n"
+" or you cound try --ignore=missingfile, to guess possible files to use.\n",
+			filekey, dsc->basename);
+		return RET_ERROR_MISSING;
+	}
+	fprintf(stderr,
+"Unable to find %s!\n"
+"Perhaps you forgot to give dpkg-buildpackage the -sa option.\n"
+"Searching for it because --ignore=missingfile was given...\n", filekey);
+
+	// TODO: if this is included and a error occours, it currently stays.
+	// how can this be fixed?
+	return files_checkincludefile(database, changes->incomingdirectory,
+			basename, filekey, checksums_p, &dummy);
+}
+
+static retvalue dsc_prepare(struct changes *changes, struct fileentry *dsc, struct database *database, struct distribution *distribution, const char *dscfilename){
+	retvalue r;
+	const struct overrideinfo *oinfo;
+	char *dscbasename;
+	char *control;
+	int i;
+	bool broken;
+
+	assert( dsc->section != NULL );
+	assert( dsc->priority != NULL );
+	assert( changes->srccomponent != NULL );
+	assert( dsc->basename != NULL );
+	assert( dsc->checksums != NULL );
+	assert( changes->source != NULL );
+	assert( changes->sourceversion != NULL );
+
+	/* First make sure this distribution has a source section at all,
+	 * for which it has to be listed in the "Architectures:"-field ;-) */
+	if( !strlist_in(&distribution->architectures,"source") ) {
+		fprintf(stderr,"Cannot put a source package into Distribution '%s' not having 'source' in its 'Architectures:'-field!\n",distribution->codename);
+		/* nota bene: this cannot be forced or ignored, as no target has
+		   been created for this. */
+		return RET_ERROR;
+	}
+
+	/* Then take a closer look in the file: */
+	r = sources_readdsc(&dsc->pkg.dsc, dscfilename, dscfilename, &broken);
+	if( RET_IS_OK(r) && broken && !IGNORING_(brokensignatures,
+"'%s' contains only broken signatures.\n"
+"This most likely means the file was damaged (or edited improperly)\n",
+				dscfilename) )
+		r = RET_ERROR;
+	if( RET_IS_OK(r) )
+		r = propersourcename(dsc->pkg.dsc.name);
+	if( RET_IS_OK(r) )
+		r = properversion(dsc->pkg.dsc.version);
+	if( RET_IS_OK(r) )
+		r = properfilenames(&dsc->pkg.dsc.files.names);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	if( strcmp(changes->source, dsc->pkg.dsc.name) != 0 ) {
+		/* This cannot be ignored, as too much depends on it yet */
+		fprintf(stderr,
+"'%s' says it is '%s', while .changes file said it is '%s'\n",
+				dsc->basename, dsc->pkg.dsc.name, changes->source);
+		return RET_ERROR;
+	}
+	if( strcmp(changes->sourceversion, dsc->pkg.dsc.version) != 0 &&
+	    !IGNORING_(wrongversion,
+"'%s' says it is version '%s', while .changes file said it is '%s'\n",
+				dsc->basename, dsc->pkg.dsc.version,
+				changes->sourceversion)) {
+		return RET_ERROR;
+	}
+
+	oinfo = override_search(distribution->overrides.dsc, dsc->pkg.dsc.name);
+
+	free(dsc->pkg.dsc.section);
+	dsc->pkg.dsc.section = strdup(dsc->section);
+	if( dsc->pkg.dsc.section == NULL )
+		return RET_ERROR_OOM;
+	free(dsc->pkg.dsc.priority);
+	dsc->pkg.dsc.priority = strdup(dsc->priority);
+	if( dsc->pkg.dsc.priority == NULL )
+		return RET_ERROR_OOM;
+
+	assert( dsc->pkg.dsc.name != NULL && dsc->pkg.dsc.version != NULL );
+
+	/* Add the dsc file to the list of files in this source package: */
+	dscbasename = strdup(dsc->basename);
+	if( dscbasename == NULL )
+		r = RET_ERROR_OOM;
+	else
+		r = checksumsarray_include(&dsc->pkg.dsc.files,
+				dscbasename, dsc->checksums);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	/* Calculate the filekeys: */
+	r = calc_dirconcats(changes->srcdirectory,
+			&dsc->pkg.dsc.files.names, &dsc->needed_filekeys);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	/* noone else might have looked yet, if we have them: */
+
+	assert( dsc->pkg.dsc.files.names.count == dsc->needed_filekeys.count );
+	for( i = 1 ; i < dsc->pkg.dsc.files.names.count ; i ++ ) {
+		if( !RET_WAS_ERROR(r) ) {
+			r = changes_check_sourcefile(
+				changes, dsc, database,
+				dsc->pkg.dsc.files.names.values[i],
+				dsc->needed_filekeys.values[i],
+				&dsc->pkg.dsc.files.checksums[i]);
+		}
+	}
+
+	if( !RET_WAS_ERROR(r) )
+		r = sources_complete(&dsc->pkg.dsc, changes->srcdirectory,
+				oinfo,
+				dsc->pkg.dsc.section, dsc->pkg.dsc.priority,
+				&control);
+	if( RET_IS_OK(r) ) {
+		free(dsc->pkg.dsc.control);
+		dsc->pkg.dsc.control = control;
+	}
+	return r;
+}
+
+
+static retvalue changes_checkpkgs(struct database *database, struct distribution *distribution, struct changes *changes) {
 	struct fileentry *e;
 	retvalue r;
 
@@ -747,29 +920,27 @@ static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribut
 			e = e->next;
 			continue;
 		}
-		fullfilename = files_calcfullfilename(filesdb,e->filekey);
+		fullfilename = files_calcfullfilename(database, e->filekey);
 		if( fullfilename == NULL )
 			return RET_ERROR_OOM;
 		if( e->type == fe_DEB ) {
-			r = deb_prepare(&e->pkg.deb,filesdb,
-				e->component,e->architecture,
-				e->section,e->priority,
+			r = deb_prepare(&e->pkg.deb,
+				e->component, e->architecture,
+				e->section, e->priority,
 				"deb",
-				distribution,fullfilename,
-				e->filekey,e->md5sum,
-				D_INPLACE,FALSE,
+				distribution, fullfilename,
+				e->filekey, e->checksums,
 				&changes->binaries,
-				changes->source,changes->sourceversion);
+				changes->source, changes->sourceversion);
 		} else if( e->type == fe_UDEB ) {
-			r = deb_prepare(&e->pkg.deb,filesdb,
-				e->component,e->architecture,
-				e->section,e->priority,
+			r = deb_prepare(&e->pkg.deb,
+				e->component, e->architecture,
+				e->section, e->priority,
 				"udeb",
-				distribution,fullfilename,
-				e->filekey,e->md5sum,
-				D_INPLACE,FALSE,
+				distribution, fullfilename,
+				e->filekey, e->checksums,
 				&changes->binaries,
-				changes->source,changes->sourceversion);
+				changes->source, changes->sourceversion);
 		} else if( e->type == fe_DSC ) {
 			if( !changes->isbinnmu || IGNORING_(dscinbinnmu,
 "File '%s' looks like a source package, but this .changes looks like a binNMU\n"
@@ -779,13 +950,8 @@ static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribut
 
 				assert(changes->srccomponent!=NULL);
 				assert(changes->srcdirectory!=NULL);
-				r = dsc_prepare(&e->pkg.dsc,filesdb,
-						changes->srccomponent,e->section,e->priority,
-						distribution,sourcedirectory,fullfilename,
-						e->filekey,e->basename,
-						changes->srcdirectory,e->md5sum,
-						D_INPLACE,
-						changes->source,changes->sourceversion);
+				r = dsc_prepare(changes, e, database,
+						distribution, fullfilename);
 			} else
 				r = RET_ERROR;
 		}
@@ -798,11 +964,13 @@ static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribut
 
 	return r;
 }
-static retvalue changes_includepkgs(const char *dbdir,references refs,struct distribution *distribution,struct changes *changes,/*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *trackingdata,bool_t *missed_p) {
+
+static retvalue changes_includepkgs(struct database *database, struct distribution *distribution, struct changes *changes, /*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *trackingdata, bool *missed_p) {
 	struct fileentry *e;
 	retvalue result,r;
+	bool markedasused;
 
-	*missed_p = FALSE;
+	*missed_p = false;
 	r = distribution_prepareforwriting(distribution);
 	if( RET_WAS_ERROR(r) )
 		return r;
@@ -815,26 +983,48 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,struct dis
 			e = e->next;
 			continue;
 		}
-		if( interrupted() ) {
-			return RET_ERROR_INTERUPTED;
-		}
+		if( interrupted() )
+			return RET_ERROR_INTERRUPTED;
+		markedasused = false;
 		if( e->type == fe_DEB ) {
-			r = deb_addprepared(e->pkg.deb,dbdir,refs,
-				e->architecture,"deb",
-				distribution,dereferencedfilekeys,trackingdata);
+			r = deb_addprepared(e->pkg.deb, database,
+				e->architecture, "deb",
+				distribution, dereferencedfilekeys,
+				trackingdata, &markedasused);
 			if( r == RET_NOTHING )
-				*missed_p = TRUE;
+				*missed_p = true;
+			if( markedasused )
+				e->included = false;
 		} else if( e->type == fe_UDEB ) {
-			r = deb_addprepared(e->pkg.deb,dbdir,refs,
-				e->architecture,"udeb",
-				distribution,dereferencedfilekeys,trackingdata);
+			r = deb_addprepared(e->pkg.deb, database,
+				e->architecture, "udeb",
+				distribution,dereferencedfilekeys,
+				trackingdata, &markedasused);
 			if( r == RET_NOTHING )
-				*missed_p = TRUE;
+				*missed_p = true;
+			if( markedasused )
+				e->included = false;
 		} else if( e->type == fe_DSC ) {
-			r = dsc_addprepared(e->pkg.dsc,dbdir,refs,
-				distribution,dereferencedfilekeys,trackingdata);
+			r = dsc_addprepared(database, &e->pkg.dsc,
+					changes->srccomponent,
+					&e->needed_filekeys, &markedasused,
+					distribution, dereferencedfilekeys,
+					trackingdata);
 			if( r == RET_NOTHING )
-				*missed_p = TRUE;
+				*missed_p = true;
+			if( markedasused ) {
+				struct fileentry *f;
+				const struct strlist *referenced = &e->needed_filekeys;
+
+				/* mark any file no longer allowed to be deleted */
+
+				for( f = changes->files ; f != NULL ; f = f->next ) {
+					if( !f->included )
+						continue;
+					if( strlist_in(referenced, f->filekey) )
+						f->included = false;
+				}
+			}
 		}
 		RET_UPDATE(result, r);
 
@@ -848,7 +1038,7 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,struct dis
 	return result;
 }
 
-static bool_t permissionssuffice(UNUSED(struct changes *changes),
+static bool permissionssuffice(UNUSED(struct changes *changes),
                                  const struct uploadpermissions *permissions) {
 	return permissions->allowall;
 }
@@ -856,12 +1046,11 @@ static bool_t permissionssuffice(UNUSED(struct changes *changes),
 /* insert the given .changes into the mirror in the <distribution>
  * if forcecomponent, forcesection or forcepriority is NULL
  * get it from the files or try to guess it. */
-retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,filesdb filesdb,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,const char *changesfilename,int delete,struct strlist *dereferencedfilekeys) {
+retvalue changes_add(struct database *database,trackingdb const tracks,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,const char *changesfilename,int delete,struct strlist *dereferencedfilekeys) {
 	retvalue result,r;
 	struct changes *changes;
 	struct trackingdata trackingdata;
-	char *directory;
-	bool_t somethingwasmissed;
+	bool somethingwasmissed;
 
 	r = changes_read(changesfilename,&changes,packagetypeonly,forcearchitecture);
 	if( RET_WAS_ERROR(r) )
@@ -916,16 +1105,6 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		}
 	}
 
-	if( IGNORABLE(missingfile) ) {
-		r = dirs_getdirectory(changesfilename,&directory);
-		if( RET_WAS_ERROR(r) ) {
-			changes_free(changes);
-			return r;
-		}
-	} else
-		directory = NULL;
-
-
 	/* look for component, section and priority to be correct or guess them*/
 	r = changes_fixfields(distribution,changesfilename,changes,forcecomponent,forcesection,forcepriority);
 
@@ -934,25 +1113,23 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		r = changes_check(changesfilename,changes,forcearchitecture,packagetypeonly);
 
 	if( interrupted() )
-		RET_UPDATE(r,RET_ERROR_INTERUPTED);
+		RET_UPDATE(r, RET_ERROR_INTERRUPTED);
 
 	if( !RET_WAS_ERROR(r) )
-		r = changes_checkfiles(filesdb,changesfilename,changes);
+		r = changes_checkfiles(database, changesfilename, changes);
 
 	if( interrupted() )
-		RET_UPDATE(r,RET_ERROR_INTERUPTED);
+		RET_UPDATE(r, RET_ERROR_INTERRUPTED);
 
 	/* add files in the pool */
 	if( !RET_WAS_ERROR(r) )
-		r = changes_includefiles(filesdb,changes,delete);
+		r = changes_includefiles(database, changes);
 
 	if( !RET_WAS_ERROR(r) )
-		r = changes_checkpkgs(filesdb,distribution,changes,directory);
-
-	free(directory);
+		r = changes_checkpkgs(database, distribution, changes);
 
 	if( RET_WAS_ERROR(r) ) {
-		changes_unincludefiles(filesdb,changes);
+		changes_unincludefiles(database, changes);
 		changes_free(changes);
 		return r;
 	}
@@ -960,6 +1137,7 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 	if( tracks != NULL ) {
 		r = trackingdata_summon(tracks,changes->source,changes->sourceversion,&trackingdata);
 		if( RET_WAS_ERROR(r) ) {
+			changes_unincludefiles(database, changes);
 			changes_free(changes);
 			return r;
 		}
@@ -972,18 +1150,20 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 				calc_dirconcat(changes->srcdirectory,basename);
 			free(basename);
 			if( changes->changesfilekey == NULL ) {
+				changes_unincludefiles(database, changes);
 				changes_free(changes);
 				trackingdata_done(&trackingdata);
 				return RET_ERROR_OOM;
 			}
 			if( interrupted() )
-				r = RET_ERROR_INTERUPTED;
+				r = RET_ERROR_INTERRUPTED;
 			else
-			/* always D_COPY, and only delete it afterwards... */
-				r = files_include(filesdb,changesfilename,
+				r = files_preinclude(database,
+					changesfilename,
 					changes->changesfilekey,
-					NULL,NULL,D_COPY);
+					NULL, &changes->includedchangesfile);
 			if( RET_WAS_ERROR(r) ) {
+				changes_unincludefiles(database, changes);
 				changes_free(changes);
 				trackingdata_done(&trackingdata);
 				return r;
@@ -993,12 +1173,13 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 	if( interrupted() ) {
 		if( tracks != NULL )
 			trackingdata_done(&trackingdata);
+		changes_unincludefiles(database, changes);
 		changes_free(changes);
-		return RET_ERROR_INTERUPTED;
+		return RET_ERROR_INTERRUPTED;
 	}
 
 	/* add the source and binary packages in the given distribution */
-	result = changes_includepkgs(dbdir,refs,
+	result = changes_includepkgs(database,
 		distribution,changes,dereferencedfilekeys,
 		(tracks!=NULL)?&trackingdata:NULL, &somethingwasmissed);
 
@@ -1006,10 +1187,13 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 		if( tracks != NULL ) {
 			trackingdata_done(&trackingdata);
 		}
+		changes_unincludefiles(database, changes);
 		changes_free(changes);
 		return result;
 	}
 
+	check_all_files_included(database, changes, somethingwasmissed);
+
 	if( tracks != NULL ) {
 		if( changes->changesfilekey != NULL ) {
 			char *changesfilekey = strdup(changes->changesfilekey);
@@ -1020,10 +1204,14 @@ retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,f
 				return RET_ERROR_OOM;
 			}
 
-			r = trackedpackage_addfilekey(tracks,trackingdata.pkg,ft_CHANGES,changesfilekey,FALSE,refs);
+			r = trackedpackage_addfilekey(tracks, trackingdata.pkg,
+					ft_CHANGES, changesfilekey, false,
+					database);
 			RET_ENDUPDATE(result,r);
+			/* no longer delete when done */
+			changes->includedchangesfile = false;
 		}
-		r = trackingdata_finish(tracks, &trackingdata, refs, dereferencedfilekeys);
+		r = trackingdata_finish(tracks, &trackingdata, database, dereferencedfilekeys);
 		RET_ENDUPDATE(result,r);
 		if( RET_WAS_ERROR(result) ) {
 			changes_free(changes);
