@@ -5,6 +5,9 @@
 #include "error.h"
 #warning "What's hapening here?"
 #endif
+#ifndef REPREPRO_DATABASE_H
+#include "database.h"
+#endif
 #ifndef REPREPRO_DISTRIBUTION_H
 #include "distribution.h"
 #endif
@@ -14,7 +17,7 @@
  * get it from the files or try to guess it.
  * if dereferencedfilekeys is != NULL, add filekeys that lost reference,
  * if tracks != NULL, update/add tracking information there... */
-retvalue changes_add(const char *dbdir,/*@null@*/trackingdb tracks,references refs,filesdb filesdb,/*@null@*/const char *packagetypeonly,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcearchitecture,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority,struct distribution *distribution,const char *changesfilename,int delete,/*@null@*/struct strlist *dereferencedfilekeys);
+retvalue changes_add(struct database *,/*@null@*/trackingdb tracks,/*@null@*/const char *packagetypeonly,/*@null@*/const char *forcecomponent,/*@null@*/const char *forcearchitecture,/*@null@*/const char *forcesection,/*@null@*/const char *forcepriority,struct distribution *distribution,const char *changesfilename,int delete,/*@null@*/struct strlist *dereferencedfilekeys);
 
 #endif
 
