@@ -26,25 +26,24 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
-#include <zlib.h>
 #include "error.h"
 #include "mprintf.h"
 #include "chunks.h"
 #include "sources.h"
-#include "md5sum.h"
 #include "dirs.h"
 #include "names.h"
 #include "release.h"
-#include "copyfile.h"
 #include "tracking.h"
 #include "override.h"
 #include "log.h"
+#include "ignore.h"
 #include "uploaderslist.h"
+#include "configparser.h"
 #include "distribution.h"
 
 extern int verbose;
 
-retvalue distribution_free(struct distribution *distribution) {
+static retvalue distribution_free(struct distribution *distribution) {
 	retvalue result,r;
 
 	if( distribution != NULL ) {
@@ -69,7 +68,9 @@ retvalue distribution_free(struct distribution *distribution) {
 		exportmode_done(&distribution->dsc);
 		exportmode_done(&distribution->deb);
 		exportmode_done(&distribution->udeb);
-		contentsoptions_done(&distribution->contents);
+		strlist_done(&distribution->contents_architectures);
+		strlist_done(&distribution->contents_components);
+		strlist_done(&distribution->contents_ucomponents);
 		override_free(distribution->overrides.deb);
 		override_free(distribution->overrides.udeb);
 		override_free(distribution->overrides.dsc);
@@ -167,234 +168,252 @@ static retvalue createtargets(struct distribution *distribution) {
 	return RET_OK;
 }
 
-struct distribution_filter {int count; const char **dists; bool_t *found;};
+struct read_distribution_data {
+	const char *logdir;
+	struct distribution *distributions;
+};
 
-static inline retvalue isinfilter(const char *codename, const struct distribution_filter filter){
-	int i;
+CFstartparse(distribution) {
+	CFstartparseVAR(distribution, result_p);
+	struct distribution *n;
+	retvalue r;
 
-	/* nothing given means all */
-	if( filter.count <= 0 )
-		return TRUE;
-
-	for( i = 0 ; i < filter.count ; i++ ) {
-		if( strcmp((filter.dists)[i],codename) == 0 ) {
-			if( filter.found[i] ) {
-				fprintf(stderr,"Multiple distribution definitions with the common codename: '%s'!\n",codename);
-				return RET_ERROR;
-
-			}
-			filter.found[i] = TRUE;
-			return RET_OK;
-		}
+	n = calloc(1, sizeof(struct distribution));
+	if( n == NULL )
+		return RET_ERROR_OOM;
+	/* set some default value: */
+	r = exportmode_init(&n->udeb, true, NULL, "Packages");
+	if( RET_WAS_ERROR(r) ) {
+		(void)distribution_free(n);
+		return r;
 	}
-	return RET_NOTHING;
+	r = exportmode_init(&n->deb, true, "Release", "Packages");
+	if( RET_WAS_ERROR(r) ) {
+		(void)distribution_free(n);
+		return r;
+	}
+	r = exportmode_init(&n->dsc, false, "Release", "Sources");
+	if( RET_WAS_ERROR(r) ) {
+		(void)distribution_free(n);
+		return r;
+	}
+	*result_p = n;
+	return RET_OK;
 }
 
-static retvalue distribution_parse_and_filter(const char *confdir,const char *logdir,struct distribution **distribution,const char *chunk,struct distribution_filter filter,bool_t lookedat) {
-	struct distribution *r;
-	retvalue ret;
+static bool notpropersuperset(const struct strlist *allowed, const char *allowedname,
+		const struct strlist *check, const char *checkname,
+		const struct configiterator *iter, const struct distribution *d) {
 	const char *missing;
-	char *option;
-static const char * const allowedfields[] = {
-"Codename", "Suite", "Version", "Origin", "Label", "Description",
-"Architectures", "Components", "Update", "SignWith", "DebOverride",
-"UDebOverride", "DscOverride", "Tracking", "NotAutomatic",
-"UDebComponents", "DebIndices", "DscIndices", "UDebIndices",
-"Pull", "Contents", "ContentsArchitectures",
-"ContentsComponents", "ContentsUComponents",
-"Uploaders", "AlsoAcceptFor", "Log",
-NULL};
 
-	assert( chunk !=NULL && distribution != NULL );
-
-	// TODO: if those are checked anyway, there should be no reason to
-	// research them later...
-	ret = chunk_checkfields(chunk,allowedfields,TRUE);
-	if( RET_WAS_ERROR(ret) )
-		return ret;
-
-	r = calloc(1,sizeof(struct distribution));
-	if( r == NULL )
-		return RET_ERROR_OOM;
-
-#define fieldrequired(name)	if( ret == RET_NOTHING ) { fputs("While parsing distribution definition, required field " name " not found!\n",stderr); ret = RET_ERROR_MISSING; }
-
-	ret = chunk_getvalue(chunk,"Codename",&r->codename);
-	fieldrequired("Codename");
-	if( RET_IS_OK(ret) )
-		ret = propercodename(r->codename);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
+	if( !strlist_subset(allowed, check, &missing) ) {
+		fprintf(stderr,
+"In distribution description of '%s' (line %u to %u in %s):\n"
+"%s contains '%s' not found in %s!\n",
+				d->codename,
+				d->firstline, d->lastline,
+				config_filename(iter),
+				checkname, missing, allowedname);
+		return true;
 	}
+	return false;
+}
 
-	ret = isinfilter(r->codename,filter);
-	if( !RET_IS_OK(ret) ) {
-		(void)distribution_free(r);
-		return ret;
+CFfinishparse(distribution) {
+	CFfinishparseVARS(distribution,n,last_p,mydata);
+	struct distribution *d;
+	retvalue r;
+
+	if( !complete ) {
+		distribution_free(n);
+		return RET_NOTHING;
 	}
+	n->firstline = config_firstline(iter);
+	n->lastline = config_line(iter) - 1;
 
-#define getpossibleemptyfield(key,fieldname) \
-		ret = chunk_getvalue(chunk,key,&r->fieldname); \
-		if(RET_WAS_ERROR(ret)) { \
-			(void)distribution_free(r); \
-			return ret; \
-		} else if( ret == RET_NOTHING) \
-			r->fieldname = NULL;
-#define getpossibleemptywordlist(key,fieldname) \
-		ret = chunk_getuniqwordlist(chunk,key,&r->fieldname); \
-		if(RET_WAS_ERROR(ret)) { \
-			(void)distribution_free(r); \
-			return ret; \
-		} else if( ret == RET_NOTHING) { \
-			r->fieldname.count = 0; \
-			r->fieldname.values = NULL; \
+	/* Do some consitency checks */
+	for( d = mydata->distributions; d != NULL; d = d->next ) {
+		if( strcmp(d->codename, n->codename) == 0 ) {
+			fprintf(stderr,
+"Multiple distributions with the common codename: '%s'!\n"
+"First was in %s line %u to %u, another in lines %u to %u",
+				n->codename, config_filename(iter),
+				d->firstline, d->lastline,
+				n->firstline, n->lastline);
+			distribution_free(n);
+			return RET_ERROR;
 		}
-
-	getpossibleemptyfield("Suite",suite);
-	getpossibleemptyfield("Version",version);
-	getpossibleemptyfield("Origin",origin);
-	getpossibleemptyfield("NotAutomatic",notautomatic);
-	getpossibleemptyfield("Label",label);
-	getpossibleemptyfield("Description",description);
-	ret = chunk_getuniqwordlist(chunk,"Architectures",&r->architectures);
-	fieldrequired("Architectures");
-	if( RET_IS_OK(ret) )
-		ret = properarchitectures(&r->architectures);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-	ret = chunk_getuniqwordlist(chunk,"Components",&r->components);
-	fieldrequired("Components");
-	if( RET_IS_OK(ret) )
-		ret = propercomponents(&r->components);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-	ret = chunk_getwordlist(chunk,"Update",&r->updates);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-	ret = chunk_getwordlist(chunk,"Pull",&r->pulls);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-	getpossibleemptyfield("SignWith",signwith);
-	getpossibleemptyfield("DebOverride",deb_override);
-	getpossibleemptyfield("UDebOverride",udeb_override);
-	getpossibleemptyfield("DscOverride",dsc_override);
-	getpossibleemptyfield("Uploaders",uploaders);
-
-	getpossibleemptywordlist("UDebComponents",udebcomponents);
-
-	// TODO: instead of checking here make sure it can have more
-	// in the rest of the code...
-	if( !strlist_subset(&r->components,&r->udebcomponents,&missing) ) {
-		fprintf(stderr,"In distribution description of '%s':\n"
-				"UDebComponent contains '%s' not found in Components!\n",
-				r->codename,missing);
-		(void)distribution_free(r);
-		return ret;
 	}
 
-	ret = chunk_getvalue(chunk,"UDebIndices",&option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	} else if( ret == RET_NOTHING)
-		option = NULL;
-	ret = exportmode_init(&r->udeb,TRUE,NULL,"Packages",option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
+	if( notpropersuperset(&n->architectures, "Architectures",
+			    &n->contents_architectures, "ContentsArchitectures",
+			    iter, n) ||
+	    notpropersuperset(&n->components, "Components",
+			    &n->contents_components, "ContentsComponents",
+			    iter, n) ||
+	    notpropersuperset(&n->udebcomponents, "UDebComponents",
+			    &n->contents_ucomponents, "ContentsUComponents",
+			    iter, n) ||
+	    // TODO: instead of checking here make sure it can have more
+	    // in the rest of the code...:
+	    notpropersuperset(&n->components, "Components",
+			    &n->udebcomponents, "UDebComponents",
+			    iter, n) ) {
+		(void)distribution_free(n);
+		return RET_ERROR;
 	}
-
-	ret = chunk_getvalue(chunk,"DebIndices",&option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	} else if( ret == RET_NOTHING)
-		option = NULL;
-	ret = exportmode_init(&r->deb,TRUE,"Release","Packages",option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	}
-
-	ret = chunk_getvalue(chunk,"DscIndices",&option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	} else if( ret == RET_NOTHING)
-		option = NULL;
-	ret = exportmode_init(&r->dsc,FALSE,"Release","Sources",option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	}
-
-	ret = chunk_getvalue(chunk,"Tracking",&option);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	} else if( ret == RET_NOTHING)
-		option = NULL;
-	ret = tracking_parse(option,r);
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
-	}
-
-	r->logger = NULL;
-	ret = chunk_getvalue(chunk, "Log", &option);
-	if( RET_IS_OK(ret) ) {
-		struct strlist notify_list;
-		ret = chunk_getextralinelist(chunk, "Log", &notify_list);
-		if( ret == RET_NOTHING )
-			ret = logger_init(confdir, logdir, r->codename,
-					option, NULL, &r->logger);
-		else if( RET_IS_OK(ret) ) {
-			ret = logger_init(confdir, logdir, r->codename,
-					option, &notify_list, &r->logger);
-			strlist_done(&notify_list);
+	/* overwrite creation of contents files based on given lists: */
+	if( n->contents_components_set ) {
+		if ( n->contents_components.count > 0 ) {
+			n->contents.flags.enabled = true;
+			n->contents.flags.nodebs = false;
+		} else {
+			n->contents.flags.nodebs = true;
 		}
-		free(option);
 	}
-	if(RET_WAS_ERROR(ret)) {
-		(void)distribution_free(r);
-		return ret;
+	if( n->contents_ucomponents_set ) {
+		if ( n->contents_ucomponents.count > 0 ) {
+			n->contents.flags.enabled = true;
+			n->contents.flags.udebs = true;
+		} else {
+			n->contents.flags.udebs = false;
+		}
 	}
+	if( n->contents_architectures_set ) {
+		if( n->contents_architectures.count > 0 )
+			n->contents.flags.enabled = true;
+		else
+			n->contents.flags.enabled = false;
+	}
+	/* prepare substructures */
 
-	ret = chunk_getuniqwordlist(chunk, "AlsoAcceptFor", &r->alsoaccept);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
+	r = createtargets(n);
+	if( RET_WAS_ERROR(r) ) {
+		(void)distribution_free(n);
+		return r;
 	}
+	n->status = RET_NOTHING;
+	n->lookedat = false;
+	n->selected = false;
 
-	ret = contentsoptions_parse(r, chunk);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-
-	ret = createtargets(r);
-	if( RET_WAS_ERROR(ret) ) {
-		(void)distribution_free(r);
-		return ret;
-	}
-	r->status = RET_NOTHING;
-	r->lookedat = lookedat;
-
-	*distribution = r;
+	/* put in linked list */
+	if( *last_p == NULL )
+		mydata->distributions = n;
+	else
+		(*last_p)->next = n;
+	*last_p = n;
 	return RET_OK;
+}
 
-#undef fieldrequired
-#undef getpossibleemptyfield
-#undef getpossibleemptywordlist
+CFallSETPROC(distribution, suite)
+CFallSETPROC(distribution, version)
+CFallSETPROC(distribution, origin)
+CFallSETPROC(distribution, notautomatic)
+CFallSETPROC(distribution, label)
+CFallSETPROC(distribution, description)
+CFkeySETPROC(distribution, signwith)
+CFfileSETPROC(distribution, deb_override)
+CFfileSETPROC(distribution, udeb_override)
+CFfileSETPROC(distribution, dsc_override)
+CFfileSETPROC(distribution, uploaders)
+CFuniqstrlistSETPROC(distribution, udebcomponents)
+CFuniqstrlistSETPROC(distribution, alsoaccept)
+CFstrlistSETPROC(distribution, updates)
+CFstrlistSETPROC(distribution, pulls)
+CFuniqstrlistSETPROCset(distribution, contents_architectures)
+CFuniqstrlistSETPROCset(distribution, contents_components)
+CFuniqstrlistSETPROCset(distribution, contents_ucomponents)
+CFexportmodeSETPROC(distribution, udeb)
+CFexportmodeSETPROC(distribution, deb)
+CFexportmodeSETPROC(distribution, dsc)
+CFcheckuniqstrlistSETPROC(distribution, components, checkforcomponent)
+CFcheckuniqstrlistSETPROC(distribution, architectures, checkforarchitecture)
+CFcheckvalueSETPROC(distribution, codename, checkforcodename)
+
+CFUSETPROC(distribution, Contents) {
+	CFSETPROCVAR(distribution, d);
+	return contentsoptions_parse(d, iter);
+}
+CFuSETPROC(distribution, logger) {
+	CFSETPROCVARS(distribution, d, mydata);
+	return logger_init(confdir, mydata->logdir, iter, &d->logger);
+}
+CFUSETPROC(distribution, Tracking) {
+	CFSETPROCVAR(distribution, d);
+	return tracking_parse(d, iter);
+}
+
+static const struct configfield distributionconfigfields[] = {
+	CF("AlsoAcceptFor",	distribution,	alsoaccept),
+	CFr("Architectures",	distribution,	architectures),
+	CFr("Codename",		distribution,	codename),
+	CFr("Components",	distribution,	components),
+	CF("ContentsArchitectures", distribution, contents_architectures),
+	CF("ContentsComponents", distribution,	contents_components),
+	CF("Contents",		distribution,	Contents),
+	CF("ContentsUComponents", distribution,	contents_ucomponents),
+	CF("DebIndices",	distribution,	deb),
+	CF("DebOverride",	distribution,	deb_override),
+	CF("Description",	distribution,	description),
+	CF("DscIndices",	distribution,	dsc),
+	CF("DscOverride",	distribution,	dsc_override),
+	CF("Label",		distribution,	label),
+	CF("Log",		distribution,	logger),
+	CF("NotAutomatic",	distribution,	notautomatic),
+	CF("Origin",		distribution,	origin),
+	CF("Pull",		distribution,	pulls),
+	CF("SignWith",		distribution,	signwith),
+	CF("Suite",		distribution,	suite),
+	CF("Tracking",		distribution,	Tracking),
+	CF("UDebComponents",	distribution,	udebcomponents),
+	CF("UDebIndices",	distribution,	udeb),
+	CF("UDebOverride",	distribution,	udeb_override),
+	CF("Update",		distribution,	updates),
+	CF("Uploaders",		distribution,	uploaders),
+	CF("Version",		distribution,	version)
+};
+
+/* read specification of all distributions */
+retvalue distribution_readall(const char *confdir, const char *logdir, struct distribution **distributions) {
+	struct read_distribution_data mydata;
+	retvalue result;
+
+	mydata.logdir = logdir;
+	mydata.distributions = NULL;
+
+	// TODO: readd some way to tell about -b or --confdir here?
+	/*
+	result = regularfileexists(fn);
+	if( RET_WAS_ERROR(result) ) {
+		fprintf(stderr,"Could not find '%s'!\n"
+"(Have you forgotten to specify a basedir by -b?\n"
+"To only set the conf/ dir use --confdir)\n",fn);
+		free(mydata.filter.found);
+		free(fn);
+		return RET_ERROR_MISSING;
+	}
+	*/
+
+	result = configfile_parse(confdir, "distributions",
+			IGNORABLE(unknownfield),
+			startparsedistribution, finishparsedistribution,
+			distributionconfigfields,
+			ARRAYCOUNT(distributionconfigfields),
+			&mydata);
+	if( result == RET_ERROR_UNKNOWNFIELD )
+		fprintf(stderr, "To ignore unknown fields use --ignore=unknownfield\n");
+	if( RET_WAS_ERROR(result) ) {
+		distribution_freelist(mydata.distributions);
+		return result;
+	}
+	if( mydata.distributions == NULL ) {
+		fprintf(stderr, "No distribution definitions found in %s/distributions!\n",
+				confdir);
+		distribution_freelist(mydata.distributions);
+		return RET_ERROR_MISSING;
+	}
+	*distributions = mydata.distributions;
+	return RET_OK;
 }
 
 /* call <action> for each part of <distribution>. */
@@ -418,6 +437,156 @@ retvalue distribution_foreach_part(struct distribution *distribution,const char
 	return result;
 }
 
+/* call <action> for each part of <distribution>, within initpackagesdb/closepackagesdb */
+retvalue distribution_foreach_rwopenedpart(struct distribution *distribution,struct database *database,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
+	retvalue result,r;
+	struct target *t;
+
+	result = RET_NOTHING;
+	for( t = distribution->targets ; t != NULL ; t = t->next ) {
+		if( component != NULL && strcmp(component,t->component) != 0 )
+			continue;
+		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+			continue;
+		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+			continue;
+		r = target_initpackagesdb(t, database, READWRITE);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(r) )
+			return result;
+		r = action(data, t, distribution);
+		RET_UPDATE(result, r);
+		// TODO: how to seperate this in those affecting distribution
+		// and those that do not?
+		RET_UPDATE(distribution->status, r);
+		r = target_closepackagesdb(t);
+		RET_UPDATE(distribution->status, r);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(result) )
+			return result;
+	}
+	return result;
+}
+
+/* call <action> for each part of <distribution>, within initpackagesdb/closepackagesdb */
+retvalue distribution_foreach_roopenedpart(struct distribution *distribution,struct database *database,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
+	retvalue result,r;
+	struct target *t;
+
+	result = RET_NOTHING;
+	for( t = distribution->targets ; t != NULL ; t = t->next ) {
+		if( component != NULL && strcmp(component,t->component) != 0 )
+			continue;
+		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+			continue;
+		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+			continue;
+		r = target_initpackagesdb(t, database, READONLY);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(r) )
+			return result;
+		r = action(data, t, distribution);
+		RET_UPDATE(result, r);
+		r = target_closepackagesdb(t);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(result) )
+			return result;
+	}
+	return result;
+}
+
+/* call <action> for each package */
+retvalue distribution_foreach_package(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action action, each_target_action target_action, void *data) {
+	retvalue result,r;
+	struct target *t;
+	struct cursor *cursor;
+	const char *package, *control;
+
+	result = RET_NOTHING;
+	for( t = distribution->targets ; t != NULL ; t = t->next ) {
+		if( component != NULL && strcmp(component,t->component) != 0 )
+			continue;
+		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+			continue;
+		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+			continue;
+		if( target_action != NULL ) {
+			r = target_action(database, distribution, t, data);
+			if( RET_WAS_ERROR(r) )
+				return result;
+			if( r == RET_NOTHING )
+				continue;
+		}
+		r = target_initpackagesdb(t, database, READONLY);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(r) )
+			return result;
+		r = table_newglobalcursor(t->packages, &cursor);
+		assert( r != RET_NOTHING );
+		if( RET_WAS_ERROR(r) ) {
+			(void)target_closepackagesdb(t);
+			return r;
+		}
+		while( cursor_nexttemp(t->packages, cursor,
+					&package, &control) ) {
+			r = action(database, distribution, t,
+					package, control, data);
+			RET_UPDATE(result, r);
+			if( RET_WAS_ERROR(r) )
+				break;
+		}
+		r = cursor_close(t->packages, cursor);
+		RET_ENDUPDATE(result, r);
+		r = target_closepackagesdb(t);
+		RET_ENDUPDATE(result, r);
+		if( RET_WAS_ERROR(result) )
+			return result;
+	}
+	return result;
+}
+
+retvalue distribution_foreach_package_c(struct distribution *distribution, struct database *database, const struct strlist *components, const char *architecture, const char *packagetype, each_package_action action, void *data) {
+	retvalue result,r;
+	struct target *t;
+	struct cursor *cursor;
+	const char *package, *control;
+
+	result = RET_NOTHING;
+	for( t = distribution->targets ; t != NULL ; t = t->next ) {
+		if( components != NULL && !strlist_in(components, t->component) )
+			continue;
+		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+			continue;
+		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+			continue;
+		r = target_initpackagesdb(t, database, READONLY);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(r) )
+			return result;
+		r = table_newglobalcursor(t->packages, &cursor);
+		assert( r != RET_NOTHING );
+		if( RET_WAS_ERROR(r) ) {
+			(void)target_closepackagesdb(t);
+			return r;
+		}
+		while( cursor_nexttemp(t->packages, cursor,
+					&package, &control) ) {
+			r = action(database, distribution, t,
+					package, control, data);
+			RET_UPDATE(result, r);
+			if( RET_WAS_ERROR(r) )
+				break;
+		}
+		r = cursor_close(t->packages, cursor);
+		RET_ENDUPDATE(result, r);
+		r = target_closepackagesdb(t);
+		RET_ENDUPDATE(result, r);
+		if( RET_WAS_ERROR(result) )
+			return result;
+	}
+	return result;
+}
+
 struct target *distribution_gettarget(const struct distribution *distribution,const char *component,const char *architecture,const char *packagetype) {
 	struct target *t = distribution->targets;
 
@@ -444,127 +613,69 @@ struct target *distribution_getpart(const struct distribution *distribution,cons
 	return t;
 }
 
-struct distmatch_mydata {
-	const char *confdir;
-	const char *logdir;
-	struct distribution_filter filter;
-	struct distribution *distributions;
-	bool_t lookedat;
-};
+/* mark all distributions matching one of the first argc argv */
+retvalue distribution_match(struct distribution *alldistributions, int argc, const char *argv[], bool lookedat) {
+	struct distribution *d;
+	bool *found;
+	int i;
 
-static retvalue adddistribution(void *d,const char *chunk) {
-	struct distmatch_mydata *mydata = d;
-	retvalue result;
-	struct distribution *distribution IFSTUPIDCC(=NULL);
+	assert( alldistributions != NULL );
 
-	result = distribution_parse_and_filter(mydata->confdir, mydata->logdir,
-			&distribution, chunk, mydata->filter, mydata->lookedat);
-	if( RET_IS_OK(result) ){
-		struct distribution *d;
-		for( d=mydata->distributions; d != NULL; d=d->next ) {
-			if( strcmp(d->codename,distribution->codename) == 0 ) {
-				fprintf(stderr,"Multiple distributions with the common codename: '%s'!\n",d->codename);
-				result = RET_ERROR;
-			}
+	if( argc <= 0 ) {
+		for( d = alldistributions ; d != NULL ; d = d->next ) {
+			d->selected = true;
+			d->lookedat = lookedat;
 		}
-		distribution->next = mydata->distributions;
-		mydata->distributions = distribution;
+		return RET_OK;
 	}
-
-	return result;
-}
-
-/* get all dists from <conf> fitting in the filter given in <argc,argv> */
-retvalue distribution_getmatched(const char *confdir,const char *logdir,int argc,const char *argv[],bool_t lookedat,struct distribution **distributions) {
-	retvalue result;
-	char *fn;
-	struct distmatch_mydata mydata;
-
-	mydata.confdir = confdir;
-	mydata.logdir = logdir;
-	mydata.filter.count = argc;
-	mydata.filter.dists = (const char**)argv;
-	mydata.filter.found = calloc(argc,sizeof(bool_t));
-	if( mydata.filter.found == NULL )
-		return RET_ERROR_OOM;
-	mydata.distributions = NULL;
-	mydata.lookedat = lookedat;
-
-	fn = calc_dirconcat(confdir,"distributions");
-	if( fn == NULL )
+	found = calloc(argc, sizeof(bool));
+	if( found == NULL )
 		return RET_ERROR_OOM;
 
-	result = regularfileexists(fn);
-	if( RET_WAS_ERROR(result) ) {
-		fprintf(stderr,"Could not find '%s'!\n"
-"(Have you forgotten to specify a basedir by -b?\n"
-"To only set the conf/ dir use --confdir)\n",fn);
-		free(mydata.filter.found);
-		free(fn);
-		return RET_ERROR_MISSING;
-	}
-
-	result = chunk_foreach(fn,adddistribution,&mydata,FALSE);
-
-	if( !RET_WAS_ERROR(result) ) {
-		int i;
+	for( d = alldistributions ; d != NULL ; d = d->next ) {
 		for( i = 0 ; i < argc ; i++ ) {
-			if( !mydata.filter.found[i] ) {
-				fprintf(stderr,"No distribution definition of '%s' found in '%s'!\n",mydata.filter.dists[i],fn);
-				result = RET_ERROR_MISSING;
+			if( strcmp(argv[i], d->codename) == 0 ) {
+				assert( !found[i] );
+				found[i] = true;
+				d->selected = true;
+				if( lookedat )
+					d->lookedat = lookedat;
 			}
 		}
 	}
-	free(fn);
-	if( result == RET_NOTHING ) {
-		/* if argc==0 and no definition in conf/distributions */
-		fprintf(stderr,"No distribution definitons found!\n");
-		result = RET_ERROR_MISSING;
-	}
-
-	if( RET_IS_OK(result) ) {
-		*distributions = mydata.distributions;
-	} else  {
-		while( mydata.distributions != NULL ) {
-			struct distribution *next = mydata.distributions->next;
-			(void)distribution_free(mydata.distributions);
-			mydata.distributions = next;
+	for( i = 0 ; i < argc ; i++ ) {
+		if( !found[i] ) {
+			fprintf(stderr, "No distribution definition of '%s' found in distributions'!\n", argv[i]);
+			free(found);
+			return RET_ERROR_MISSING;
 		}
 	}
-	free(mydata.filter.found);
-	return result;
+	free(found);
+	return RET_OK;
 }
 
-retvalue distribution_get(const char *confdir,const char *logdir,const char *name,bool_t lookedat,struct distribution **distribution) {
-	retvalue result;
+retvalue distribution_get(struct distribution *alldistributions, const char *name, bool lookedat, struct distribution **distribution) {
 	struct distribution *d;
 
-	/* This is a bit overkill, as it does not stop when it finds the
-	 * definition of the distribution. But this way we can warn
-	 * about emtpy lines in the definition (as this would split
-	 * it in two definitions, the second one no valid one).
-	 */
-	result = distribution_getmatched(confdir,logdir,1,&name,lookedat,&d);
-
-	if( RET_WAS_ERROR(result) )
-		return result;
-
-	if( result == RET_NOTHING ) {
-		fprintf(stderr,"Cannot find definition of distribution '%s' in %s/distributions!\n",name,confdir);
+	d = alldistributions;
+	while( d != NULL && strcmp(name, d->codename) != 0 )
+		d = d->next;
+	if( d == NULL ) {
+		fprintf(stderr,"Cannot find definition of distribution '%s'!\n", name);
 		return RET_ERROR_MISSING;
 	}
-	assert( d != NULL && d->next == NULL );
-
+	d->selected = true;
+	if( lookedat )
+		d->lookedat = true;
 	*distribution = d;
 	return RET_OK;
 }
 
-retvalue distribution_snapshot(struct distribution *distribution,
-		const char *confdir, const char *dbdir, const char *distdir,
-		references refs, const char *name) {
+retvalue distribution_snapshot(struct distribution *distribution, const char *distdir, struct database *database, const char *name) {
 	struct target *target;
 	retvalue result,r;
 	struct release *release;
+	char *id;
 
 	assert( distribution != NULL );
 
@@ -578,42 +689,47 @@ retvalue distribution_snapshot(struct distribution *distribution,
 		RET_ENDUPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
-		r = target_export(target,confdir,dbdir,FALSE,TRUE,release);
+		r = target_export(target, database, false, true, release);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
 		if( target->exportmode->release != NULL ) {
-			r = release_directorydescription(release,distribution,target,target->exportmode->release,FALSE);
+			r = release_directorydescription(release, distribution, target, target->exportmode->release, false);
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) )
 				break;
 		}
 	}
+	if( !RET_WAS_ERROR(result) ) {
+		result = release_prepare(release, distribution, false);
+		assert( result != RET_NOTHING );
+	}
 	if( RET_WAS_ERROR(result) ) {
 		release_free(release);
 		return result;
 	}
-	result = release_write(release,distribution,FALSE);
+	result = release_finish(release, distribution);
 	if( RET_WAS_ERROR(result) )
 		return r;
-	/* add references so that the pool files belonging to it are not deleted */
-	for( target=distribution->targets; target != NULL ; target = target->next ) {
-		r = target_addsnapshotreference(target,dbdir,refs,name);
-		RET_UPDATE(result,r);
-	}
+	id = mprintf("s=%s=%s", distribution->codename, name);
+	if( id == NULL )
+		return RET_ERROR_OOM;
+	r = distribution_foreach_package(distribution, database,
+			NULL, NULL, NULL,
+			package_referenceforsnapshot, NULL, id);
+	free(id);
+	RET_UPDATE(result,r);
 	return result;
 }
 
-static retvalue export(struct distribution *distribution,
-		const char *confdir, const char *dbdir, const char *distdir,
-		filesdb files, bool_t onlyneeded) {
+static retvalue export(struct distribution *distribution, const char *distdir, struct database *database, bool onlyneeded) {
 	struct target *target;
 	retvalue result,r;
 	struct release *release;
 
 	assert( distribution != NULL );
 
-	r = release_init(dbdir,distdir,distribution->codename,&release);
+	r = release_init(&release, database, distdir, distribution->codename);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -623,7 +739,7 @@ static retvalue export(struct distribution *distribution,
 		RET_ENDUPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
-		r = target_export(target,confdir,dbdir,onlyneeded,FALSE,release);
+		r = target_export(target, database, onlyneeded, false, release);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
 			break;
@@ -634,15 +750,35 @@ static retvalue export(struct distribution *distribution,
 				break;
 		}
 	}
-	if( !RET_WAS_ERROR(result) && distribution->contents.rate > 0 ) {
-		r = contents_generate(files, distribution, dbdir, release, onlyneeded);
+	if( !RET_WAS_ERROR(result) && distribution->contents.flags.enabled ) {
+		r = contents_generate(database, distribution,
+				release, onlyneeded);
 	}
-	if( RET_WAS_ERROR(result) )
+	if( !RET_WAS_ERROR(result) ) {
+		result = release_prepare(release,distribution,onlyneeded);
+		if( result == RET_NOTHING ) {
+			release_free(release);
+			return result;
+		}
+	}
+	if( RET_WAS_ERROR(result) ) {
+		bool workleft = false;
 		release_free(release);
-	else {
-		retvalue r;
-
-		r = release_write(release,distribution,onlyneeded);
+		fprintf(stderr, "ERROR: Could not finish exporting '%s'!\n", distribution->codename);
+		for( target=distribution->targets; target != NULL ; target = target->next ) {
+			workleft |= target->saved_wasmodified;
+		}
+		if( workleft ) {
+			(void)fputs(
+"This means that from outside your repository will still look like before (and\n"
+"should still work if this old state worked), but the things you changed in this\n"
+"call will not be visible until you call export directly (via reprepro export)\n"
+"(things will also get visible when something else changes the same file and\n"
+"thus creates a new export of that file, but even changes to other parts of the\n"
+"same distribution will not!\n",	stderr);
+		}
+	} else {
+		r = release_finish(release,distribution);
 		RET_UPDATE(result,r);
 	}
 	if( RET_IS_OK(result) )
@@ -650,8 +786,8 @@ static retvalue export(struct distribution *distribution,
 	return result;
 }
 
-retvalue distribution_fullexport(struct distribution *distribution,const char *confdir,const char *dbdir,const char *distdir, filesdb files) {
-	return export(distribution,confdir,dbdir,distdir,files,FALSE);
+retvalue distribution_fullexport(struct distribution *distribution, const char *distdir, struct database *database) {
+	return export(distribution, distdir, database, false);
 }
 
 retvalue distribution_freelist(struct distribution *distributions) {
@@ -667,12 +803,9 @@ retvalue distribution_freelist(struct distribution *distributions) {
 	return result;
 }
 
-retvalue distribution_exportandfreelist(enum exportwhen when,
-		struct distribution *distributions,
-		const char *confdir,const char *dbdir, const char *distdir,
-		filesdb files) {
+retvalue distribution_exportlist(enum exportwhen when, struct distribution *distributions, const char *distdir, struct database *database) {
 	retvalue result,r;
-	bool_t todo = FALSE;
+	bool todo = false;
 	struct distribution *d;
 
 	if( when == EXPORT_NEVER ) {
@@ -682,10 +815,12 @@ retvalue distribution_exportandfreelist(enum exportwhen when,
 	}
 
 	for( d=distributions; d != NULL; d = d->next ) {
+		if( !d->selected )
+			continue;
 		if( d->lookedat && (RET_IS_OK(d->status) ||
 			( d->status == RET_NOTHING && when != EXPORT_CHANGED) ||
 			when == EXPORT_FORCE)) {
-			todo = TRUE;
+			todo = true;
 		}
 	}
 
@@ -693,10 +828,9 @@ retvalue distribution_exportandfreelist(enum exportwhen when,
 		printf("Exporting indices...\n");
 
 	result = RET_NOTHING;
-	while( distributions != NULL ) {
-		d = distributions;
-		distributions = d->next;
-
+	for( d=distributions; d != NULL; d = d->next ) {
+		if( !d->selected )
+			continue;
 		if( !d->lookedat ) {
 			if( verbose >= 30 )
 				printf(
@@ -725,7 +859,7 @@ retvalue distribution_exportandfreelist(enum exportwhen when,
 "Please report this and how you got this message as bugreport. Thanks.\n"
 "Doing a export despite --export=changed....\n",
 						d->codename);
-					r = export(d,confdir,dbdir,distdir,files,TRUE);
+					r = export(d, distdir, database, true);
 					RET_UPDATE(result,r);
 					break;
 				}
@@ -735,17 +869,14 @@ retvalue distribution_exportandfreelist(enum exportwhen when,
 					( d->status == RET_NOTHING &&
 					  when != EXPORT_CHANGED) ||
 					when == EXPORT_FORCE);
-			r = export(d,confdir,dbdir,distdir,files, TRUE);
+			r = export(d, distdir, database, true);
 			RET_UPDATE(result,r);
 		}
-
-		r = distribution_free(d);
-		RET_ENDUPDATE(result,r);
 	}
 	return result;
 }
 
-retvalue distribution_export(enum exportwhen when, struct distribution *distribution,const char *confdir,const char *dbdir,const char *distdir, filesdb files) {
+retvalue distribution_export(enum exportwhen when, struct distribution *distribution, const char *distdir, struct database *database) {
 	if( when == EXPORT_NEVER ) {
 		if( verbose >= 10 )
 			fprintf(stderr,
@@ -778,8 +909,8 @@ retvalue distribution_export(enum exportwhen when, struct distribution *distribu
 "Please report this and how you got this message as bugreport. Thanks.\n"
 "Doing a export despite --export=changed....\n",
 						distribution->codename);
-				return export(distribution,
-						confdir,dbdir,distdir,files,TRUE);
+				return export(distribution, distdir,
+						database, true);
 				break;
 			}
 		}
@@ -788,7 +919,7 @@ retvalue distribution_export(enum exportwhen when, struct distribution *distribu
 	}
 	if( verbose >= 0 )
 		printf("Exporting indices...\n");
-	return export(distribution,confdir,dbdir,distdir,files, TRUE);
+	return export(distribution, distdir, database, true);
 }
 
 /* get a pointer to the apropiate part of the linked list */
@@ -827,25 +958,25 @@ struct distribution *distribution_find(struct distribution *distributions, const
 	return NULL;
 }
 
-retvalue distribution_loadalloverrides(struct distribution *distribution, const char *overridedir) {
+retvalue distribution_loadalloverrides(struct distribution *distribution, const char *confdir, const char *overridedir) {
 	retvalue r;
 
 	if( distribution->overrides.deb == NULL ) {
-		r = override_read(overridedir,distribution->deb_override,&distribution->overrides.deb);
+		r = override_read(confdir, overridedir, distribution->deb_override, &distribution->overrides.deb);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.deb = NULL;
 			return r;
 		}
 	}
 	if( distribution->overrides.udeb == NULL ) {
-		r = override_read(overridedir,distribution->udeb_override,&distribution->overrides.udeb);
+		r = override_read(confdir, overridedir, distribution->udeb_override, &distribution->overrides.udeb);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.udeb = NULL;
 			return r;
 		}
 	}
 	if( distribution->overrides.dsc == NULL ) {
-		r = override_read(overridedir,distribution->dsc_override,&distribution->overrides.dsc);
+		r = override_read(confdir, overridedir, distribution->dsc_override, &distribution->overrides.dsc);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.dsc = NULL;
 			return r;
@@ -886,6 +1017,57 @@ retvalue distribution_prepareforwriting(struct distribution *distribution) {
 		if( RET_WAS_ERROR(r) )
 			return r;
 	}
-	distribution->lookedat = TRUE;
+	distribution->lookedat = true;
 	return RET_OK;
 }
+
+/* delete every package decider returns RET_OK for */
+retvalue distribution_remove_packages(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action decider, struct strlist *dereferenced, struct trackingdata *trackingdata, void *data) {
+	retvalue result,r;
+	struct target *t;
+	struct cursor *cursor;
+	const char *package, *control;
+
+	result = RET_NOTHING;
+	for( t = distribution->targets ; t != NULL ; t = t->next ) {
+		if( component != NULL && strcmp(component,t->component) != 0 )
+			continue;
+		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
+			continue;
+		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+			continue;
+		r = target_initpackagesdb(t, database, READWRITE);
+		RET_UPDATE(result, r);
+		if( RET_WAS_ERROR(r) )
+			return result;
+		r = table_newglobalcursor(t->packages, &cursor);
+		assert( r != RET_NOTHING );
+		if( RET_WAS_ERROR(r) ) {
+			(void)target_closepackagesdb(t);
+			return r;
+		}
+		while( cursor_nexttemp(t->packages, cursor,
+					&package, &control) ) {
+			r = decider(database, distribution, t,
+					package, control, data);
+			RET_UPDATE(result, r);
+			if( RET_WAS_ERROR(r) )
+				break;
+			if( RET_IS_OK(r) ) {
+				r = target_removepackage_by_cursor(t,
+					distribution->logger, database, cursor,
+					package, control, NULL, dereferenced,
+					trackingdata);
+				RET_UPDATE(result, r);
+				RET_UPDATE(distribution->status, r);
+			}
+		}
+		r = cursor_close(t->packages, cursor);
+		RET_ENDUPDATE(result, r);
+		r = target_closepackagesdb(t);
+		RET_ENDUPDATE(result, r);
+		if( RET_WAS_ERROR(result) )
+			return result;
+	}
+	return result;
+}
