@@ -5,22 +5,24 @@
 #include "globals.h"
 #endif
 
-bool_t interrupted(void);
+bool interrupted(void);
 
 /* retvalue is simply an int.
  * just named to show it follows the given semantics */
-typedef int retvalue;
-
-#define RET_OK (retvalue)1
-#define RET_NOTHING (retvalue)0
-#define RET_ERROR (retvalue)-1
-#define RET_ERROR_WRONG_MD5 (retvalue)-2
-#define RET_ERROR_OOM (retvalue)-3
-#define RET_ERROR_EXIST (retvalue)-4
-#define RET_ERROR_GPGME (retvalue)-5
-#define RET_ERROR_BADSIG (retvalue)-6
-#define RET_ERROR_MISSING (retvalue)-7
-#define RET_ERROR_INTERUPTED (retvalue)-8
+/*@numabstract@*/ enum retvalue_enum {
+	RET_ERROR_INTERRUPTED = -9,
+	RET_ERROR_UNKNOWNFIELD = -8,
+	RET_ERROR_MISSING = -7,
+	RET_ERROR_BADSIG = -6,
+	RET_ERROR_GPGME = -5,
+	RET_ERROR_EXIST = -4,
+	RET_ERROR_OOM = -3,
+	RET_ERROR_WRONG_MD5 = -2,
+	RET_ERROR = -1,
+	RET_NOTHING = 0,
+	RET_OK  = 1
+};
+typedef enum retvalue_enum retvalue;
 
 #define RET_IS_OK(r) ((r) == RET_OK)
 #define RET_WAS_NO_ERROR(r) ((r) >= (retvalue)0)
