@@ -17,9 +17,11 @@ struct exportmode {
 	char *hook;
 };
 
-retvalue exportmode_init(/*@out@*/struct exportmode *mode,bool_t uncompressed,/*@null@*/const char *release,const char *indexfile,/*@null@*//*@only@*/char *options);
+retvalue exportmode_init(/*@out@*/struct exportmode *mode, bool uncompressed, /*@null@*/const char *release, const char *indexfile);
+struct configiterator;
+retvalue exportmode_set(struct exportmode *mode, const char *confdir, struct configiterator *iter);
 void exportmode_done(struct exportmode *mode);
 
-retvalue export_target(const char *confdir,const char *relativedir,packagesdb packages,const struct exportmode *exportmode,struct release *release, bool_t onlymissing, bool_t snapshot);
+retvalue export_target(const char *relativedir, struct table *packages, const struct exportmode *exportmode, struct release *release, bool onlymissing, bool snapshot);
 
 #endif
