@@ -25,10 +25,10 @@
 #include <getopt.h>
 #include <string.h>
 #include <malloc.h>
-#include <zlib.h>
 #include "error.h"
-#include "md5sum.h"
 #include "chunks.h"
+#include "filecntl.h"
+#include "readtextfile.h"
 #include "debfile.h"
 
 #ifdef HAVE_LIBARCHIVE
@@ -40,7 +40,7 @@
 // * found.
 // **********************************************************************
 
-static retvalue try_extractcontrol(char **control,const char *debfile, bool_t brokentar) {
+static retvalue try_extractcontrol(char **control, const char *debfile, bool brokentar) {
 	int pipe1[2];
 	int pipe2[2];
 	int ret;
@@ -113,34 +113,46 @@ static retvalue try_extractcontrol(char **control,const char *debfile, bool_t br
 
 	}
 
-	close(pipe1[0]);close(pipe1[1]);
-	close(pipe2[1]);
+	close(pipe1[0]); close(pipe1[1]);
+	markcloseonexec(pipe2[0]); close(pipe2[1]);
+
+	controlchunk = NULL;
 
 	/* read data: */
 	if( RET_IS_OK(result) ) {
-		gzFile f;
-		//TODO: making a gzdopen here is kinda stupid...
-		f = gzdopen(pipe2[0],"r");
-		if( f == NULL ) {
-			fprintf(stderr,"Error opening gzip-stream for pipe!\n");
-			RET_UPDATE(result,RET_ERROR);
-		} else {
-			r = chunk_read(f,&controlchunk);
-			if( r == RET_NOTHING ) {
-				controlchunk = NULL;
-				fprintf(stderr,"Got no control information from .deb!\n");
-				/* only report error now if we haven't try everything yet */
-				if( brokentar )
-					r = RET_ERROR_MISSING;
-			}
-			RET_UPDATE(result,r);
-			gzclose(f);
-		}
-	}
+		size_t len; char *afterchanges;
 
-	/* avoid being a memory leak */
-	if( !(RET_IS_OK(result)) )
-		controlchunk = NULL;
+		r = readtextfilefd(pipe2[0],
+				brokentar?
+				"output from ar p <debfile> control.tar.gz | tar -XOzf - control":
+				"output from ar p <debfile> control.tar.gz | tar -XOzf - ./control",
+				&controlchunk, NULL);
+		if( RET_IS_OK(r) ) {
+			len = chunk_extract(controlchunk, controlchunk, &afterchanges);
+			if( len == 0 )
+				r = RET_NOTHING;
+			if( *afterchanges != '\0' ) {
+				fprintf(stderr,
+"Unexpected emtpy line in control information within '%s'\n"
+"(obtained via 'ar p %s control.tar.gz | tar -XOzf - %scontrol')\n",
+						debfile, debfile,
+						brokentar?"":"./");
+				free(controlchunk);
+				controlchunk = NULL;
+				r = RET_ERROR;
+			}
+		}
+		if( r == RET_NOTHING ) {
+			free(controlchunk);
+			controlchunk = NULL;
+			fprintf(stderr,"Got no control information from .deb!\n");
+			/* only report error now if we haven't try everything yet */
+			if( brokentar )
+				r = RET_ERROR_MISSING;
+		}
+		RET_UPDATE(result,r);
+
+	}
 
 	while( ar != -1 || tar != -1 ) {
 		pid=wait(&status);
@@ -174,7 +186,7 @@ static retvalue try_extractcontrol(char **control,const char *debfile, bool_t br
 				}
 			} else {
 				// WTH?
-				fprintf(stderr,"Who is %d, and why does this bother me?\n",pid);
+				fprintf(stderr,"Who is %d, and why does this bother me?\n",(int)pid);
 			}
 		}
 
@@ -193,12 +205,12 @@ static retvalue try_extractcontrol(char **control,const char *debfile, bool_t br
 retvalue extractcontrol(char **control,const char *debfile) {
 	retvalue r;
 
-	r = try_extractcontrol(control, debfile, FALSE);
+	r = try_extractcontrol(control, debfile, false);
 	if( r != RET_NOTHING )
 		return r;
 	/* perhaps the control.tar.gz is packaged by hand wrongly,
 	 * try again: */
-	r = try_extractcontrol(control, debfile, TRUE);
+	r = try_extractcontrol(control, debfile, true);
 	if( RET_IS_OK(r) ) {
 		fprintf(stderr, "WARNING: '%s' contains a broken/unusual control.tar.gz.\n"
 				"reprepro was able to work around this but other tools or versions might not.\n", debfile);
@@ -207,23 +219,31 @@ retvalue extractcontrol(char **control,const char *debfile) {
 	return r;
 }
 
-retvalue getfilelist(/*@out@*/char **filelist, const char *debfile) {
+retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char *debfile) {
+	fprintf(stderr, "Extracing of filelist currently not implemented without libarchive.\n");
+	return RET_ERROR;
+#if 0
 	int pipe1[2];
 	int pipe2[2];
 	int ret;
 	pid_t ar,tar,pid;
 	int status;
-	char *list = NULL;
-	size_t listsize = 0;
-	size_t len=0, last = 0;
+	struct filelistcompressor c;
+	size_t last = 0;
 	retvalue result;
 
+#error this still needs to be reimplemented...
+	result = filelistcompressor_setup(&c);
+	if( RET_WAS_ERROR(result) )
+		return result;
+
 	result = RET_OK;
 
 	ret = pipe(pipe1);
 	if( ret < 0 ) {
 		int e = errno;
 		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
+		filelistcompressor_cancel(&c);
 		return RET_ERRNO(e);
 	}
 
@@ -232,6 +252,7 @@ retvalue getfilelist(/*@out@*/char **filelist, const char *debfile) {
 		int e = errno;
 		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
 		close(pipe1[0]);close(pipe1[1]);
+		filelistcompressor_cancel(&c);
 		return RET_ERRNO(e);
 	}
 
@@ -242,6 +263,7 @@ retvalue getfilelist(/*@out@*/char **filelist, const char *debfile) {
 		result = RET_ERRNO(e);
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
+		filelistcompressor_cancel(&c);
 		return result;
 	}
 
@@ -348,7 +370,7 @@ retvalue getfilelist(/*@out@*/char **filelist, const char *debfile) {
 				len -= ignore;
 			}
 		}
-	} while( TRUE );
+	} while( true );
 	if( len != last ) {
 		fprintf(stderr, "WARNING: unterminated output from tar over pipe while extracting filelist of %s\n",debfile);
 		list[len] = '\0';
@@ -395,11 +417,11 @@ retvalue getfilelist(/*@out@*/char **filelist, const char *debfile) {
 				fprintf(stderr,"Who is %d, and why does this bother me?\n",pid);
 			}
 		}
-
 	}
 	if( RET_IS_OK(result) )
-		*filelist = list;
+		return filelistcompressor_finish(&c, filelist);
 	else
-		free(list);
+		filelistcompressor_cancel(&c);
 	return result;
+#endif
 }
