@@ -26,6 +26,9 @@
 #include <unistd.h>
 #include <string.h>
 #include <strings.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <signal.h>
 #include <stdio.h>
 #include <time.h>
 #include <malloc.h>
@@ -33,18 +36,20 @@
 #include "strlist.h"
 #include "dirs.h"
 #include "target.h"
+#include "configparser.h"
 #include "log.h"
+#include "filecntl.h"
 
 extern int verbose;
 
-/*@null@*/ static struct logfile {
-	struct logfile *next;
+/*@null@*/ static /*@refcounted@*/ struct logfile {
+	/*@null@*/struct logfile *next;
 	char *filename;
-	size_t refcount;
+	/*@refs@*/size_t refcount;
 	int fd;
 } *logfile_root = NULL;
 
-static retvalue logfile_reference(const char *logdir,const char *filename,/*@out@*/struct logfile **logfile) {
+static retvalue logfile_reference(const char *logdir,/*@only@*/char *filename,/*@out@*/struct logfile **logfile) {
 	struct logfile *l;
 
 	assert( logdir != NULL && filename != NULL );
@@ -53,16 +58,21 @@ static retvalue logfile_reference(const char *logdir,const char *filename,/*@out
 		if( strcmp(l->filename, filename) == 0 ) {
 			l->refcount++;
 			*logfile = l;
+			free(filename);
 			return RET_OK;
 		}
 	}
 	l = malloc(sizeof(struct logfile));
-	if( l == NULL )
+	if( l == NULL ) {
+		free(filename);
 		return RET_ERROR_OOM;
+	}
 	if( filename[0] == '/' )
-		l->filename = strdup(filename);
-	else
+		l->filename = filename;
+	else {
 		l->filename = calc_dirconcat(logdir,filename);
+		free(filename);
+	}
 	if( l->filename == NULL ) {
 		free(l);
 		return RET_ERROR_OOM;
@@ -193,192 +203,201 @@ struct notificator {
 	/*@null@*/char *packagetype;
 	/*@null@*/char *component;
 	/*@null@*/char *architecture;
-	bool_t withcontrol, changesacceptrule;
+	bool withcontrol, changesacceptrule;
 };
 
-static void notificator_done(struct notificator *n) {
+static void notificator_done(/*@special@*/struct notificator *n) /*@releases n->scriptname,n->packagename,n->component,n->architecture@*/{
 	free(n->scriptname);
 	free(n->packagetype);
 	free(n->component);
 	free(n->architecture);
 }
 
-static retvalue notificator_parse(struct notificator *n, const char *confdir, const char *codename, const char *line) {
-	const char *p,*q,*s;
+static retvalue notificator_parse(struct notificator *n, const char *confdir, struct configiterator *iter) {
+	retvalue r;
+	int c;
 
-	p = line; q = line;
-	while( *q != '\0' ) {
-		while( *p == ' ' || *p == '\t' )
-			p++;
-		q = p;
-		while( *q != '\0' && *q != ' ' && *q != '\t' )
-			q++;
-		if( *p == '-' ) {
+	memset(n, 0, sizeof(struct notificator));
+	while( (c = config_nextnonspaceinline(iter)) != EOF ) {
+		if( c == '-' ) {
+			char *word, *s, *detachedargument = NULL;
+			const char *argument;
 			char **value_p = NULL;
-			p++;
-			s = p;
-			while( s < q && *s != '=' )
+			bool error = false;
+
+			r = config_completeword(iter, c, &word);
+			assert( r != RET_NOTHING );
+			if( RET_WAS_ERROR(r) )
+				return r;
+
+			s = word + 1;
+			while( *s != '\0' && *s != '=' )
 				s++;
-			switch( s-p ) {
-				case 1:
-					if( *p == 'A' )
+			if( *s == '=' ) {
+				argument = s+1;
+				s[0] = '\0';
+			} else
+				argument = NULL;
+			switch( s-word ) {
+				case 2:
+					if( word[1] == 'A' )
 						value_p = &n->architecture;
-					else if( *p == 'C' )
+					else if( word[1] == 'C' )
 						value_p = &n->component;
-					else if( *p == 'T' )
+					else if( word[1] == 'T' )
 						value_p = &n->packagetype;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '-%c' (in '%s')\n",
-							codename,
-							*p,
-							line);
-						return RET_ERROR;
-					}
+					else
+						error = true;
 					break;
-				case 5:
-					if( memcmp(p, "-type", 5) == 0 )
+				case 6:
+					if( strcmp(word, "--type") == 0 )
 						value_p = &n->packagetype;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '%6s' (in '%s')\n",
-							codename,
-							p-1,
-							line);
-						return RET_ERROR;
-					}
+					else
+						error = true;
 					break;
-				case 8:
-					if( memcmp(p, "-changes", 8) == 0 )
-						n->changesacceptrule = TRUE;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '%8s' (in '%s')\n",
-							codename,
-							p-1,
-							line);
-						return RET_ERROR;
-					}
+				case 9:
+					if( strcmp(word, "--changes") == 0 )
+						n->changesacceptrule = true;
+					else
+						error = true;
 					break;
-				case 10:
-					if( memcmp(p, "-component", 10) == 0 )
+				case 11:
+					if( strcmp(word, "--component") == 0 )
 						value_p = &n->component;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '%11s' (in '%s')\n",
-							codename,
-							p-1,
-							line);
-						return RET_ERROR;
-					}
-					break;
-				case 12:
-					if( memcmp(p, "-withcontrol", 12) == 0 )
-						n->withcontrol = TRUE;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '%12s' (in '%s')\n",
-							codename,
-							p-1,
-							line);
-						return RET_ERROR;
-					}
+					else
+						error = true;
 					break;
 				case 13:
-					if( memcmp(p, "-architecture", 13) == 0 )
+					if( strcmp(word, "--withcontrol") == 0 )
+						n->withcontrol = true;
+					else
+						error = true;
+					break;
+				case 14:
+					if( strcmp(word, "--architecture") == 0 )
 						value_p = &n->architecture;
-					else {
-						fprintf(stderr,
-"Unknown option in notifiers of '%s': '%14s' (in '%s')\n",
-							codename,
-							p-1,
-							line);
-						return RET_ERROR;
-					}
+					else
+						error = true;
 					break;
 				default:
-					fprintf(stderr,
-"Unknown option in notifiers of '%s': '%.*s' (in '%s')\n",
-							codename,
-							(int)(1+s-p), p-1,
-							line);
-					return RET_ERROR;
+					error = true;
+					break;
+			}
+			if( error ) {
+				fprintf(stderr,
+"Unknown Log notifier option in %s, line %u, column %u: '%s'\n",
+					config_filename(iter),
+					config_markerline(iter),
+					config_markercolumn(iter), word);
+				free(word);
+				return RET_ERROR;
 			}
 			if( value_p == NULL ) {
-				if( s != q ) {
+				if( argument != NULL ) {
 					fprintf(stderr,
-"Unexpected '=' in notifiers of '%s' after '%.*s' (in '%s')\n",
-							codename,
-							(int)(1+s-p), p-1,
-							line);
+"Log notifier option has = but may not, in %s, line %u, column %u: '%s'\n",
+						config_filename(iter),
+						config_markerline(iter),
+						config_markercolumn(iter),
+						word);
+					free(word);
 					return RET_ERROR;
 				}
-				p = q;
+				free(word);
 				continue;
 			}
 			/* option expecting string value: */
-			if( *s != '=' ) {
-				fprintf(stderr,
-"Missing '=' in notifiers of '%s' after '%.*s' (in '%s')\n",
-						codename, (int)(1+s-p), p-1, line);
-				return RET_ERROR;
-			}
 			if( *value_p != NULL ) {
 				fprintf(stderr,
-"Double notifier option '%.*s' (in '%s' from '%s')\n",
-						(int)(1+s-p), p-1, line, codename);
+"Repeated notifier option %s in %s, line %u, column %u!\n", word,
+					config_filename(iter),
+					config_markerline(iter), config_markercolumn(iter));
+				free(word);
 				return RET_ERROR;
 			}
-			*value_p = strndup(s+1, q-s-1);
+			if( argument == NULL ) {
+				r = config_getwordinline(iter, &detachedargument);
+				if( RET_WAS_ERROR(r) )
+					return r;
+				if( r == RET_NOTHING ) {
+					fprintf(stderr,
+"Log notifier option %s misses an argument in %s, line %u, column %u\n",
+						word, config_filename(iter),
+						config_line(iter),
+						config_column(iter));
+					free(word);
+					return RET_ERROR;
+				}
+			}
+			if( argument == NULL )
+				*value_p = detachedargument;
+			else
+				*value_p = strdup(argument);
+			free(word);
 			if( *value_p == NULL )
 				return RET_ERROR_OOM;
-			p = q;
 		} else {
+			char *script;
+
 			if( n->changesacceptrule && n->architecture != NULL ) {
 				fprintf(stderr,
-"--changes and --architecture cannot be combined! (notifier in '%s')\n",
-						codename);
+"Error: --changes and --architecture cannot be combined! (line %u in '%s')\n",
+					config_markerline(iter), config_filename(iter));
 				return RET_ERROR;
 			}
 			if( n->changesacceptrule && n->component != NULL ) {
 				fprintf(stderr,
-"--changes and --component cannot be combined! (notifier in '%s')\n",
-						codename);
+"Error: --changes and --component cannot be combined! (line %u in %s)\n",
+					config_markerline(iter), config_filename(iter));
 				return RET_ERROR;
 			}
 			if( n->changesacceptrule && n->packagetype != NULL ) {
 				fprintf(stderr,
-"--changes and --type cannot be combined! (notifier in '%s')\n",
-						codename);
+"Error: --changes and --type cannot be combined! (line %u in %s)\n",
+					config_markerline(iter), config_filename(iter));
 				return RET_ERROR;
 			}
-			if( *q != '\0' ) {
+
+			r = config_completeword(iter, c, &script);
+			assert( r != RET_NOTHING );
+			if( RET_WAS_ERROR(r) )
+				return r;
+
+			c = config_nextnonspaceinline(iter);
+			if( c != EOF ) {
 				fprintf(stderr,
-"Unexpected data at end of notifier for '%s': '%s'\n",
-						codename, q);
+"Error parsing config file %s, line %u, column %u:\n"
+"Unexpected data at end of notifier after scriptname '%s'\n",
+					config_filename(iter),
+					config_line(iter), config_column(iter),
+					script);
+				free(script);
 				return RET_ERROR;
 			}
-			if( *p == '/' )
-				n->scriptname = strdup(p);
-			else
-				n->scriptname = calc_dirconcat(confdir, p);
+			if( script[0] == '/' )
+				n->scriptname = script;
+			else {
+				n->scriptname = calc_dirconcat(confdir, script);
+				free(script);
+			}
 			if( n->scriptname == NULL )
 				return RET_ERROR_OOM;
 			return RET_OK;
 		}
 	}
 	fprintf(stderr,
-"Missing notification script to call in '%s' of '%s'\n",
-		line, codename);
+"Error parsing config file %s, line %u, column %u:\n"
+"Unexpected end of line: name of notifier script missing!\n",
+		config_filename(iter), config_line(iter), config_column(iter));
 	return RET_ERROR;
 }
 
 /*@null@*/ static struct notification_process {
-	struct notification_process *next;
+	/*@null@*/struct notification_process *next;
 	char **arguments;
 	/* data to send to the process */
 	size_t datalen, datasent;
-	char *data;
+	/*@null@*/char *data;
 	/* process */
 	pid_t child;
 	int fd;
@@ -449,7 +468,7 @@ static int catchchildren(void) {
 	return returned;
 }
 
-static void feedchildren(bool_t wait) {
+static void feedchildren(bool wait) {
 	struct notification_process *p;
 	fd_set w;
 	int ret;
@@ -485,7 +504,7 @@ static void feedchildren(bool_t wait) {
 "Error '%s' while sending data to '%s', sending SIGABRT to it!\n",
 						strerror(e),
 						p->arguments[0]);
-				kill(p->child, SIGABRT);
+				(void)kill(p->child, SIGABRT);
 			}
 			p->datasent += sent;
 			if( p->datasent >= p->datalen ) {
@@ -532,34 +551,23 @@ static retvalue startchild(void) {
 	}
 	child = fork();
 	if( child == 0 ) {
-		int maxopen;
-
 		if( p->datalen > 0 ) {
 			dup2(filedes[0], 0);
 			if( filedes[0] != 0)
 				(void)close(filedes[0]);
 		}
 		/* Try to close all open fd but 0,1,2 */
-		maxopen = sysconf(_SC_OPEN_MAX);
-		if( maxopen > 0 ) {
-			int fd;
-			for( fd = 3 ; fd < maxopen ; fd++ )
-				(void)close(fd);
-		} else {
-			/* close at least the ones definitly causing problems*/
-			const struct notification_process *q;
-			for( q = processes; q != NULL ; q = q->next ) {
-				if( q != p && q->fd >= 0 )
-					(void)close(q->fd);
-			}
-		}
+		(void)close(filedes[1]);
+		closefrom(3);
 		(void)execv(p->arguments[0], p->arguments);
 		fprintf(stderr, "Error executing '%s': %s\n", p->arguments[0],
 				strerror(errno));
 		_exit(255);
 	}
-	if( p->datalen > 0 )
-		close(filedes[0]);
+	if( p->datalen > 0 ) {
+		(void)close(filedes[0]);
+		markcloseonexec(p->fd);
+	}
 	if( child < 0 ) {
 		int e = errno;
 		fprintf(stderr, "Error forking: %d=%s!\n", e, strerror(e));
@@ -582,7 +590,7 @@ static retvalue startchild(void) {
 				return RET_ERROR;
 			}
 			if( (polldata.revents & POLLHUP) != 0 ) {
-				close(p->fd);
+				(void)close(p->fd);
 				p->fd = -1;
 				return RET_OK;
 			}
@@ -599,12 +607,11 @@ static retvalue startchild(void) {
 "Error '%s' while sending data to '%s', sending SIGABRT to it!\n",
 							strerror(e),
 							p->arguments[0]);
-					kill(p->child, SIGABRT);
+					(void)kill(p->child, SIGABRT);
 					return RET_ERRNO(e);
 				}
 				p->datasent += written;
 				if( p->datasent >= p->datalen ) {
-					int ret;
 					free(p->data);
 					p->data = NULL;
 					ret = close(p->fd);
@@ -630,7 +637,7 @@ static retvalue notificator_enqueuechanges(struct notificator *n,const char *cod
 	struct notification_process *p;
 
 	catchchildren();
-	feedchildren(FALSE);
+	feedchildren(false);
 	if( !n->changesacceptrule )
 		return RET_NOTHING;
 	count = 6; /* script "accepted" codename name version safename */
@@ -681,19 +688,22 @@ static retvalue notificator_enqueuechanges(struct notificator *n,const char *cod
 	p->datasent = 0;
 	p->data = NULL;
 	// TODO: implement --withcontrol
+	// until that changeschunk is not yet needed:
+	changeschunk = changeschunk;
+
 	if( runningchildren() < 1 )
 		startchild();
 	return RET_OK;
 }
 
-static retvalue notificator_enqueue(struct notificator *n,struct target *target,const char *name,/*@null@*/const char *version,/*@null@*/const char *oldversion,/*@null@*/const char *control,/*@null@*/const char *oldcontrol,/*@null@*/const struct strlist *filekeys,/*@null@*/const struct strlist *oldfilekeys,bool_t renotification) {
-	size_t count,i,j;
+static retvalue notificator_enqueue(struct notificator *n, struct target *target, const char *name, /*@null@*/const char *version, /*@null@*/const char *oldversion, /*@null@*/const char *control, /*@null@*/const char *oldcontrol, /*@null@*/const struct strlist *filekeys, /*@null@*/const struct strlist *oldfilekeys, bool renotification) {
+	size_t count, i;
 	char **arguments;
 	const char *action = NULL;
 	struct notification_process *p;
 
 	catchchildren();
-	feedchildren(FALSE);
+	feedchildren(false);
 	if( n->changesacceptrule )
 		return RET_NOTHING;
 	// some day, some atom handling for those would be nice
@@ -753,12 +763,14 @@ static retvalue notificator_enqueue(struct notificator *n,struct target *target,
 	if( oldversion != NULL )
 		arguments[i++] = strdup(oldversion);
 	if( version != NULL ) {
+		int j;
 		arguments[i++] = strdup("--");
 		if( filekeys != NULL )
 			for( j = 0 ; j < filekeys->count ; j++ )
 				arguments[i++] = strdup(filekeys->values[j]);
 	}
 	if( oldversion != NULL ) {
+		int j;
 		arguments[i++] = strdup("--");
 		if( oldfilekeys != NULL )
 			for( j = 0 ; j < oldfilekeys->count ; j++ )
@@ -766,13 +778,15 @@ static retvalue notificator_enqueue(struct notificator *n,struct target *target,
 	}
 	assert( i == count );
 	arguments[i] = NULL;
-	for( i = 0 ; i < count ; i++ )
+	for( i = 0 ; i < count ; i++ ) {
+		size_t j;
 		if( arguments[i] == NULL ) {
 			for( j = 0 ; j < count ; j++ )
 				free(arguments[j]);
 			free(arguments);
 			return RET_ERROR_OOM;
 		}
+	}
 	if( processes == NULL ) {
 		p = malloc(sizeof(struct notification_process));
 		processes = p;
@@ -784,6 +798,7 @@ static retvalue notificator_enqueue(struct notificator *n,struct target *target,
 		p = p->next;
 	}
 	if( p == NULL ) {
+		size_t j;
 		for( j = 0 ; j < count ; j++ )
 			free(arguments[j]);
 		free(arguments);
@@ -797,6 +812,8 @@ static retvalue notificator_enqueue(struct notificator *n,struct target *target,
 	p->datasent = 0;
 	p->data = NULL;
 	// TODO: implement --withcontrol
+	// until that control is not yet needed:
+	control = control; oldcontrol = oldcontrol;
 	if( runningchildren() < 1 )
 		startchild();
 	return RET_OK;
@@ -807,7 +824,7 @@ void logger_wait(void) {
 		catchchildren();
 		if( interrupted() )
 			break;
-		feedchildren(TRUE);
+		feedchildren(true);
 		// TODO: add option to start multiple at the same time
 		if( runningchildren() < 1 )
 			startchild();
@@ -822,7 +839,7 @@ void logger_warn_waiting(void) {
 	struct notification_process *p;
 
 	if( processes != NULL ) {
-		fputs(
+		(void)fputs(
 "WARNING: some notificator hooks were not run!\n"
 "(most likely due to receiving an interruption request)\n"
 "You will either have to run them by hand or run rerunnotifiers if\n"
@@ -833,20 +850,20 @@ void logger_warn_waiting(void) {
 			if( c == NULL )
 				continue;
 			while( *c != NULL ) {
-				fputc('"', stderr);
-				fputs(*c, stderr);
-				fputc('"', stderr);
+				(void)fputc('"', stderr);
+				(void)fputs(*c, stderr);
+				(void)fputc('"', stderr);
 				c++;
 				if( *c != NULL )
-					fputc(' ', stderr);
+					(void)fputc(' ', stderr);
 			}
-			fputc('\n', stderr);
+			(void)fputc('\n', stderr);
 		}
 	}
 }
 
 struct logger {
-	/*@dependent@*/struct logfile *logfile;
+	/*@dependent@*//*@null@*/struct logfile *logfile;
 	size_t notificator_count;
 	struct notificator *notificators;
 };
@@ -858,7 +875,7 @@ void logger_free(struct logger *logger) {
 	if( logger->logfile != NULL )
 		logfile_dereference(logger->logfile);
 	if( logger->notificators != NULL ) {
-		int i;
+		size_t i;
 
 		for( i = 0 ; i < logger->notificator_count ; i++ )
 			notificator_done(&logger->notificators[i]);
@@ -868,56 +885,68 @@ void logger_free(struct logger *logger) {
 	free(logger);
 }
 
-retvalue logger_init(const char *confdir,const char *logdir,const char *codename,const char *option,const struct strlist *notificators,struct logger **logger_p) {
+retvalue logger_init(const char *confdir,const char *logdir,struct configiterator *iter,struct logger **logger_p) {
 	struct logger *n;
 	retvalue r;
+	char *logfilename;
+	bool havenotificators;
 
-	if( (option == NULL || *option == '\0')
-		&& (notificators == NULL || notificators->count == 0) ) {
+	r = config_getfileinline(iter, &logfilename);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	if( r == RET_NOTHING )
+		logfilename = NULL;
+	if( config_nextnonspaceinline(iter) != EOF ) {
+		fprintf(stderr, "Error parsing %s, line %u, column %u:\n"
+				"Unexpected second filename for logfile.\n",
+				config_filename(iter), config_line(iter),
+				config_column(iter));
+		free(logfilename);
+		return RET_ERROR;
+	}
+	config_overline(iter);
+	havenotificators = config_nextline(iter);
+	if( !havenotificators && logfilename == NULL ) {
 		*logger_p = NULL;
 		return RET_NOTHING;
 	}
+
 	n = malloc(sizeof(struct logger));
-	if( n == NULL )
+	if( n == NULL ) {
+		free(logfilename);
 		return RET_ERROR_OOM;
-	if( option != NULL && *option != '\0' ) {
-		r = logfile_reference(logdir, option, &n->logfile);
+	}
+	if( logfilename != NULL ) {
+		assert( *logfilename != '\0');
+		r = logfile_reference(logdir, logfilename, &n->logfile);
 		if( RET_WAS_ERROR(r) ) {
 			free(n);
 			return r;
 		}
 	} else
 		n->logfile = NULL;
-	if( notificators != NULL && notificators->count > 0 ) {
-		int i;
-		n->notificator_count = notificators->count;
-		n->notificators = calloc(n->notificator_count, sizeof(struct notificator));
-		if( n->notificators == NULL ) {
-			if( n->logfile != NULL )
-				logfile_dereference(n->logfile);
-			free(n);
+
+	n->notificators = NULL;
+	n->notificator_count = 0;
+
+	while( havenotificators ) {
+		struct notificator *newnot;
+		newnot = realloc(n->notificators,
+				(n->notificator_count+1)* sizeof(struct notificator));
+		if( newnot == NULL ) {
+			logger_free(n);
 			return RET_ERROR_OOM;
 		}
-		for( i = 0 ; i < notificators->count ; i++ ) {
-			r = notificator_parse(&n->notificators[i], confdir, codename,
-					notificators->values[i]);
-			if( RET_WAS_ERROR(r) ) {
-				/* a bit ugly: also free the just failed item here */
-				while( i >= 0 ) {
-					notificator_done(&n->notificators[i]);
-					i--;
-				}
-
-				if( n->logfile != NULL )
-					logfile_dereference(n->logfile);
-				free(n->notificators);
-				free(n);
-				return r;
-			}
+		n->notificators = newnot;
+		r = notificator_parse(&n->notificators[n->notificator_count++],
+				confdir, iter);
+		if( RET_WAS_ERROR(r) ) {
+			/* a bit ugly: also free the just failed item here */
+			logger_free(n);
+			return r;
 		}
-	} else {
-		n->notificators = NULL;
-		n->notificator_count = 0;
+		// TODO assert eol here...
+		havenotificators = config_nextline(iter);
 	}
 	*logger_p = n;
 	return RET_OK;
@@ -935,12 +964,12 @@ retvalue logger_prepare(struct logger *logger) {
 		r = RET_OK;
 	return r;
 }
-bool_t logger_isprepared(/*@null@*/const struct logger *logger) {
+bool logger_isprepared(/*@null@*/const struct logger *logger) {
 	if( logger == NULL )
-		return TRUE;
+		return true;
 	if( logger->logfile != NULL && logger->logfile->fd < 0 )
-		return FALSE;
-	return TRUE;
+		return false;
+	return true;
 }
 
 void logger_log(struct logger *log,struct target *target,const char *name,const char *version,const char *oldversion,const char *control,const char *oldcontrol,const struct strlist *filekeys, const struct strlist *oldfilekeys) {
@@ -962,7 +991,7 @@ void logger_log(struct logger *log,struct target *target,const char *name,const
 		notificator_enqueue(&log->notificators[i], target,
 				name, version, oldversion,
 				control, oldcontrol,
-				filekeys, oldfilekeys, FALSE);
+				filekeys, oldfilekeys, false);
 	}
 }
 
@@ -981,8 +1010,8 @@ void logger_logchanges(struct logger *log,const char *codename,const char *name,
 	}
 }
 
-bool_t logger_rerun_needs_target(const struct logger *logger,const struct target *target) {
-	int i;
+bool logger_rerun_needs_target(const struct logger *logger, const struct target *target) {
+	size_t i;
 	struct notificator *n;
 
 	for( i = 0 ; i < logger->notificator_count ; i++ ) {
@@ -1000,9 +1029,9 @@ bool_t logger_rerun_needs_target(const struct logger *logger,const struct target
 				strcmp(n->packagetype,target->packagetype) != 0 ) {
 			continue;
 		}
-		return TRUE;
+		return true;
 	}
-	return FALSE;
+	return false;
 }
 
 retvalue logger_reruninfo(struct logger *logger,struct target *target,const char *name,const char *version,const char *control,/*@null@*/const struct strlist *filekeys) {
@@ -1019,7 +1048,7 @@ retvalue logger_reruninfo(struct logger *logger,struct target *target,const char
 		r = notificator_enqueue(&logger->notificators[i], target,
 				name, version, NULL,
 				control, NULL,
-				filekeys, NULL, TRUE);
+				filekeys, NULL, true);
 		RET_UPDATE(result,r);
 	}
 	return result;
