@@ -32,9 +32,8 @@ struct MD5Context {
 	UWORD32 in[16];
 };
 
-void MD5Init(struct MD5Context *context);
+void MD5Init(/*@out@*/struct MD5Context *context);
 void MD5Update(struct MD5Context *context, md5byte const *buf, unsigned int len);
-void MD5Final(unsigned char digest[16], struct MD5Context *context);
-//void MD5Transform(UWORD32 buf[4], UWORD32 const in[16]);
+void MD5Final(/*@out@*/unsigned char digest[16], struct MD5Context *context);
 
 #endif /* !MD5_H */
