@@ -20,9 +20,6 @@ char *calc_filekey(const char *component,const char *sourcename,const char *file
 char *calc_fullfilename(const char *mirrordir,const char *filekey);
 char *calc_fullsrcfilename(const char *mirrordir,const char *directory,const char *filename);
 char *calc_identifier(const char *codename,const char *component,const char *architecture,const char *packagetype);
-char *calc_concatmd5andsize(const char *md5sum,const char *size);
-char *names_concatmd5sumandsize(const char *md5start,const char *md5end,const char *sizestart,const char *sizeend);
-retvalue calc_extractsize(const char *checksum, /*@out@*/off_t *size);
 char *calc_trackreferee(const char *codename,const char *sourcename,const char *sourceversion);
 
 char *calc_downloadedlistfile(const char *listdir,const char *codename,const char *origin,const char *component,const char *architecture,const char *packagetype);
@@ -30,12 +27,14 @@ char *calc_downloadedlistpattern(const char *codename);
 
 /* Create a strlist consisting out of calc_dirconcat'ed entries of the old */
 retvalue calc_dirconcats(const char *directory, const struct strlist *basefilenames,/*@out@*/struct strlist *files);
+retvalue calc_inplacedirconcats(const char *directory, struct strlist *);
 
 /* split a "<md5> <size> <filename>" into md5sum and filename */
-retvalue calc_parsefileline(const char *fileline,/*@out@*/char **filename,/*@out@*//*@null@*/char **md5sum);
+struct checksums;
+retvalue calc_parsefileline(const char *fileline, /*@out@*/char **filename, /*@out@*//*@null@*/struct checksums **);
 
 /* move over a version number, if epochsuppresed is true, colons may happen even without epoch there */
-void names_overversion(const char **version,bool_t epochsuppressed);
+void names_overversion(const char **version, bool epochsuppressed);
 
 /* check for forbidden characters */
 retvalue propersourcename(const char *string);
@@ -44,19 +43,8 @@ retvalue properfilename(const char *string);
 retvalue properfilenames(const struct strlist *names);
 retvalue properpackagename(const char *string);
 retvalue properversion(const char *string);
-retvalue propercodename(const char *codename);
-retvalue properarchitectures(const struct strlist *architectures);
-retvalue propercomponents(const struct strlist *components);
 
-/* create two stringlists out of one,
- * from[i]=into[i]=list[i] if list[i] contains no '>'
- * from[i] '>' into[i] = list[i] when list[i] contains a '>'
- */
-retvalue splitlist(/*@out@*/struct strlist *from,
-				/*@out@*/struct strlist *into,
-				const struct strlist *list);
-
-static inline bool_t endswith(const char *name, const char *suffix) {
+static inline bool endswith(const char *name, const char *suffix) {
 	size_t ln,ls;
 	ln = strlen(name);
 	ls = strlen(suffix);
