@@ -17,6 +17,7 @@
 
 #include <errno.h>
 #include <assert.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <sys/types.h>
@@ -32,14 +33,13 @@
 #ifdef HAVE_LIBBZ2
 #include <bzlib.h>
 #endif
-#include <db.h>
+#define CHECKSUMS_CONTEXT visible
 #include "error.h"
 #include "mprintf.h"
 #include "strlist.h"
+#include "filecntl.h"
 #include "chunks.h"
-#include "md5.h"
-#include "md5sum.h"
-#include "copyfile.h"
+#include "checksums.h"
 #include "dirs.h"
 #include "names.h"
 #include "signature.h"
@@ -48,25 +48,30 @@
 
 extern int verbose;
 
+#define INPUT_BUFFER_SIZE 1024
+#define GZBUFSIZE 40960
+#define BZBUFSIZE 40960
 
 struct release {
 	/* The base-directory of the distribution we are exporting */
 	char *dirofdist;
 	/* anything new yet added */
-	bool_t new;
+	bool new;
 	/* snapshot */
-	bool_t snapshot;
+	bool snapshot;
 	/* the files yet for the list */
 	struct release_entry {
 		struct release_entry *next;
 		char *relativefilename;
-		char *md5sum;
+		struct checksums *checksums;
 		/* both == NULL if not new, only final==NULL means delete */
 		char *fullfinalfilename;
 		char *fulltemporaryfilename;
 	} *files;
+	/* the Release file in preperation (only valid between _prepare and _finish) */
+	struct signedfile *signedfile;
 	/* the cache database for old files */
-	DB *cachedb;
+	struct table *cachedb;
 };
 
 void release_free(struct release *release) {
@@ -76,13 +81,15 @@ void release_free(struct release *release) {
 	while( (e = release->files) != NULL ) {
 		release->files = e->next;
 		free(e->relativefilename);
-		free(e->md5sum);
+		checksums_free(e->checksums);
 		free(e->fullfinalfilename);
 		free(e->fulltemporaryfilename);
 		free(e);
 	}
+	if( release->signedfile != NULL )
+		signedfile_free(release->signedfile);
 	if( release->cachedb != NULL ) {
-		release->cachedb->close(release->cachedb,0);
+		table_close(release->cachedb);
 	}
 	free(release);
 }
@@ -92,7 +99,7 @@ const char *release_dirofdist(struct release *release) {
 }
 
 static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relativefilename,
-		/*@only@*/ char *md5sum,
+		/*@only@*/ struct checksums *checksums,
 		/*@only@*/ /*@null@*/ char *fullfinalfilename,
 		/*@only@*/ /*@null@*/ char *fulltemporaryfilename ) {
 	struct release_entry *n,*p;
@@ -101,7 +108,7 @@ static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relati
 		return RET_ERROR_OOM;
 	n->next = NULL;
 	n->relativefilename = relativefilename;
-	n->md5sum = md5sum;
+	n->checksums = checksums;
 	n->fullfinalfilename = fullfinalfilename;
 	n->fulltemporaryfilename = fulltemporaryfilename;
 	if( release->files == NULL )
@@ -115,10 +122,9 @@ static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relati
 	return RET_OK;
 }
 
-retvalue release_init(const char *dbdir, const char *distdir, const char *codename, struct release **release) {
+retvalue release_init(struct release **release, struct database *database, const char *distdir, const char *codename) {
 	struct release *n;
-	int dbret;
-	char *filename;
+	retvalue r;
 
 	n = calloc(1,sizeof(struct release));
 	n->dirofdist = calc_dirconcat(distdir,codename);
@@ -126,30 +132,14 @@ retvalue release_init(const char *dbdir, const char *distdir, const char *codena
 		free(n);
 		return RET_ERROR_OOM;
 	}
-	dbret = db_create(&n->cachedb,NULL,0);
-	if( dbret < 0 ) {
-		fprintf(stderr, "db_create: %s\n", db_strerror(dbret));
+	r = database_openreleasecache(database, codename, &n->cachedb);
+	assert( r != RET_NOTHING );
+	if( RET_WAS_ERROR(r) ) {
 		n->cachedb = NULL;
 		free(n->dirofdist);
 		free(n);
-		return RET_DBERR(dbret);
+		return r;
 	}
-	filename = calc_dirconcat(dbdir,"release.cache.db");
-	if( filename == NULL ) {
-		(void)n->cachedb->close(n->cachedb,0);
-		return RET_ERROR_OOM;
-	}
-	dbret = DB_OPEN(n->cachedb,filename,codename, DB_HASH, DB_CREATE);
-	if( dbret < 0 ) {
-		n->cachedb->err(n->cachedb, dbret, "%s(%s)",
-				filename,codename);
-		(void)n->cachedb->close(n->cachedb,0);
-		free(filename);
-		free(n->dirofdist);
-		free(n);
-		return RET_DBERR(dbret);
-	}
-	free(filename);
 	*release = n;
 	return RET_OK;
 }
@@ -164,7 +154,7 @@ retvalue release_initsnapshot(const char *distdir, const char *codename, const c
 		return RET_ERROR_OOM;
 	}
 	n->cachedb = NULL;
-	n->snapshot = TRUE;
+	n->snapshot = true;
 	*release = n;
 	return RET_OK;
 }
@@ -183,7 +173,8 @@ retvalue release_adddel(struct release *release,/*@only@*/char *reltmpfile) {
 
 retvalue release_addnew(struct release *release,/*@only@*/char *reltmpfile,/*@only@*/char *relfilename) {
 	retvalue r;
-	char *filename,*finalfilename,*md5sum;
+	char *filename, *finalfilename;
+	struct checksums *checksums;
 
 	filename = calc_dirconcat(release->dirofdist,reltmpfile);
 	if( filename == NULL ) {
@@ -192,7 +183,7 @@ retvalue release_addnew(struct release *release,/*@only@*/char *reltmpfile,/*@on
 		return RET_ERROR_OOM;
 	}
 	free(reltmpfile);
-	r = md5sum_read(filename,&md5sum);
+	r = checksums_read(filename, &checksums);
 	if( !RET_IS_OK(r) ) {
 		free(relfilename);
 		free(filename);
@@ -202,169 +193,190 @@ retvalue release_addnew(struct release *release,/*@only@*/char *reltmpfile,/*@on
 	if( finalfilename == NULL ) {
 		free(relfilename);
 		free(filename);
-		free(md5sum);
+		checksums_free(checksums);
 		return RET_ERROR_OOM;
 	}
-	release->new = TRUE;
-	return newreleaseentry(release,relfilename,md5sum,finalfilename,filename);
+	release->new = true;
+	return newreleaseentry(release, relfilename,
+			checksums, finalfilename, filename);
 }
 
 retvalue release_addold(struct release *release,/*@only@*/char *relfilename) {
 	retvalue r;
-	char *filename,*md5sum;
+	char *filename;
+	struct checksums *checksums;
 
 	filename = calc_dirconcat(release->dirofdist,relfilename);
 	if( filename == NULL ) {
 		free(filename);
 		return RET_ERROR_OOM;
 	}
-	r = md5sum_read(filename,&md5sum);
+	r = checksums_read(filename, &checksums);
 	free(filename);
 	if( !RET_IS_OK(r) ) {
 		free(relfilename);
 		return r;
 	}
-	return newreleaseentry(release,relfilename,md5sum,NULL,NULL);
+	return newreleaseentry(release, relfilename, checksums, NULL, NULL);
+}
+
+static char *calc_compressedname(const char *name, enum indexcompression ic) {
+	switch( ic ) {
+		case ic_uncompressed:
+			return strdup(name);
+		case ic_gzip:
+			return calc_addsuffix(name, "gz");
+#ifdef HAVE_LIBBZ2
+		case ic_bzip2:
+			return calc_addsuffix(name, "bz2");
+#endif
+		default:
+			assert( "Huh?" == NULL );
+			return NULL;
+	}
 }
 
-static retvalue getcachevalue(struct release *,const char *, char **);
 static retvalue release_usecached(struct release *release,
 				const char *relfilename,
 				compressionset compressions) {
 	retvalue result,r;
-	char *filename,*md5sum;
-	char *gzrelfilename,*gzmd5sum;
-#ifdef HAVE_LIBBZ2
-	char *bzrelfilename,*bzmd5sum;
-#endif
+	enum indexcompression ic;
+	char *filename[ic_count];
+	struct checksums *checksums[ic_count];
 
-	/* first look if the there are actual files, in case
-	 * the cache still lists them but they got lost */
-	filename = calc_dirconcat(release->dirofdist,relfilename);
-	if( filename == NULL ) {
-		return RET_ERROR_OOM;
-	}
-	if( (compressions & IC_FLAG(ic_uncompressed)) != 0 ) {
-		if( !isregularfile(filename) ) {
-			free(filename);
-			return RET_NOTHING;
+	memset(filename, 0, sizeof(filename));
+	memset(checksums, 0, sizeof(checksums));
+	result = RET_OK;
+
+	for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+		if( ic != ic_uncompressed &&
+		    (compressions & IC_FLAG(ic)) == 0 )
+			continue;
+		filename[ic] = calc_compressedname(relfilename, ic);
+		if( filename[ic] == NULL ) {
+			result = RET_ERROR_OOM;
+			break;
 		}
 	}
-	if( (compressions & IC_FLAG(ic_gzip)) != 0 ) {
-		char *gzfilename;
-		gzfilename = calc_addsuffix(filename,"gz");
-		if( gzfilename == NULL ) {
-			free(filename);
-			return RET_ERROR_OOM;
+	if( RET_IS_OK(result) ) {
+		/* first look if the there are actual files, in case
+		 * the cache still lists them but they got lost */
+
+		for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+			char *fullfilename;
+
+			if( (compressions & IC_FLAG(ic)) == 0 )
+				continue;
+			assert( filename[ic] != NULL );
+			fullfilename = calc_dirconcat(release->dirofdist,
+					filename[ic]);
+			if( fullfilename == NULL ) {
+				result = RET_ERROR_OOM;
+				break;
+			}
+			if( !isregularfile(fullfilename) ) {
+				free(fullfilename);
+				result = RET_NOTHING;
+				break;
+			}
+			free(fullfilename);
 		}
-		if( !isregularfile(gzfilename) ) {
-			free(filename);
-			free(gzfilename);
-			return RET_NOTHING;
-		}
-		free(gzfilename);
 	}
-#ifdef HAVE_LIBBZ2
-	if( (compressions & IC_FLAG(ic_bzip2)) != 0 ) {
-		char *bzfilename;
-		bzfilename = calc_addsuffix(filename,"bz2");
-		if( bzfilename == NULL ) {
-			free(filename);
-			return RET_ERROR_OOM;
-		}
-		if( !isregularfile(bzfilename) ) {
-			free(filename);
-			free(bzfilename);
-			return RET_NOTHING;
-		}
-		free(bzfilename);
+	if( RET_IS_OK(result) && release->cachedb == NULL )
+		result = RET_NOTHING;
+	if( !RET_IS_OK(result) ) {
+		for( ic = ic_uncompressed ; ic < ic_count ; ic++ )
+			free(filename[ic]);
+		return result;
 	}
-#endif
-	free(filename);
 
 	/* now that the files are there look into the cache
-	 * what md5sum they have. */
+	 * what checksums they have. */
 
-	filename = strdup(relfilename);
-	if( filename == NULL )
-		return RET_ERROR_OOM;
+	for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+		char *combinedchecksum;
 
-	r = getcachevalue(release,relfilename,&md5sum);
-	if( !RET_IS_OK(r) ) {
-		free(filename);
-		return r;
-	}
-	if( (compressions & IC_FLAG(ic_gzip)) != 0 ) {
-		gzrelfilename = calc_addsuffix(relfilename,"gz");
-		if( gzrelfilename == NULL ) {
-			free(filename);
-			free(md5sum);
-			return RET_ERROR_OOM;
-		}
-		r = getcachevalue(release,gzrelfilename,&gzmd5sum);
+		if( filename[ic] == NULL )
+			continue;
+		r = table_getrecord(release->cachedb, filename[ic],
+				&combinedchecksum);
 		if( !RET_IS_OK(r) ) {
-			free(filename);
-			free(md5sum);
-			return r;
+			result = r;
+			break;
 		}
-	} else {
-		gzrelfilename = NULL;
-		gzmd5sum = NULL;
-	}
-#ifdef HAVE_LIBBZ2
-	if( (compressions & IC_FLAG(ic_bzip2)) != 0 ) {
-		bzrelfilename = calc_addsuffix(relfilename,"bz2");
-		if( bzrelfilename == NULL ) {
-			free(filename);
-			free(md5sum);
-			free(gzmd5sum);
-			free(gzrelfilename);
-			return RET_ERROR_OOM;
-		}
-		r = getcachevalue(release,bzrelfilename,&bzmd5sum);
+		r = checksums_parse(&checksums[ic], combinedchecksum);
+		// TODO: handle malformed checksums better?
+		free(combinedchecksum);
 		if( !RET_IS_OK(r) ) {
-			free(filename);
-			free(md5sum);
-			free(gzmd5sum);
-			free(gzrelfilename);
-			return r;
+			result = r;
+			break;
 		}
-	} else {
-		bzrelfilename = NULL;
-		bzmd5sum = NULL;
 	}
-#endif
-
-	result = newreleaseentry(release,filename,md5sum,NULL,NULL);
-	if( gzrelfilename != NULL ) {
-		r = newreleaseentry(release,gzrelfilename,gzmd5sum,NULL,NULL);
-		RET_UPDATE(result,r);
+	/* some files might not yet have some type of checksum available,
+	 * so calculate them (checking the other checksums match...): */
+	if( RET_IS_OK(result) ) {
+		for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+			char *fullfilename;
+			if( filename[ic] == NULL )
+				continue;
+			fullfilename = calc_dirconcat(release->dirofdist,
+					filename[ic]);
+			if( fullfilename == NULL )
+				r = RET_ERROR_OOM;
+			else
+				r = checksums_complete(&checksums[ic],
+					fullfilename);
+			if( r == RET_ERROR_WRONG_MD5 ) {
+				fprintf(stderr,
+"WARNING: '%s' is different from recorded checksums.\n"
+"(This was only catched because some new checksum type was not yet available.)\n"
+"Triggering recreation of that file.\n", fullfilename);
+				r = RET_NOTHING;
+			}
+			free(fullfilename);
+			if( !RET_IS_OK(r) ) {
+				result = r;
+				break;
+			}
+		}
 	}
-#ifdef HAVE_LIBBZ2
-	if( bzrelfilename != NULL ) {
-		r = newreleaseentry(release,bzrelfilename,bzmd5sum,NULL,NULL);
-		RET_UPDATE(result,r);
+	if( !RET_IS_OK(result) ) {
+		for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+			if( filename[ic] == NULL )
+				continue;
+			free(filename[ic]);
+			checksums_free(checksums[ic]);
+		}
+		return result;
+	}
+	/* everything found, commit it: */
+	result = RET_OK;
+	for( ic = ic_uncompressed ; ic < ic_count ; ic++ ) {
+		if( filename[ic] == NULL )
+			continue;
+		r = newreleaseentry(release, filename[ic],
+				checksums[ic],
+				NULL, NULL);
+		RET_UPDATE(result, r);
 	}
-#endif
 	return result;
 }
 
+
 struct filetorelease {
 	retvalue state;
 	struct openfile {
 		int fd;
-		struct MD5Context context;
-		off_t filesize;
+		struct checksumscontext context;
 		char *relativefilename;
 		char *fullfinalfilename;
 		char *fulltemporaryfilename;
 	} f[ic_count];
+	/* input buffer, to checksum/compress data at once */
+	unsigned char *buffer; size_t waiting_bytes;
 	/* output buffer for gzip compression */
 	unsigned char *gzoutputbuffer; size_t gz_waiting_bytes;
 	z_stream gzstream;
-#ifdef OLDZLIB
-	uLong crc;
-#endif
 #ifdef HAVE_LIBBZ2
 	/* output buffer for bzip2 compression */
 	char *bzoutputbuffer; size_t bz_waiting_bytes;
@@ -377,7 +389,7 @@ void release_abortfile(struct filetorelease *file) {
 
 	for( i = ic_uncompressed ; i < ic_count ; i++ ) {
 		if( file->f[i].fd >= 0 ) {
-			close(file->f[i].fd);
+			(void)close(file->f[i].fd);
 			if( file->f[i].fulltemporaryfilename != NULL )
 				(void)unlink(file->f[i].fulltemporaryfilename);
 		}
@@ -385,19 +397,20 @@ void release_abortfile(struct filetorelease *file) {
 		free(file->f[i].fullfinalfilename);
 		free(file->f[i].fulltemporaryfilename);
 	}
+	free(file->buffer);
 	free(file->gzoutputbuffer);
 	if( file->gzstream.next_out != NULL ) {
-		deflateEnd(&file->gzstream);
+		(void)deflateEnd(&file->gzstream);
 	}
 #ifdef HAVE_LIBBZ2
 	free(file->bzoutputbuffer);
 	if( file->bzstream.next_out != NULL ) {
-		BZ2_bzCompressEnd(&file->bzstream);
+		(void)BZ2_bzCompressEnd(&file->bzstream);
 	}
 #endif
 }
 
-bool_t release_oldexists(struct filetorelease *file) {
+bool release_oldexists(struct filetorelease *file) {
 	if( file->f[ic_uncompressed].fullfinalfilename != NULL ) {
 		if( file->f[ic_gzip].fullfinalfilename != NULL ) {
 			return isregularfile(file->f[ic_gzip].fullfinalfilename) &&
@@ -432,8 +445,7 @@ static retvalue openfile(const char *dirofdist, struct openfile *f) {
 
 static retvalue writetofile(struct openfile *file, const unsigned char *data, size_t len) {
 
-	file->filesize += len;
-	MD5Update(&file->context,data,len);
+	checksumscontext_update(&file->context, data, len);
 
 	if( file->fd < 0 )
 		return RET_NOTHING;
@@ -455,35 +467,13 @@ static retvalue writetofile(struct openfile *file, const unsigned char *data, si
 	return RET_OK;
 }
 
-#define GZBUFSIZE 40960
-
 static retvalue	initgzcompression(struct filetorelease *f) {
 	int zret;
 
-#ifndef OLDZLIB
 	if( (zlibCompileFlags() & (1<<17)) !=0 ) {
 		fprintf(stderr,"libz compiled without .gz supporting code\n");
 		return RET_ERROR;
 	}
-#else
-	unsigned char header[10] = {
-		31,139, Z_DEFLATED,
-		/* flags */
-		0,
-		/* time */
-		0, 0, 0, 0,
-		/* xtra-flags */
-		0,
-		/* os (3 = unix, 255 = unknown)
-		 * using unknown to generate the same file
-		 * with or without OLDZLIB */
-		255};
-	retvalue r = writetofile(&f->f[ic_gzip],header,10);
-	if( RET_WAS_ERROR(r) )
-		return r;
-	/* we have to calculate the crc ourself */
-	f->crc = crc32(0L, NULL, 0);
-#endif
 	f->gzoutputbuffer = malloc(GZBUFSIZE);
 	if( f->gzoutputbuffer == NULL )
 		return RET_ERROR_OOM;
@@ -494,7 +484,6 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 	f->gzstream.zalloc = NULL;
 	f->gzstream.zfree = NULL;
 	f->gzstream.opaque = NULL;
-#ifndef OLDZLIB
 	zret = deflateInit2(&f->gzstream,
 			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */
 			Z_DEFAULT_COMPRESSION,
@@ -507,20 +496,7 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 			/* default or Z_FILTERED or Z_HUFFMAN_ONLY or Z_RLE */
 			Z_DEFAULT_STRATEGY
 			);
-#else
-	zret = deflateInit2(&f->gzstream,
-			/* Level: 0-9 or Z_DEFAULT_COMPRESSION: */
-			Z_DEFAULT_COMPRESSION,
-			/* only possibility yet: */
-			Z_DEFLATED,
-			/* negative to no generate zlib header */
-			-MAX_WBITS,
-			/* how much memory to use 1-9 */
-			8,
-			/* default or Z_FILTERED or Z_HUFFMAN_ONLY or Z_RLE */
-			Z_DEFAULT_STRATEGY
-			);
-#endif
+	f->gz_waiting_bytes = GZBUFSIZE - f->gzstream.avail_out;
 	if( zret == Z_MEM_ERROR )
 		return RET_ERROR_OOM;
 	if( zret != Z_OK ) {
@@ -538,8 +514,6 @@ static retvalue	initgzcompression(struct filetorelease *f) {
 
 #ifdef HAVE_LIBBZ2
 
-#define BZBUFSIZE 40960
-
 static retvalue	initbzcompression(struct filetorelease *f) {
 	int bzret;
 
@@ -574,7 +548,7 @@ static retvalue	initbzcompression(struct filetorelease *f) {
 
 static retvalue startfile(struct release *release,
 		char *filename, compressionset compressions,
-		bool_t usecache,
+		bool usecache,
 		struct filetorelease **file) {
 	struct filetorelease *n;
 	enum indexcompression i;
@@ -594,6 +568,11 @@ static retvalue startfile(struct release *release,
 		free(filename);
 		return RET_ERROR_OOM;
 	}
+	n->buffer = malloc(INPUT_BUFFER_SIZE);
+	if( n->buffer == NULL ) {
+		release_abortfile(n);
+		return RET_ERROR_OOM;
+	}
 	for( i = ic_uncompressed ; i < ic_count ; i ++ ) {
 		n->f[i].fd = -1;
 	}
@@ -622,7 +601,7 @@ static retvalue startfile(struct release *release,
 			release_abortfile(n);
 			return r;
 		}
-		MD5Init(&n->f[ic_gzip].context);
+		checksumscontext_init(&n->f[ic_gzip].context);
 		r = initgzcompression(n);
 		if( RET_WAS_ERROR(r) ) {
 			release_abortfile(n);
@@ -642,7 +621,7 @@ static retvalue startfile(struct release *release,
 			release_abortfile(n);
 			return r;
 		}
-		MD5Init(&n->f[ic_bzip2].context);
+		checksumscontext_init(&n->f[ic_bzip2].context);
 		r = initbzcompression(n);
 		if( RET_WAS_ERROR(r) ) {
 			release_abortfile(n);
@@ -650,14 +629,14 @@ static retvalue startfile(struct release *release,
 		}
 	}
 #endif
-	MD5Init(&n->f[ic_uncompressed].context);
+	checksumscontext_init(&n->f[ic_uncompressed].context);
 	*file = n;
 	return RET_OK;
 }
 
 retvalue release_startfile2(struct release *release,
 		const char *relative_dir, const char *filename, compressionset compressions,
-		bool_t usecache,
+		bool usecache,
 		struct filetorelease **file) {
 	char *relfilename;
 
@@ -668,7 +647,7 @@ retvalue release_startfile2(struct release *release,
 }
 retvalue release_startfile(struct release *release,
 		const char *filename, compressionset compressions,
-		bool_t usecache,
+		bool usecache,
 		struct filetorelease **file) {
 	char *relfilename;
 
@@ -679,7 +658,7 @@ retvalue release_startfile(struct release *release,
 }
 
 static retvalue releasefile(struct release *release, struct openfile *f) {
-	char *md5sum;
+	struct checksums *checksums;
 	retvalue r;
 
 	if( f->relativefilename == NULL ) {
@@ -692,33 +671,25 @@ static retvalue releasefile(struct release *release, struct openfile *f) {
 	  	|| (f->fullfinalfilename != NULL
 		  && f->fulltemporaryfilename != NULL));
 
-	r = md5sum_genstring(&md5sum,&f->context,f->filesize);
-	assert( r != RET_NOTHING );
+	r = checksums_from_context(&checksums, &f->context);
 	if( RET_WAS_ERROR(r) )
 		return r;
-	r = newreleaseentry(release,f->relativefilename,md5sum,
+	r = newreleaseentry(release, f->relativefilename, checksums,
 			f->fullfinalfilename,
 			f->fulltemporaryfilename);
 	f->relativefilename = NULL;
 	f->fullfinalfilename = NULL;
 	f->fulltemporaryfilename = NULL;
-	return RET_OK;
+	return r;
 }
 
-static retvalue writegz(struct filetorelease *f, const char *data, size_t len) {
+static retvalue writegz(struct filetorelease *f) {
 	int zret;
 
 	assert( f->f[ic_gzip].fd >= 0  );
 
-	if( len == 0 )
-		return RET_NOTHING;
-
-#ifdef OLDZLIB
-	f->crc = crc32(f->crc,data,len);
-#endif
-
-	f->gzstream.next_in = (unsigned char*)data;
-	f->gzstream.avail_in = len;
+	f->gzstream.next_in = f->buffer;
+	f->gzstream.avail_in = INPUT_BUFFER_SIZE;
 
 	do {
 		f->gzstream.next_out = f->gzoutputbuffer + f->gz_waiting_bytes;
@@ -766,16 +737,11 @@ static retvalue writegz(struct filetorelease *f, const char *data, size_t len) {
 
 static retvalue finishgz(struct filetorelease *f) {
 	int zret;
-	unsigned char dummy;
-#ifdef OLDZLIB
-	unsigned char buffer[4];
-	retvalue r;
-#endif
 
 	assert( f->f[ic_gzip].fd >= 0  );
 
-	f->gzstream.next_in = &dummy;
-	f->gzstream.avail_in = 0;
+	f->gzstream.next_in = f->buffer;
+	f->gzstream.avail_in = f->waiting_bytes;
 
 	do {
 		f->gzstream.next_out = f->gzoutputbuffer + f->gz_waiting_bytes;
@@ -830,38 +796,19 @@ static retvalue finishgz(struct filetorelease *f) {
 		return RET_ERROR;
 	}
 
-#ifdef OLDZLIB
-	buffer[0] = f->crc & 0xFF;
-	buffer[1] = (f->crc >> (8*1)) & 0xFF;
-	buffer[2] = (f->crc >> (8*2)) & 0xFF;
-	buffer[3] = (f->crc >> (8*3)) & 0xFF;
-	r = writetofile(&f->f[ic_gzip],buffer,4);
-	if( RET_WAS_ERROR(r) )
-		return r;
-	buffer[0] = f->f[ic_uncompressed].filesize & 0xFF;
-	buffer[1] = (f->f[ic_uncompressed].filesize >> (8*1)) & 0xFF;
-	buffer[2] = (f->f[ic_uncompressed].filesize >> (8*2)) & 0xFF;
-	buffer[3] = (f->f[ic_uncompressed].filesize >> (8*3)) & 0xFF;
-	r = writetofile(&f->f[ic_gzip],buffer,4);
-	if( RET_WAS_ERROR(r) )
-		return r;
-#endif
 
 	return RET_OK;
 }
 
 #ifdef HAVE_LIBBZ2
 
-static retvalue writebz(struct filetorelease *f, const char *data, size_t len) {
+static retvalue writebz(struct filetorelease *f) {
 	int bzret;
 
 	assert( f->f[ic_bzip2].fd >= 0  );
 
-	if( len == 0 )
-		return RET_NOTHING;
-
-	f->bzstream.next_in = (char*)data;
-	f->bzstream.avail_in = len;
+	f->bzstream.next_in = (char*)f->buffer;
+	f->bzstream.avail_in = INPUT_BUFFER_SIZE;
 
 	do {
 		f->bzstream.next_out = f->bzoutputbuffer + f->bz_waiting_bytes;
@@ -897,12 +844,11 @@ static retvalue writebz(struct filetorelease *f, const char *data, size_t len) {
 
 static retvalue finishbz(struct filetorelease *f) {
 	int bzret;
-	char dummy;
 
 	assert( f->f[ic_bzip2].fd >= 0  );
 
-	f->bzstream.next_in = &dummy;
-	f->bzstream.avail_in = 0;
+	f->bzstream.next_in = (char*)f->buffer;
+	f->bzstream.avail_in = f->waiting_bytes;
 
 	do {
 		f->bzstream.next_out = f->bzoutputbuffer + f->bz_waiting_bytes;
@@ -953,6 +899,12 @@ retvalue release_finishfile(struct release *release, struct filetorelease *file)
 		return r;
 	}
 
+	r = writetofile(&file->f[ic_uncompressed],
+			file->buffer, file->waiting_bytes);
+	if( RET_WAS_ERROR(r) ) {
+		release_abortfile(file);
+		return r;
+	}
 	if( file->f[ic_uncompressed].fd >= 0 ) {
 		if( close(file->f[ic_uncompressed].fd) != 0 ) {
 			int e = errno;
@@ -992,7 +944,7 @@ retvalue release_finishfile(struct release *release, struct filetorelease *file)
 		file->f[ic_bzip2].fd = -1;
 	}
 #endif
-	release->new = TRUE;
+	release->new = true;
 	result = RET_OK;
 
 	for( i = ic_uncompressed ; i < ic_count ; i++ ) {
@@ -1003,6 +955,7 @@ retvalue release_finishfile(struct release *release, struct filetorelease *file)
 		}
 		RET_UPDATE(result,r);
 	}
+	free(file->buffer);
 	free(file->gzoutputbuffer);
 #ifdef HAVE_LIBBZ2
 	free(file->bzoutputbuffer);
@@ -1011,31 +964,69 @@ retvalue release_finishfile(struct release *release, struct filetorelease *file)
 	return result;
 }
 
-
-retvalue release_writedata(struct filetorelease *file, const char *data, size_t len) {
+static retvalue release_processbuffer(struct filetorelease *file) {
 	retvalue result,r;
 
 	result = RET_OK;
-	/* always call this so that md5sum is calculated */
-	r = writetofile(&file->f[ic_uncompressed],(const unsigned char*)data,len);
-	RET_UPDATE(result,r);
+	assert( file->waiting_bytes == INPUT_BUFFER_SIZE );
+
+	/* always call this - even if there is no uncompressed file
+	 * to generate - so that checksums are calculated */
+	r = writetofile(&file->f[ic_uncompressed],
+			file->buffer, INPUT_BUFFER_SIZE);
+	RET_UPDATE(result, r);
+
 	if( file->f[ic_gzip].relativefilename != NULL ) {
-		r = writegz(file,data,len);
-		RET_UPDATE(result,r);
+		r = writegz(file);
+		RET_UPDATE(result, r);
 	}
-	RET_UPDATE(file->state,result);
+	RET_UPDATE(file->state, result);
 #ifdef HAVE_LIBBZ2
 	if( file->f[ic_bzip2].relativefilename != NULL ) {
-		r = writebz(file,data,len);
+		r = writebz(file);
 		RET_UPDATE(result,r);
 	}
-	RET_UPDATE(file->state,result);
+	RET_UPDATE(file->state, result);
 #endif
 	return result;
 }
 
+retvalue release_writedata(struct filetorelease *file, const char *data, size_t len) {
+	retvalue result,r;
+	size_t free_bytes;
+
+	result = RET_OK;
+	/* move stuff into buffer, so stuff is not processed byte by byte */
+	free_bytes = INPUT_BUFFER_SIZE - file->waiting_bytes;
+	if( len < free_bytes ) {
+		memcpy(file->buffer + file->waiting_bytes, data, len);
+		file->waiting_bytes += len;
+		assert( file->waiting_bytes < INPUT_BUFFER_SIZE );
+		return RET_OK;
+	}
+	memcpy(file->buffer + file->waiting_bytes, data, free_bytes);
+	len -= free_bytes;
+	data += free_bytes;
+	file->waiting_bytes += free_bytes;
+	r = release_processbuffer(file);
+	RET_UPDATE(result, r);
+	while( len >= INPUT_BUFFER_SIZE ) {
+		/* should not hopefully not happen, as all this copying
+		 * is quite slow... */
+		memcpy(file->buffer, data, INPUT_BUFFER_SIZE);
+		len -= INPUT_BUFFER_SIZE;
+		data += INPUT_BUFFER_SIZE;
+		r = release_processbuffer(file);
+		RET_UPDATE(result, r);
+	}
+	memcpy(file->buffer, data, len);
+	file->waiting_bytes = len;
+	assert( file->waiting_bytes < INPUT_BUFFER_SIZE );
+	return result;
+}
+
 /* Generate a "Release"-file for arbitrary directory */
-retvalue release_directorydescription(struct release *release, const struct distribution *distribution,const struct target *target,const char *releasename,bool_t onlyifneeded) {
+retvalue release_directorydescription(struct release *release, const struct distribution *distribution, const struct target *target, const char *releasename, bool onlyifneeded) {
 	retvalue r;
 	struct filetorelease *f;
 
@@ -1066,112 +1057,79 @@ retvalue release_directorydescription(struct release *release, const struct dist
 	return r;
 }
 
-#define CLEARDBT(dbt) {memset(&dbt,0,sizeof(dbt));}
-#define SETDBT(dbt,datastr) {const char *my = datastr;memset(&dbt,0,sizeof(dbt));dbt.data=(void *)my;dbt.size=strlen(my)+1;}
-
-static retvalue getcachevalue(struct release *r,const char *relfilename, char **md5sum) {
-	int dbret;
-	DBT key,data;
-
-	if( r->cachedb == NULL )
-		return RET_NOTHING;
-
-	SETDBT(key,relfilename);
-	CLEARDBT(data);
-	dbret = r->cachedb->get(r->cachedb, NULL, &key, &data, 0);
-	if( dbret == 0 ) {
-		char *c;
-		c = strdup(data.data);
-		if( c == NULL )
-			return RET_ERROR_OOM;
-		else {
-			*md5sum = c;
-			return RET_OK;
-		}
-	} else if( dbret == DB_NOTFOUND ) {
-		return RET_NOTHING;
-	} else {
-		r->cachedb->err(r->cachedb, dbret, "release.cache.db:");
-		return RET_DBERR(dbret);
-	}
-}
-
-static retvalue storechecksums(struct release *r) {
-
+static retvalue storechecksums(struct release *release) {
 	struct release_entry *file;
+	retvalue result, r;
+	const char *combinedchecksum;
+	/* size including trailing '\0' character: */
+	size_t len;
 
-	for( file = r->files ; file != NULL ; file = file->next ) {
-		int dbret;
-		DBT key,data;
+	result = RET_OK;
 
-		if( file->md5sum == NULL || file->relativefilename == NULL )
+	for( file = release->files ; file != NULL ; file = file->next ) {
+
+		if( file->relativefilename == NULL )
 			continue;
 
-		SETDBT(key,file->relativefilename);
+		r = table_deleterecord(release->cachedb,
+				file->relativefilename, true);
+		if( RET_WAS_ERROR(r) )
+			return r;
 
-		dbret = r->cachedb->del(r->cachedb, NULL, &key, 0);
-		if( dbret < 0 && dbret != DB_NOTFOUND ) {
-			r->cachedb->err(r->cachedb, dbret, "release.cache.db:");
-			return RET_DBERR(dbret);
-		}
-		SETDBT(key,file->relativefilename);
-		SETDBT(data,file->md5sum);
+		r = checksums_getcombined(file->checksums, &combinedchecksum, &len);
+		RET_UPDATE(result, r);
+		if( !RET_IS_OK(r) )
+			continue;
 
-		dbret = r->cachedb->put(r->cachedb,NULL,&key,&data,DB_NOOVERWRITE);
-		if( dbret < 0 ) {
-			r->cachedb->err(r->cachedb, dbret, "release.cache.db:");
-			return RET_DBERR(dbret);
-		}
+		r = table_adduniqsizedrecord(release->cachedb,
+				file->relativefilename, combinedchecksum, len+1,
+				false, false);
+		RET_UPDATE(result, r);
 	}
-	return RET_OK;
+	return result;
 }
-#undef CLEARDBT
-#undef SETDBT
 
 /* Generate a main "Release" file for a distribution */
-retvalue release_write(/*@only@*/struct release *release, struct distribution *distribution, bool_t onlyifneeded) {
+retvalue release_prepare(struct release *release, struct distribution *distribution, bool onlyifneeded) {
 	size_t s;
-	int e;
-	retvalue result,r;
+	retvalue r;
 	char buffer[100];
 	time_t t;
 	struct tm *gmt;
-	int i;
 	struct release_entry *file;
-	bool_t somethingwasdone;
-	struct signedfile *signedfile;
+	enum checksumtype cs;
+	int i;
+	static const char * const release_checksum_headers[cs_hashCOUNT] =
+		{ "MD5Sum:\n", "SHA1:\n" };
 
+	// TODO: check for existance of Release file here first?
 	if( onlyifneeded && !release->new ) {
-		release_free(release);
 		return RET_NOTHING;
 	}
 
 	(void)time(&t);
 	gmt = gmtime(&t);
 	if( gmt == NULL ) {
-		release_free(release);
 		return RET_ERROR_OOM;
 	}
 	s=strftime(buffer,99,"%a, %d %b %Y %H:%M:%S +0000",gmt);
 	if( s == 0 || s >= 99) {
 		fprintf(stderr,"strftime is doing strange things...\n");
-		release_free(release);
 		return RET_ERROR;
 	}
 
 	if( distribution->signwith != NULL )
 		r = signature_startsignedfile(release->dirofdist, "Release",
 				distribution->signwith,
-				&signedfile);
+				&release->signedfile);
 	else
 		r = signature_startunsignedfile(release->dirofdist, "Release",
-				&signedfile);
+				&release->signedfile);
 	if( RET_WAS_ERROR(r) ) {
-		release_free(release);
 		return r;
 	}
-#define writestring(s) signedfile_write(signedfile, s, strlen(s))
-#define writechar(c) {char __c = c ; signedfile_write(signedfile, &__c, 1); }
+#define writestring(s) signedfile_write(release->signedfile, s, strlen(s))
+#define writechar(c) {char __c = c ; signedfile_write(release->signedfile, &__c, 1); }
 
 	if( distribution->origin != NULL ) {
 		writestring("Origin: ");
@@ -1217,24 +1175,45 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 		writestring("\nNotAutomatic: ");
 		writestring(distribution->notautomatic);
 	}
+	writechar('\n');
 
-	writestring("\nMD5Sum:\n");
-	for( file = release->files ; file != NULL ; file = file->next ) {
-		if( file->md5sum != NULL && file->relativefilename != NULL ) {
+	for( cs = cs_md5sum ; cs < cs_hashCOUNT ; cs++ ) {
+		assert( release_checksum_headers[cs] != NULL );
+		writestring(release_checksum_headers[cs]);
+		for( file = release->files ; file != NULL ; file = file->next ) {
+			const char *hash, *size;
+			size_t hashlen, sizelen;
+			if( file->relativefilename == NULL )
+				continue;
+			if( !checksums_gethashpart(file->checksums, cs,
+					&hash, &hashlen, &size, &sizelen) )
+				continue;
 			writechar(' ');
-			writestring(file->md5sum);
+			signedfile_write(release->signedfile, hash, hashlen);
+			writechar(' ');
+			signedfile_write(release->signedfile, size, sizelen);
 			writechar(' ');
 			writestring(file->relativefilename);
 			writechar('\n');
 		}
 	}
-	r = signedfile_prepare(signedfile, distribution->signwith);
+	r = signedfile_prepare(release->signedfile, distribution->signwith);
 	if( RET_WAS_ERROR(r) ) {
-		signedfile_free(signedfile);
-		release_free(release);
+		signedfile_free(release->signedfile);
+		release->signedfile = NULL;
 		return r;
 	}
-	somethingwasdone = FALSE;
+	return RET_OK;
+}
+
+/* Generate a main "Release" file for a distribution */
+retvalue release_finish(/*@only@*/struct release *release, struct distribution *distribution) {
+	retvalue result,r;
+	int e;
+	struct release_entry *file;
+	bool somethingwasdone;
+
+	somethingwasdone = false;
 	result = RET_OK;
 
 	for( file = release->files ; file != NULL ; file = file->next ) {
@@ -1255,38 +1234,40 @@ retvalue release_write(/*@only@*/struct release *release, struct distribution *d
 				/* after something was done, do not stop
 				 * but try to do as much as possible */
 				if( !somethingwasdone ) {
-					signedfile_free(signedfile);
 					release_free(release);
 					return r;
 				}
 				RET_UPDATE(result,r);
 			} else
-				somethingwasdone = TRUE;
+				somethingwasdone = true;
 		}
 	}
-	r = signedfile_finalize(signedfile, &somethingwasdone);
-	signedfile_free(signedfile);
+	r = signedfile_finalize(release->signedfile, &somethingwasdone);
 	if( RET_WAS_ERROR(r) && !somethingwasdone ) {
 		release_free(release);
 		return r;
 	}
 	RET_UPDATE(result,r);
-
+	if( RET_WAS_ERROR(result) && somethingwasdone ) {
+		fprintf(stderr,
+"ATTENTION: some files were already moved to place, some could not be.\n"
+"The generated index files for %s might be in a inconsistent state\n"
+"and currently not useable! You should remove the reason for the failure\n"
+"(most likely bad access permissions) and export the affected distributions\n"
+"manually (via reprepro export codenames) as soon as possible!\n",
+			distribution->codename);
+	}
 	if( release->cachedb != NULL ) {
-	 	int dbret;
-
+		// TODO: split this in removing before and adding later?
+		// remember which file were changed in case of error, so
+		// only those are changed...
 		/* now update the cache database, so we find those the next time */
 		r = storechecksums(release);
 		RET_UPDATE(result,r);
 
-		/* check for an possible error in the cachedb,
-		 * release_free return no error */
-		dbret = release->cachedb->close(release->cachedb,0);
-		if( dbret < 0 ) {
-			release_free(release);
-			return RET_DBERR(dbret);
-		}
+		r = table_close(release->cachedb);
 		release->cachedb = NULL;
+		RET_ENDUPDATE(result, r);
 	}
 	/* free everything */
 	release_free(release);
