@@ -8,9 +8,13 @@
 #ifndef REPREPRO_STRLIST_H
 #include "strlist.h"
 #endif
+#ifndef REPREPRO_DATABASE_H
+#include "database.h"
+#endif
 
 struct release;
 
+#define ic_first ic_uncompressed
 enum indexcompression {ic_uncompressed=0, ic_gzip,
 #ifdef HAVE_LIBBZ2
 			ic_bzip2,
@@ -21,7 +25,7 @@ typedef unsigned int compressionset; /* 1 << indexcompression */
 #define IC_FLAG(a) (1<<(a))
 
 /* Initialize Release generation */
-retvalue release_init(const char *dbdir, const char *distdir, const char *codename, struct release **release);
+retvalue release_init(struct release **release, struct database *database, const char *distdir, const char *codename);
 /* same but for a snapshot */
 retvalue release_initsnapshot(const char *distdir, const char *codename, const char *name, struct release **release);
 
@@ -35,12 +39,12 @@ retvalue release_addold(struct release *release,/*@only@*/char *relfilename);
 
 struct filetorelease;
 
-retvalue release_startfile2(struct release *release, const char *relative_dir, const char *filename, compressionset compressions, bool_t usecache, struct filetorelease **file);
+retvalue release_startfile2(struct release *release, const char *relative_dir, const char *filename, compressionset compressions, bool usecache, struct filetorelease **file);
 
-retvalue release_startfile(struct release *release, const char *filename, compressionset compressions, bool_t usecache, struct filetorelease **file);
+retvalue release_startfile(struct release *release, const char *filename, compressionset compressions, bool usecache, struct filetorelease **file);
 
-/* return TRUE if an old file is already there */
-bool_t release_oldexists(struct filetorelease *file);
+/* return true if an old file is already there */
+bool release_oldexists(struct filetorelease *file);
 
 /* errors will be cached for release_finishfile */
 retvalue release_writedata(struct filetorelease *file, const char *data, size_t len);
@@ -51,9 +55,10 @@ retvalue release_finishfile(struct release *release, /*@only@*/struct filetorele
 
 struct distribution;
 struct target;
-retvalue release_directorydescription(struct release *release, const struct distribution *distribution,const struct target *target,const char *filename,bool_t onlyifneeded);
+retvalue release_directorydescription(struct release *release, const struct distribution *distribution, const struct target *target, const char *filename, bool onlyifneeded);
 
 void release_free(/*@only@*/struct release *release);
-retvalue release_write(/*@only@*/struct release *release, struct distribution *distribution, bool_t onlyneeded);
+retvalue release_prepare(struct release *release, struct distribution *distribution, bool onlyneeded);
+retvalue release_finish(/*@only@*/struct release *release, struct distribution *distribution);
 
 #endif
