@@ -25,20 +25,23 @@
 #include <string.h>
 #include <malloc.h>
 #include <fcntl.h>
+#ifdef HAVE_LIBGPGME
 #include <gpgme.h>
+#endif
+#include "globals.h"
 #include "error.h"
-#include "ignore.h"
 #include "mprintf.h"
 #include "strlist.h"
 #include "dirs.h"
 #include "names.h"
-#include "md5sum.h"
 #include "chunks.h"
 #include "release.h"
+#include "readtextfile.h"
 #include "signature.h"
 
 extern int verbose;
 
+#ifdef HAVE_LIBGPGME
 static gpgme_ctx_t context = NULL;
 
 static retvalue gpgerror(gpgme_error_t err) {
@@ -69,8 +72,10 @@ static gpgme_error_t signature_getpassphrase(UNUSED(void *hook), const char *uid
 	free(msg);
 	return GPG_ERR_NO_ERROR;
 }
+#endif /* HAVE_LIBGPGME */
 
-retvalue signature_init(bool_t allowpassphrase){
+retvalue signature_init(bool allowpassphrase){
+#ifdef HAVE_LIBGPGME
 	gpgme_error_t err;
 
 	if( context != NULL )
@@ -87,25 +92,28 @@ retvalue signature_init(bool_t allowpassphrase){
 	if( allowpassphrase )
 		gpgme_set_passphrase_cb(context,signature_getpassphrase,NULL);
 	gpgme_set_armor(context,1);
+#endif /* HAVE_LIBGPGME */
 	return RET_OK;
 }
 
 void signatures_done(void) {
+#ifdef HAVE_LIBGPGME
 	if( context != NULL ) {
 		gpgme_release(context);
 		context = NULL;
 	}
+#endif /* HAVE_LIBGPGME */
 }
 
+#ifdef HAVE_LIBGPGME
 static inline retvalue containskey(const char *key, const char *fingerprint) {
-
 	size_t fl,kl;
 	const char *keypart,*p;
 
 	fl = strlen(fingerprint);
 
 	keypart = key;
-	while( TRUE ) {
+	while( true ) {
 		while( *keypart != '\0' && xisspace(*keypart) )
 			keypart++;
 		if( *keypart == '\0' )
@@ -115,7 +123,8 @@ static inline retvalue containskey(const char *key, const char *fingerprint) {
 		while( *p != '\0' && !xisspace(*p) && *p != '|' )
 			p++;
 		kl = p-keypart;
-		if( kl < 8 && !IGNORING("Ignoring","To ignore this",shortkeyid,"Too short keyid specified (less than 8 characters) in '%s'!\n",key)) {
+		if( kl < 8 ) {
+			fprintf(stderr, "Too short keyid specified (less than 8 characters) in '%s'!\n",key);
 			return RET_ERROR;
 		}
 		if( kl < fl && strncasecmp(fingerprint+fl-kl,keypart,kl) == 0 )
@@ -133,24 +142,27 @@ static inline retvalue containskey(const char *key, const char *fingerprint) {
 		}
 	}
 }
+#endif /* HAVE_LIBGPGME */
 
 retvalue signature_check(const char *options, const char *releasegpg, const char *release) {
 	retvalue r;
+#ifdef HAVE_LIBGPGME
 	gpgme_error_t err;
 	int fd,gpgfd;
 	gpgme_data_t dh,dh_gpg;
 	gpgme_verify_result_t result;
 	gpgme_signature_t s;
+#endif /* HAVE_LIBGPGME */
 
 	if( release == NULL || releasegpg == NULL )
 		return RET_ERROR_OOM;
 
-	r = signature_init(FALSE);
+	r = signature_init(false);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
+#ifdef HAVE_LIBGPGME
 	/* Read the file and its signature into memory: */
-
 	gpgfd = open(releasegpg, O_RDONLY|O_NOCTTY);
 	if( gpgfd < 0 ) {
 		int e = errno;
@@ -160,19 +172,19 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 	fd = open(release, O_RDONLY|O_NOCTTY);
 	if( fd < 0 ) {
 		int e = errno;
-		close(gpgfd);
+		(void)close(gpgfd);
 		fprintf(stderr, "Error opening '%s': %s\n", release, strerror(e));
 		return RET_ERRNO(e);
 	}
 	err = gpgme_data_new_from_fd(&dh_gpg, gpgfd);
 	if( err != 0 ) {
-		close(gpgfd);close(fd);
+		(void)close(gpgfd); (void)close(fd);
 		return gpgerror(err);
 	}
 	err = gpgme_data_new_from_fd(&dh, fd);
 	if( err != 0 ) {
 		gpgme_data_release(dh_gpg);
-		close(gpgfd);close(fd);
+		(void)close(gpgfd); (void)close(fd);
 		return gpgerror(err);
 	}
 
@@ -194,9 +206,14 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 		r = containskey(options, s->fpr);
 		if( RET_WAS_ERROR(r) )
 			return r;
-		if( r == RET_NOTHING )
+		if( r == RET_NOTHING ) {
+			if( gpgme_err_code(s->status) == GPG_ERR_NO_ERROR &&
+					verbose > 10 ) {
+				printf("Valid signature with key '%s', but that key is not looked at.\n", s->fpr);
+			}
 			/* there's no use in checking signatures not sufficient */
 			continue;
+		}
 		assert( RET_IS_OK(r) );
 		switch( gpgme_err_code(s->status) ) {
 			case GPG_ERR_NO_ERROR:
@@ -263,17 +280,24 @@ retvalue signature_check(const char *options, const char *releasegpg, const char
 			gpgme_err_code(s->status));
 		return RET_ERROR_GPGME;
 	}
-
 	return RET_NOTHING;
+#else /* HAVE_LIBGPGME */
+	fprintf(stderr,
+"ERROR: Cannot check signature as this reprepro binary is compiled with support\n"
+"for libgpgme.\n"); // TODO: "Only running external programs is supported.\n"
+	return RET_ERROR_GPGME;
+#endif /* HAVE_LIBGPGME */
 }
 
 retvalue signature_sign(const char *options, const char *filename, const char *signaturename) {
 	retvalue r;
+	int ret;
+#ifdef HAVE_LIBGPGME
 	gpgme_error_t err;
 	gpgme_data_t dh,dh_gpg;
-	int ret;
+#endif /* HAVE_LIBGPGME */
 
-	r = signature_init(FALSE);
+	r = signature_init(false);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -292,7 +316,9 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 		// TODO: allow external programs, too
 		fprintf(stderr,"'!' not allowed at start of signing options yet.\n");
 		return RET_ERROR;
-	} else if( strcasecmp(options,"yes") == 0 || strcasecmp(options,"default") == 0 ) {
+	}
+#ifdef HAVE_LIBGPGME
+	if( strcasecmp(options,"yes") == 0 || strcasecmp(options,"default") == 0 ) {
 		gpgme_signers_clear(context);
 		options = NULL;
 	} else {
@@ -397,8 +423,8 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 			written = write(fd,p,signature_len);
 			if( written < 0 ) {
 				int e = errno;
-				fprintf(stderr, "Error writing to %s: %s\n",
-						signaturename,
+				fprintf(stderr, "Error %d writing to %s: %s\n",
+						e, signaturename,
 						strerror(e));
 				free(signature_data);
 				return RET_ERRNO(e);
@@ -414,8 +440,8 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 		ret = close(fd);
 		if( ret < 0 ) {
 			int e = errno;
-			fprintf(stderr, "Error writing to %s: %s\n",
-					signaturename,
+			fprintf(stderr, "Error %d writing to %s: %s\n",
+					e, signaturename,
 					strerror(e));
 			return RET_ERRNO(e);
 		}
@@ -425,16 +451,23 @@ retvalue signature_sign(const char *options, const char *filename, const char *s
 	}
 
 	return r;
+#else /* HAVE_LIBGPGME */
+	fprintf(stderr,
+"ERROR: Cannot creature signatures as this reprepro binary is compiled with support\n"
+"for libgpgme.\n"); // TODO: "Only running external programs is supported.\n"
+	return RET_ERROR_GPGME;
+#endif /* HAVE_LIBGPGME */
 }
 
+#ifdef HAVE_LIBGPGME
 /* retrieve a list of fingerprints of keys having signed (valid) or
  * which are mentioned in the signature (all). set broken if all signatures
  * was broken (hints to a broken file, as opposed to expired or whatever
  * else may make a signature invalid)). */
-static retvalue checksigs(const char *filename, struct strlist *valid, struct strlist *all, bool_t *broken) {
+static retvalue checksigs(const char *filename, struct strlist *valid, struct strlist *all, bool *broken) {
 	gpgme_verify_result_t result;
 	gpgme_signature_t s;
-	bool_t had_valid=FALSE, had_broken=FALSE;
+	bool had_valid = false, had_broken = false;
 
 	result = gpgme_op_verify_result(context);
 	if( result == NULL ) {
@@ -449,7 +482,7 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 		}
 		switch( gpgme_err_code(s->status) ) {
 			case GPG_ERR_NO_ERROR:
-				had_valid = TRUE;
+				had_valid = true;
 				if( valid != NULL ) {
 					retvalue r = strlist_add_dup(valid,
 								s->fpr);
@@ -458,21 +491,21 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 				}
 				continue;
 			case GPG_ERR_KEY_EXPIRED:
-				had_valid = TRUE;
+				had_valid = true;
 				if( verbose > 0 )
 					fprintf(stderr,
 "Ignoring signature with '%s' on '%s', as the key has expired.\n",
 						s->fpr, filename);
 				continue;
 			case GPG_ERR_CERT_REVOKED:
-				had_valid = TRUE;
+				had_valid = true;
 				if( verbose > 0 )
 					fprintf(stderr,
 "Ignoring signature with '%s' on '%s', as the key is revoked.\n",
 						s->fpr, filename);
 				continue;
 			case GPG_ERR_SIG_EXPIRED:
-				had_valid = TRUE;
+				had_valid = true;
 				if( verbose > 0 ) {
 					time_t timestamp = s->timestamp,
 					      exp_timestamp = s->exp_timestamp;
@@ -485,7 +518,7 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 				}
 				continue;
 			case GPG_ERR_BAD_SIGNATURE:
-				had_broken = TRUE;
+				had_broken = true;
 				if( verbose > 0 ) {
 					fprintf(stderr,
 "WARNING: '%s' has a invalid signature with '%s'\n", filename, s->fpr);
@@ -518,12 +551,47 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 		return RET_ERROR_GPGME;
 	}
 	if( broken != NULL && had_broken && ! had_valid )
-		*broken = TRUE;
+		*broken = true;
 	return RET_OK;
 }
+#endif /* HAVE_LIBGPGME */
 
-/* Read a single chunk from a file, that may be signed. */
-retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool_t *brokensignature) {
+static inline void extractchunk(const char *buffer, const char **begin, const char **end, const char **next) {
+	const char *startofchanges,*endofchanges,*afterchanges;
+
+	startofchanges = buffer;
+	while( *startofchanges == ' ' || *startofchanges == '\t' ||
+			*startofchanges == '\r' || *startofchanges =='\n' )
+		startofchanges++;
+
+	endofchanges = startofchanges;
+	afterchanges = NULL;
+	while( *endofchanges != '\0' ) {
+		if( *endofchanges == '\n' ) {
+			endofchanges++;
+			afterchanges = endofchanges;
+			while( *afterchanges =='\r' )
+				afterchanges++;
+			if( *afterchanges == '\n' )
+				break;
+			endofchanges = afterchanges;
+			afterchanges = NULL;
+		} else
+			endofchanges++;
+	}
+
+	if( afterchanges == NULL )
+		afterchanges = endofchanges;
+	else
+		while( *afterchanges == '\n' || *afterchanges =='\r' )
+			afterchanges++;
+	*begin = startofchanges;
+	*end = endofchanges;
+	*next = afterchanges;
+}
+
+#ifdef HAVE_LIBGPGME
+static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature, bool *failed) {
 	const char *startofchanges,*endofchanges,*afterchanges;
 	char *chunk;
 	gpgme_error_t err;
@@ -532,31 +600,31 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 	char *plain_data;
 	retvalue r;
 	struct strlist validfingerprints, allfingerprints;
-	bool_t foundbroken = FALSE;
+	bool foundbroken = false;
 
 	strlist_init(&validfingerprints);
 	strlist_init(&allfingerprints);
 
-	r = signature_init(FALSE);
+	r = signature_init(false);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
-	/* I wished from_fd would actually read the data like this one does... */
-	err = gpgme_data_new_from_file(&dh_gpg, filename, 1);
-	if( err != 0 ) {
+	err = gpgme_data_new_from_mem(&dh_gpg, buffer, bufferlen, 0);
+	if( err != 0 )
 		return gpgerror(err);
-	}
+
 	err = gpgme_data_new(&dh);
 	if( err != 0 ) {
 		gpgme_data_release(dh_gpg);
 		return gpgerror(err);
 	}
-	err = gpgme_op_verify(context,dh_gpg,NULL,dh);
+	err = gpgme_op_verify(context, dh_gpg, NULL, dh);
 	if( gpgme_err_code(err) == GPG_ERR_NO_DATA ) {
 		if( verbose > -1 )
-			fprintf(stderr,"Data seems not to be signed trying to use directly...\n");
+			fprintf(stderr,"Data seems not to be signed trying to use directly....\n");
 		gpgme_data_release(dh);
-		plain_data = gpgme_data_release_and_get_mem(dh_gpg,&plain_len);
+		gpgme_data_release(dh_gpg);
+		return RET_NOTHING;
 	} else {
 		if( err != 0 ) {
 			gpgme_data_release(dh_gpg);
@@ -564,7 +632,7 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 			return gpgerror(err);
 		}
 		if( validkeys != NULL || allkeys != NULL || brokensignature != NULL ) {
-			r = checksigs(filename,
+			r = checksigs(filenametoshow,
 				(validkeys!=NULL)?&validfingerprints:NULL,
 				(allkeys!=NULL)?&allfingerprints:NULL,
 				(brokensignature!=NULL)?&foundbroken:NULL);
@@ -578,18 +646,23 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 		plain_data = gpgme_data_release_and_get_mem(dh,&plain_len);
 		if( plain_data == NULL ) {
 			fprintf(stderr,
-"gpgme failed to extract the plain data out of '%s'.\n"
+"(not yet fatal) ERROR: libgpgme failed to extract the plain data out of\n"
+"'%s'.\n"
 "While it did so in a way indicating running out of memory, experience says\n"
 "this also happens when gpg returns a error code it does not understand.\n"
-"To check this please try running gpg --output '%s' manually.\n",
+"To check this please try running gpg --verify '%s' manually.\n"
+"Continuing extracting it ignoring all signatures...",
 					filenametoshow, filenametoshow);
-			return RET_ERROR_GPGME;
+			*failed = true;
+			return RET_NOTHING;
 		}
 	}
 
 	if( plain_data == NULL )
 		r = RET_ERROR_OOM;
 	else {
+		// TODO: check if the new extractchunk can be used...
+
 		startofchanges = plain_data;
 		while( (size_t)(startofchanges - plain_data) < plain_len &&
 				*startofchanges != '\0' && xisspace(*startofchanges)) {
@@ -654,6 +727,141 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 	}
 	return r;
 }
+#endif /* HAVE_LIBGPGME */
+
+/* Read a single chunk from a file, that may be signed. */
+retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature) {
+	char *chunk, *h, *afterchunk;
+	const char *startofchanges,*endofchanges,*afterchanges;
+	size_t chunklen, len;
+	retvalue r;
+	bool failed = false;
+
+	r = readtextfile(filename, filenametoshow, &chunk, &chunklen);
+	if( !RET_IS_OK(r) )
+		return r;
+
+	if( chunklen == 0 ) {
+		fprintf(stderr, "Unexpected empty file '%s'!\n",
+					filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+
+	extractchunk(chunk, &startofchanges, &endofchanges, &afterchanges);
+	if( endofchanges == startofchanges ) {
+		fprintf(stderr, "Could only find spaces within '%s'!\n",
+					filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+
+	/* fast-track unsigned chunks: */
+	if( startofchanges[0] != '-' && *afterchanges == '\0' ) {
+		if( verbose > -1 )
+			fprintf(stderr,"Data seems not to be signed trying to use directly...\n");
+		len = chunk_extract(chunk, chunk, &afterchunk);
+		assert( *afterchunk == '\0' );
+		assert( chunk[len] == '\0' );
+		h = realloc(chunk, len + 1);
+		if( h != NULL )
+			chunk = h;
+		*chunkread = chunk;
+		if( validkeys != NULL )
+			strlist_init(validkeys);
+		if( allkeys != NULL )
+			strlist_init(allkeys);
+		if( brokensignature != NULL )
+			*brokensignature = false;
+		return RET_OK;
+	}
+
+	if( startofchanges[0] != '-' ) {
+		fprintf(stderr, "Error parsing '%s': Seems not to be signed but has spurious empty line.\n", filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+
+#ifdef HAVE_LIBGPGME
+	r = extract_signed_data(chunk, chunklen, filenametoshow, chunkread,
+			validkeys, allkeys, brokensignature, &failed);
+	if( r != RET_NOTHING ) {
+		free(chunk);
+		return r;
+	}
+#endif
+
+	/* We have no libgpgme, it failed, or could not find signature data,
+	 * trying to extract it manually, ignoring signatures: */
+
+	if( *afterchanges == '\0' ) {
+		fprintf(stderr,
+"First non-space character is a '-' but there is no empty line in\n"
+"'%s'.\n"
+"Unable to extract any data from it!\n", filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+	if( strncmp(startofchanges, "-----BEGIN", 10) != 0 ) {
+		fprintf(stderr,
+"Strange content of '%s': First non-space character is '-',\n"
+"but it does not begin with '-----BEGIN'.\n", filenametoshow);
+		failed = true;
+#ifndef HAVE_LIBGPGME
+	} else {
+		fprintf(stderr,
+"Cannot check signatures from '%s' as compiled without support for libgpgme!\n"
+"Extracting the content manually without looking at the signature...\n", filenametoshow);
+#endif
+	}
+
+	len = chunk_extract(chunk, afterchanges, &afterchunk);
+
+	if( len == 0 ) {
+		fprintf(stderr,"Could not find any data within '%s'!\n",
+				filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+
+	if( *afterchunk == '\0' ) {
+		const char *endmarker;
+
+		endmarker = strstr(chunk, "\n-----");
+		if( endmarker != NULL ) {
+			endmarker++;
+			assert( (size_t)(endmarker-chunk) < len );
+			len = endmarker-chunk;
+			chunk[len] = '\0';
+		} else {
+			fprintf(stderr,
+"ERROR: Could not find end marker of signed data within '%s'.\n"
+"Cannot determine what is data and what is not!\n",
+					filenametoshow);
+			free(chunk);
+			return RET_ERROR;
+		}
+	} else if( strncmp(afterchunk, "-----", 5) != 0 ) {
+		fprintf(stderr,"ERROR: Spurious empty line within '%s'.\n"
+"Cannot determine what is data and what is not!\n",
+				filenametoshow);
+		free(chunk);
+		return RET_ERROR;
+	}
+
+	assert( chunk[len] == '\0' );
+	h = realloc(chunk, len + 1);
+	if( h != NULL )
+		chunk = h;
+	*chunkread = chunk;
+	if( validkeys != NULL )
+		strlist_init(validkeys);
+	if( allkeys != NULL )
+		strlist_init(allkeys);
+	if( brokensignature != NULL )
+		*brokensignature = false;
+	return RET_OK;
+}
 
 struct signedfile {
 	char *plainfilename, *newplainfilename;
@@ -699,9 +907,9 @@ static retvalue newpossiblysignedfile(const char *directory, const char *basenam
 	return RET_OK;
 }
 
-retvalue signedfile_free(struct signedfile *f) {
+void signedfile_free(struct signedfile *f) {
 	if( f == NULL )
-		return RET_NOTHING;
+		return;
 	assert( f->fd < 0 );
 	if( f->newplainfilename != NULL ) {
 		(void)unlink(f->newplainfilename);
@@ -714,7 +922,7 @@ retvalue signedfile_free(struct signedfile *f) {
 	}
 	free(f->signfilename);
 	free(f);
-	return RET_OK;
+	return;
 }
 
 retvalue signature_startsignedfile(const char *directory, const char *basename, UNUSED(const char *options), struct signedfile **out) {
@@ -748,8 +956,8 @@ void signedfile_write(struct signedfile *f, const void *data, size_t len) {
 			written = write(f->fd, data, len);
 			if( written < 0 ) {
 				int e = errno;
-				fprintf(stderr, "Error writing to file '%s': %s\n",
-						f->newplainfilename,
+				fprintf(stderr, "Error %d writing to file '%s': %s\n",
+						e, f->newplainfilename,
 						strerror(e));
 				(void)close(f->fd);
 				(void)unlink(f->newplainfilename);
@@ -772,8 +980,8 @@ retvalue signedfile_prepare(struct signedfile *f, const char *options) {
 		f->fd = -1;
 		if( ret < 0 ) {
 			int e = errno;
-			fprintf(stderr, "Error writing to file '%s': %s\n",
-					f->newplainfilename,
+			fprintf(stderr, "Error %d writing to file '%s': %s\n",
+					e, f->newplainfilename,
 					strerror(e));
 			(void)unlink(f->newplainfilename);
 			free(f->newplainfilename);
@@ -794,6 +1002,9 @@ retvalue signedfile_prepare(struct signedfile *f, const char *options) {
 		if( f->newsignfilename == NULL )
 			return RET_ERROR_OOM;
 
+		// TODO: make this in-situ (i.e. not signing the file but the memory
+		// content to be saved before):
+
 		r = signature_sign(options,
 				f->newplainfilename,
 				f->newsignfilename);
@@ -803,8 +1014,8 @@ retvalue signedfile_prepare(struct signedfile *f, const char *options) {
 	return RET_OK;
 }
 
-retvalue signedfile_finalize(struct signedfile *f, bool_t *toolate) {
-	int result = RET_OK, r;
+retvalue signedfile_finalize(struct signedfile *f, bool *toolate) {
+	retvalue result = RET_OK, r;
 	int e;
 
 	if( f->newsignfilename != NULL && f->signfilename != NULL ) {
@@ -823,7 +1034,7 @@ retvalue signedfile_finalize(struct signedfile *f, bool_t *toolate) {
 			/* does not need deletion any more */
 			free(f->newsignfilename);
 			f->newsignfilename = NULL;
-			*toolate = TRUE;
+			*toolate = true;
 		}
 	}
 	e = rename(f->newplainfilename, f->plainfilename);
