@@ -6,15 +6,15 @@
 #warning "What's hapening here?"
 #endif
 
-/* does not need to be called if allowpassphrase if FALSE,
+/* does not need to be called if allowpassphrase if false,
  * argument will only take effect if called the first time */
-retvalue signature_init(bool_t allowpassphrase);
+retvalue signature_init(bool allowpassphrase);
 
 retvalue signature_check(const char *options, const char *releasegpg, const char *release);
 retvalue signature_sign(const char *options, const char *filename, const char *signeturename);
 
 /* Read a single chunk from a file, that may be signed. */
-retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool_t *brokensignature);
+retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature);
 
 struct signedfile;
 
@@ -24,9 +24,9 @@ void signedfile_write(struct signedfile *, const void *, size_t);
 /* generate signature in temporary file */
 retvalue signedfile_prepare(struct signedfile *, const char *options);
 /* move temporary files to final places */
-retvalue signedfile_finalize(struct signedfile *, bool_t *toolate);
+retvalue signedfile_finalize(struct signedfile *, bool *toolate);
 /* may only be called after signedfile_prepare */
-retvalue signedfile_free(struct signedfile *);
+void signedfile_free(/*@only@*/struct signedfile *);
 
 void signatures_done(void);
 
