@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2006,2007 Bernhard R. Link
+ *  Copyright (C) 2006,2007,2008 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -27,15 +27,13 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <time.h>
-#define DEFINE_IGNORE_VARIABLES
 #include "error.h"
-#include "ignore.h"
+#include "filecntl.h"
 #include "mprintf.h"
 #include "strlist.h"
 #include "names.h"
 #include "dirs.h"
-#include "copyfile.h"
-#include "md5sum.h"
+#include "checksums.h"
 #include "chunks.h"
 #include "chunkedit.h"
 #include "signature.h"
@@ -43,10 +41,10 @@
 
 /* for compatibility with used code */
 int verbose=0;
-bool_t interrupted(void) {return FALSE;}
+bool interrupted(void) {return false;}
 
-static void about(bool_t help) NORETURN;
-static void about(bool_t help) {
+static void about(bool help) NORETURN;
+static void about(bool help) {
 	fprintf(help?stdout:stderr,
 "modifychanges: Modify a Debian style .changes file\n"
 "Syntax: modifychanges [--create] <changesfile> <commands>\n"
@@ -77,7 +75,7 @@ struct binaryfile {
 	char *maintainer;
 	char *section, *priority;
 	char *shortdescription;
-	bool_t hasmd5sums;
+	bool hasmd5sums;
 };
 
 static void binaryfile_free(struct binaryfile *p) {
@@ -100,12 +98,14 @@ static void binaryfile_free(struct binaryfile *p) {
 enum filetype { ft_UNKNOWN,
 			ft_TAR_GZ, ft_ORIG_TAR_GZ, ft_DIFF_GZ,
 	                ft_TAR_BZ2, ft_ORIG_TAR_BZ2, ft_DIFF_BZ2,
+	                ft_TAR_LZMA, ft_ORIG_TAR_LZMA, ft_DIFF_LZMA,
 #define ft_MaxInSource ft_DSC-1
 			ft_DSC, ft_DEB, ft_UDEB , ft_Count};
 #define ft_Max ft_Count-1
 static const char * const typesuffix[ft_Count] = { "?",
 	".tar.gz", ".orig.tar.gz", ".diff.gz",
 	".tar.bz2", ".orig.tar.bz2", ".diff.bz2",
+	".tar.lzma", ".orig.tar.lzma", ".diff.lzma",
 	".dsc", ".deb", ".udeb"};
 
 struct dscfile {
@@ -122,10 +122,10 @@ struct dscfile {
 	size_t filecount;
 	struct sourcefile {
 		char *basename;
-		char *expectedmd5sum;
+		struct checksums *expectedchecksums;
 		struct fileentry *file;
 	} *files;
-	bool_t parsed, modified;
+	bool parsed, modified, checksumsimproved;
 };
 
 static void dscfile_free(struct dscfile *p) {
@@ -145,7 +145,7 @@ static void dscfile_free(struct dscfile *p) {
 	if( p->files != NULL )
 		for( i = 0 ; i < p->filecount ; i++ ) {
 			free(p->files[i].basename);
-			free(p->files[i].expectedmd5sum);
+			checksums_free(p->files[i].expectedchecksums);
 		}
 	free(p->files);
 	free(p);
@@ -155,8 +155,8 @@ struct fileentry {
 	struct fileentry *next;
 	char *basename; size_t namelen;
 	char *fullfilename;
-	char *changesmd5sum, /* NULL means was not listed there yet */
-	     *realmd5sum;
+	struct checksums *checksumsfromchanges, /* NULL means was not listed there yet */
+			 *realchecksums;
 	char *section, *priority;
 	enum filetype type;
 	/* only if type deb or udeb */
@@ -166,7 +166,7 @@ struct fileentry {
 	int refcount;
 };
 struct changes;
-static struct fileentry *add_fileentry(struct changes *c, const char *basename, size_t len, bool_t source);
+static struct fileentry *add_fileentry(struct changes *c, const char *basename, size_t len, bool source);
 
 struct changes {
 	/* the filename of the .changes file */
@@ -185,10 +185,10 @@ struct changes {
 		char *name;
 		char *description;
 		struct binaryfile *files;
-		bool_t missedinheader, uncheckable;
+		bool missedinheader, uncheckable;
 	} *binaries;
 	struct fileentry *files;
-	bool_t modified;
+	bool modified, checksumsimproved;
 };
 
 static void fileentry_free(struct fileentry *f) {
@@ -196,8 +196,8 @@ static void fileentry_free(struct fileentry *f) {
 		return;
 	free(f->basename);
 	free(f->fullfilename);
-	free(f->changesmd5sum);
-	free(f->realmd5sum);
+	checksums_free(f->checksumsfromchanges);
+	checksums_free(f->realchecksums);
 	free(f->section);
 	free(f->priority);
 	if( f->type == ft_DEB || f->type == ft_UDEB ) {
@@ -236,7 +236,7 @@ static void changes_free(struct changes *c) {
 	free(c);
 }
 
-static struct fileentry *add_fileentry(struct changes *c, const char *basename, size_t len, bool_t source) {
+static struct fileentry *add_fileentry(struct changes *c, const char *basename, size_t len, bool source) {
 	struct fileentry **fp = &c->files;
 	struct fileentry *f;
 	while( (f=*fp) != NULL ) {
@@ -266,7 +266,7 @@ static struct fileentry *add_fileentry(struct changes *c, const char *basename,
 
 static retvalue searchforfile(const char *changesdir, const char *basename,
 		const struct strlist *searchpath, char **result) {
-	int i; bool_t found;
+	int i; bool found;
 	char *fullname = calc_dirconcat(changesdir,basename);
 	if( fullname == NULL )
 		return RET_ERROR_OOM;
@@ -280,7 +280,7 @@ static retvalue searchforfile(const char *changesdir, const char *basename,
 		if( fullname == NULL )
 			return RET_ERROR_OOM;
 		if( isregularfile(fullname) ) {
-			found = TRUE;
+			found = true;
 			break;
 		}
 		i++;
@@ -370,8 +370,8 @@ static struct binary *get_binary(struct changes *c, const char *p, size_t len) {
 		c->binaries[j].name = name;
 		c->binaries[j].description = NULL;
 		c->binaries[j].files = NULL;
-		c->binaries[j].missedinheader = TRUE;
-		c->binaries[j].uncheckable = FALSE;
+		c->binaries[j].missedinheader = true;
+		c->binaries[j].uncheckable = false;
 	}
 	assert( j < c->binarycount );
 	return &c->binaries[j];
@@ -409,6 +409,7 @@ static retvalue parse_changes_description(struct changes *c, struct strlist *tmp
 static retvalue parse_changes_files(struct changes *c, struct strlist *tmp) {
 	int i;
 	struct fileentry *f;
+	retvalue r;
 
 	for( i = 0 ; i < tmp->count ; i++ ) {
 		const char *p,*md5start, *md5end, *sizestart, *sizeend, *sectionstart, *sectionend, *priostart, *prioend, *filestart, *fileend;
@@ -451,15 +452,17 @@ static retvalue parse_changes_files(struct changes *c, struct strlist *tmp) {
 			fprintf(stderr,"Unexpected sixth argument in '%s'!\n", tmp->values[i]);
 			return RET_ERROR;
 		}
-		f = add_fileentry(c, filestart, fileend-filestart, FALSE);
-		if( f->changesmd5sum != NULL ) {
+		f = add_fileentry(c, filestart, fileend-filestart, false);
+		if( f->checksumsfromchanges != NULL ) {
 			fprintf(stderr, "WARNING: Multiple occourance of '%s' in .changes file!\nIgnoring all but the first one.\n",
 					f->basename);
 			continue;
 		}
-		f->changesmd5sum = names_concatmd5sumandsize(md5start,md5end,sizestart,sizeend);
-		if( f->changesmd5sum == NULL )
-			return RET_ERROR_OOM;
+		r = checksums_set(&f->checksumsfromchanges,
+				md5start, md5end - md5start,
+				sizestart, sizeend - sizestart);
+		if( RET_WAS_ERROR(r) )
+			return r;
 		if( sectionend - sectionstart == 1 && *sectionstart == '-' ) {
 			f->section = NULL;
 		} else {
@@ -495,7 +498,7 @@ static retvalue read_dscfile(const char *fullfilename, struct dscfile **dsc) {
 		free(n);
 		return r;
 	}
-	r = chunk_getname(n->controlchunk, "Source",&n->name,FALSE);
+	r = chunk_getname(n->controlchunk, "Source", &n->name, false);
 	if( RET_WAS_ERROR(r) ) {
 		dscfile_free(n);
 		return r;
@@ -531,7 +534,7 @@ static retvalue read_dscfile(const char *fullfilename, struct dscfile **dsc) {
 			n->files[j].file = NULL;
 			r = calc_parsefileline(tmp.values[i],
 					&n->files[j].basename,
-					&n->files[j].expectedmd5sum);
+					&n->files[j].expectedchecksums);
 			if( RET_WAS_ERROR(r) ) {
 				strlist_done(&tmp);
 				dscfile_free(n);
@@ -554,7 +557,7 @@ static retvalue read_dscfile(const char *fullfilename, struct dscfile **dsc) {
 static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
 	struct dscfile *n;
 	retvalue r;
-	int i;
+	size_t i;
 
 	if( dscfile->fullfilename == NULL )
 		return RET_NOTHING;
@@ -566,7 +569,7 @@ static retvalue parse_dsc(struct fileentry *dscfile, struct changes *changes) {
 		n->files[i].file = add_fileentry(changes,
 				n->files[i].basename,
 				strlen(n->files[i].basename),
-				TRUE);
+				true);
 		if( n->files[i].file == NULL ) {
 			dscfile_free(n);
 			return RET_ERROR_OOM;
@@ -586,7 +589,7 @@ static retvalue write_dsc_file(struct fileentry *dscfile, unsigned int flags) {
 	struct chunkeditfield *cef;
 	retvalue r;
 	char *control; size_t controllen;
-	char *md5sum;
+	struct checksums *checksums;
 	char *destfilename;
 
 	if( flagset(DSC_WRITE_FILES) ) {
@@ -595,8 +598,9 @@ static retvalue write_dsc_file(struct fileentry *dscfile, unsigned int flags) {
 			return RET_ERROR_OOM;
 		for( i = 0 ; i < dsc->filecount ; i++ ) {
 			struct sourcefile *f = &dsc->files[i];
+			const char *md5sum = checksums_getmd5sum(f->expectedchecksums);
 			cef_setline(cef, i, 2,
-					f->expectedmd5sum, f->basename, NULL);
+					md5sum, f->basename, NULL);
 		}
 	} else
 		cef = NULL;
@@ -619,7 +623,7 @@ static retvalue write_dsc_file(struct fileentry *dscfile, unsigned int flags) {
 		return RET_ERROR_OOM;
 	}
 
-	r = md5sum_replace(destfilename, control, controllen, &md5sum);
+	r = checksums_replace(destfilename, control, controllen, &checksums);
 	if( RET_WAS_ERROR(r) ) {
 		free(destfilename);
 		free(control);
@@ -629,8 +633,8 @@ static retvalue write_dsc_file(struct fileentry *dscfile, unsigned int flags) {
 
 	free(dscfile->fullfilename);
 	dscfile->fullfilename = destfilename;
-	free(dscfile->realmd5sum);
-	dscfile->realmd5sum = md5sum;
+	checksums_free(dscfile->realchecksums);
+	dscfile->realchecksums = checksums;
 	free(dsc->controlchunk);
 	dsc->controlchunk = control;
 	return RET_OK;
@@ -653,7 +657,7 @@ static retvalue read_binaryfile(const char *fullfilename, struct binaryfile **re
 			return RET_NOTHING;
 	}
 
-	r = chunk_getname(n->controlchunk, "Package", &n->name, FALSE);
+	r = chunk_getname(n->controlchunk, "Package", &n->name, false);
 	if( RET_WAS_ERROR(r) ) {
 		binaryfile_free(n);
 		return r;
@@ -762,7 +766,7 @@ static retvalue processfiles(const char *changesfilename, struct changes *change
 					free(dir);
 					return RET_ERROR_OOM;
 				}
-				b->uncheckable = TRUE;
+				b->uncheckable = true;
 			}
 		}
 	}
@@ -881,7 +885,7 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 	strlist_init(&binaries);
 
 	for( f = c->files; f != NULL ; f = f->next ) {
-		if( f->changesmd5sum != NULL )
+		if( f->checksumsfromchanges != NULL )
 			filecount++;
 	}
 
@@ -891,10 +895,11 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 			return RET_ERROR_OOM;
 		i = 0;
 		for( f = c->files; f != NULL ; f = f->next ) {
-			if( f->changesmd5sum == NULL )
+			const char *md5sum;
+			if( f->checksumsfromchanges == NULL )
 				continue;
-			cef_setline(cef, i, 4,
-					f->changesmd5sum,
+			md5sum = checksums_getmd5sum(f->checksumsfromchanges);
+			cef_setline(cef, i, 4, md5sum,
 					f->section?f->section:"-",
 					f->priority?f->priority:"-",
 					f->basename, NULL);
@@ -1081,7 +1086,7 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 
 	// TODO: add options to place changed files in different directory...
 
-	r = md5sum_replace(changesfilename, control, controllen, NULL);
+	r = checksums_replace(changesfilename, control, controllen, NULL);
 	if( RET_WAS_ERROR(r) ) {
 		free(control);
 		return r;
@@ -1093,7 +1098,7 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 	return RET_OK;
 }
 
-static retvalue getmd5sums(struct changes *changes) {
+static retvalue getchecksums(struct changes *changes) {
 	struct fileentry *file;
 	retvalue r;
 
@@ -1101,45 +1106,54 @@ static retvalue getmd5sums(struct changes *changes) {
 
 		if( file->fullfilename == NULL )
 			continue;
-		assert( file->realmd5sum == NULL );
+		assert( file->realchecksums == NULL );
 
-		r = md5sum_read(file->fullfilename, &file->realmd5sum);
+		r = checksums_read(file->fullfilename, &file->realchecksums);
 		if( r == RET_ERROR_OOM )
 			return r;
 		else if( !RET_IS_OK(r) ) {
 			// assume everything else is not fatal and means
 			// a file not readable...
-			file->realmd5sum = NULL;
+			file->realchecksums = NULL;
 		}
 	}
 	return RET_OK;
 }
 
-static void verify_sourcefile_md5sums(struct sourcefile *f, const char *dscfile) {
-	if( f->file == NULL ) {
+static void verify_sourcefile_checksums(struct sourcefile *f, const char *dscfile) {
+	assert( f->file != NULL );
+
+	if( f->file->checksumsfromchanges == NULL ) {
 		if( endswith(f->basename, typesuffix[ft_ORIG_TAR_GZ])
 		|| endswith(f->basename, typesuffix[ft_ORIG_TAR_BZ2])) {
 			fprintf(stderr,
-"Could not check md5sum of '%s', as not included.\n",
+"Not checking checksums of '%s', as not included in .changes file.\n",
 				f->basename);
-		} else {
+			return;
+		} else if( f->file->realchecksums == NULL ) {
 			fprintf(stderr,
 "ERROR: File '%s' mentioned in '%s' was not found and is not mentioned in the .changes!\n",
 				f->basename, dscfile);
+			return;
 		}
-		return;
 	}
-	if( strcmp(f->expectedmd5sum, f->file->realmd5sum) != 0 ) {
-		if( f->file->changesmd5sum != NULL &&
-		    strcmp(f->expectedmd5sum, f->file->changesmd5sum) == 0 )
-			fprintf(stderr,
-"ERROR: '%s' lists the same wrong md5sum for '%s' like the .changes file!\n",
-				dscfile, f->basename);
-		else
-			fprintf(stderr,
-"ERROR: '%s' says '%s' has md5sum %s but it has %s!\n",
-				dscfile, f->basename,
-				f->expectedmd5sum, f->file->realmd5sum);
+	if( f->file->realchecksums == NULL )
+		/* there will be an message later about that */
+		return;
+	if( checksums_check(f->expectedchecksums, f->file->realchecksums, NULL))
+		return;
+
+	if( f->file->checksumsfromchanges != NULL &&
+	    checksums_check(f->expectedchecksums, f->file->checksumsfromchanges, NULL) )
+		fprintf(stderr,
+"ERROR: checksums of '%s' differ from the ones listed in both '%s' and the .changes file!\n",
+				f->basename, dscfile);
+	else {
+		fprintf(stderr,
+"ERROR: checksums of '%s' differ from those listed in '%s':\n!\n",
+				f->basename, dscfile);
+		checksums_printdifferences(stderr,
+				f->expectedchecksums, f->file->realchecksums);
 	}
 }
 
@@ -1191,7 +1205,7 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 	for( file = changes->files; file != NULL ; file = file->next ) {
 		const char *name, *version, *p;
 		size_t namelen, versionlen, l;
-		bool_t has_tar, has_diff, has_orig;
+		bool has_tar, has_diff, has_orig;
 
 		if( file->type != ft_DSC )
 			continue;
@@ -1326,9 +1340,9 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 					(unsigned int)namelen, name,
 					(unsigned int)versionlen, version);
 		}
-		has_tar = FALSE;
-		has_diff = FALSE;
-		has_orig = FALSE;
+		has_tar = false;
+		has_diff = false;
+		has_orig = false;
 		for( j = 0 ; j < file->dsc->filecount ; j++ ) {
 			const struct sourcefile *f = &file->dsc->files[j];
 			size_t expectedversionlen;
@@ -1340,26 +1354,32 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 						file->fullfilename,
 						f->basename);
 					break;
-				case ft_TAR_GZ: case ft_TAR_BZ2:
+				case ft_TAR_GZ:
+				case ft_TAR_BZ2:
+				case ft_TAR_LZMA:
 					if( has_tar || has_orig )
 						fprintf(stderr,
 "ERROR: '%s' lists multiple .tar files!\n",
 						file->fullfilename);
-					has_tar = TRUE;
+					has_tar = true;
 					break;
-				case ft_ORIG_TAR_GZ: case ft_ORIG_TAR_BZ2:
+				case ft_ORIG_TAR_GZ:
+				case ft_ORIG_TAR_BZ2:
+				case ft_ORIG_TAR_LZMA:
 					if( has_tar || has_orig )
 						fprintf(stderr,
 "ERROR: '%s' lists multiple .tar files!\n",
 						file->fullfilename);
-					has_orig = TRUE;
+					has_orig = true;
 					break;
-				case ft_DIFF_GZ: case ft_DIFF_BZ2:
+				case ft_DIFF_GZ:
+				case ft_DIFF_BZ2:
+				case ft_DIFF_LZMA:
 					if( has_diff )
 						fprintf(stderr,
 "ERROR: '%s' lists multiple .diff files!\n",
 						file->fullfilename);
-					has_diff = TRUE;
+					has_diff = true;
 					break;
 				default:
 					assert( f->file->type == ft_UNKNOWN );
@@ -1567,30 +1587,31 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 		// todo: check for md5sums file, verify it...
 	}
 
-	printf("Checking md5sums...\n");
-	r = getmd5sums(changes);
+	printf("Checking checksums...\n");
+	r = getchecksums(changes);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	for( file = changes->files; file != NULL ; file = file->next ) {
 
 		if( file->fullfilename == NULL ) {
-			fprintf(stderr, "WARNING: Could not check md5sum of '%s' as file not found!\n", file->basename);
+			fprintf(stderr, "WARNING: Could not check checksums of '%s' as file not found!\n", file->basename);
 			if( file->type == ft_DSC ) {
-				fprintf(stderr, "WARNING: This file most likely contains other md5sums which could also not be checked because it was not found!\n");
+				fprintf(stderr, "WARNING: This file most likely contains additional checksums which could also not be checked because it was not found!\n");
 			}
 			continue;
 		}
-		if( file->changesmd5sum == NULL )
+		if( file->checksumsfromchanges == NULL )
 			/* nothing to check here */
 			continue;
 
-		if( file->realmd5sum == NULL ) {
-			fprintf(stderr, "WARNING: Could not check md5sum of '%s'! File vanished while checking or not readable?\n", file->basename);
-		} else if( strcmp(file->realmd5sum, file->changesmd5sum) != 0 ) {
-			fprintf(stderr, "ERROR: md5sum of '%s' is %s instead of expected %s!\n",
-					file->fullfilename,
-					file->realmd5sum,
-					file->changesmd5sum);
+		if( file->realchecksums == NULL ) {
+			fprintf(stderr, "WARNING: Could not check checksums of '%s'! File vanished while checking or not readable?\n", file->basename);
+		} else if( !checksums_check(file->realchecksums, file->checksumsfromchanges, NULL)) {
+			fprintf(stderr, "ERROR: checksums of '%s' differ from those listed in .changes:\n",
+					file->fullfilename);
+			checksums_printdifferences(stderr,
+					file->checksumsfromchanges,
+					file->realchecksums);
 		}
 
 		if( file->type == ft_DSC ) {
@@ -1606,8 +1627,8 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 			for( i = 0 ; i < file->dsc->filecount ; i++ ) {
 				struct sourcefile *f = &file->dsc->files[i];
 
-				assert( f->expectedmd5sum != NULL );
-				verify_sourcefile_md5sums(f,file->fullfilename);
+				assert( f->expectedchecksums != NULL );
+				verify_sourcefile_checksums(f,file->fullfilename);
 			}
 		}
 		// TODO: check .deb files
@@ -1615,24 +1636,24 @@ static retvalue verify(const char *changesfilename, struct changes *changes) {
 	return RET_OK;
 }
 
-static bool_t isarg(int argc, char **argv, const char *name) {
+static bool isarg(int argc, char **argv, const char *name) {
 	while( argc > 0 ) {
 		if( strcmp(*argv, name) == 0 )
-			return TRUE;
+			return true;
 		argc--;
 		argv++;
 	}
-	return FALSE;
+	return false;
 }
 
-static retvalue updatemd5sums(const char *changesfilename, struct changes *c, int argc, char **argv) {
+static retvalue updatechecksums(const char *changesfilename, struct changes *c, int argc, char **argv) {
 	retvalue r;
 	struct fileentry *file;
 
-	r = getmd5sums(c);
+	r = getchecksums(c);
 	if( RET_WAS_ERROR(r) )
 		return r;
-	/* first update all .dsc files and perhaps recalculate their md5sums */
+	/* first update all .dsc files and perhaps recalculate their checksums */
 	for( file = c->files; file != NULL ; file = file->next ) {
 		unsigned int i;
 
@@ -1641,61 +1662,67 @@ static retvalue updatemd5sums(const char *changesfilename, struct changes *c, in
 
 		if( file->dsc == NULL ) {
 			fprintf(stderr,
-"WARNING: Could not read '%s', hopeing the content and its md5sum are correct!\n",
+"WARNING: Could not read '%s', hopeing the content and its checksums are correct!\n",
 					file->basename);
 			continue;
 		}
 		assert( file->fullfilename != NULL );
 		for( i = 0 ; i < file->dsc->filecount ; i++ ) {
 			struct sourcefile *f = &file->dsc->files[i];
-			bool_t doit;
-			char *md5sum = NULL;
-			const char *realmd5;
+			bool doit;
+			bool improves;
 
-			assert( f->expectedmd5sum != NULL );
+			assert( f->expectedchecksums != NULL );
 			assert( f->basename != NULL );
 
 			doit = isarg(argc,argv,f->basename);
 			if( argc > 0 && !doit )
 				continue;
 
-			if( f->file == NULL || f->file->changesmd5sum == NULL ) {
+			assert( f->file != NULL );
+			if( f->file->checksumsfromchanges == NULL ) {
 				if( !doit ) {
 					fprintf(stderr,
-"Not checking '%s' as not in .changes and not specified on command line.\n",
+"Not checking/updating '%s' as not in .changes and not specified on command line.\n",
 						f->basename);
 					continue;
 				}
-				// ... get md5sum ..;
-				realmd5 = md5sum;
+				if( f->file->realchecksums == NULL ) {
+					fprintf(stderr, "WARNING: Could not check checksums of '%s'!\n", f->basename);
+					continue;
+				}
 			} else {
-				if( f->file->realmd5sum == NULL ) {
-					fprintf(stderr, "WARNING: Could not check md5sum of '%s'!\n", f->basename);
+				if( f->file->realchecksums == NULL ) {
+					fprintf(stderr, "WARNING: Could not check checksums of '%s'!\n", f->basename);
 					continue;
 				}
-				realmd5 = f->file->realmd5sum;
 			}
 
-			if( strcmp(f->expectedmd5sum, realmd5) == 0 ) {
-				/* already correct */
-				free(md5sum);
+			if( checksums_check(f->expectedchecksums, f->file->realchecksums, &improves) ) {
+				if( !improves ) {
+					/* already correct */
+					continue;
+				}
+				/* future versions might be able to store them in the dsc */
+				r = checksums_combine(&f->expectedchecksums, f->file->realchecksums);
+				if( RET_WAS_ERROR(r) )
+					return r;
+				file->dsc->checksumsimproved = true;
 				continue;
 			}
 			fprintf(stderr,
 "Going to update '%s' in '%s'\nfrom '%s'\nto   '%s'.\n",
 					f->basename, file->fullfilename,
-					realmd5, f->expectedmd5sum);
-			free(f->expectedmd5sum);
-			if( md5sum != NULL )
-				f->expectedmd5sum = md5sum;
-			else {
-				free(md5sum);
-				f->expectedmd5sum = strdup(realmd5);
-				if( f->expectedmd5sum == NULL )
-					return RET_ERROR_OOM;
-			}
-			file->dsc->modified = TRUE;
+					checksums_getmd5sum(f->expectedchecksums),
+					checksums_getmd5sum(f->file->realchecksums));
+			checksums_free(f->expectedchecksums);
+			f->expectedchecksums = checksums_dup(f->file->realchecksums);
+			if( f->expectedchecksums == NULL )
+				return RET_ERROR_OOM;
+			file->dsc->modified = true;
 		}
+		// TODO: once .dsc files can store shas, decide if it needs
+		// an update for those here:
 		if( file->dsc->modified ) {
 			r = write_dsc_file(file, DSC_WRITE_FILES);
 			if( RET_WAS_ERROR(r) )
@@ -1703,30 +1730,42 @@ static retvalue updatemd5sums(const char *changesfilename, struct changes *c, in
 		}
 	}
 	for( file = c->files; file != NULL ; file = file->next ) {
+		bool improves;
 
-		if( file->changesmd5sum == NULL )
+		if( file->checksumsfromchanges == NULL )
 			/* nothing to check here */
 			continue;
-		if( file->realmd5sum == NULL ) {
-			fprintf(stderr, "WARNING: Could not check md5sum of '%s'! Leaving it as it is.\n", file->basename);
+		if( file->realchecksums == NULL ) {
+			fprintf(stderr, "WARNING: Could not check checksums of '%s'! Leaving it as it is.\n", file->basename);
 			continue;
 		}
-		if( strcmp(file->realmd5sum, file->changesmd5sum) == 0 ) {
+		if( checksums_check(file->checksumsfromchanges, file->realchecksums, &improves) ) {
+			if( !improves )
+				continue;
+			/* future versions might store sha sums in .changes: */
+			r = checksums_combine(&file->checksumsfromchanges,
+					file->realchecksums);
+			if( RET_WAS_ERROR(r) )
+				return r;
+			c->checksumsimproved = true;
 			continue;
 		}
 		fprintf(stderr,
 "Going to update '%s' in '%s'\nfrom '%s'\nto   '%s'.\n",
-					file->basename, changesfilename,
-					file->realmd5sum, file->changesmd5sum);
-		free(file->changesmd5sum);
-		file->changesmd5sum = strdup(file->realmd5sum);
-		if( file->changesmd5sum == NULL )
+				file->basename, changesfilename,
+				checksums_getmd5sum(file->checksumsfromchanges),
+				checksums_getmd5sum(file->realchecksums));
+		checksums_free(file->checksumsfromchanges);
+		file->checksumsfromchanges = checksums_dup(file->realchecksums);
+		if( file->checksumsfromchanges == NULL )
 			return RET_ERROR_OOM;
-		c->modified = TRUE;
+		c->modified = true;
 	}
 	if( c->modified ) {
 		return write_changes_file(changesfilename, c, CHANGES_WRITE_FILES);
 	} else
+		// TODO: once .changes files can store shas, decide if it needs
+		// an update for those here:
 		return RET_NOTHING;
 }
 
@@ -1749,18 +1788,18 @@ static retvalue includeallsources(const char *changesfilename, struct changes *c
 		for( i = 0 ; i < file->dsc->filecount ; i++ ) {
 			struct sourcefile *f = &file->dsc->files[i];
 
-			assert( f->expectedmd5sum != NULL );
+			assert( f->expectedchecksums != NULL );
 			assert( f->basename != NULL );
 			assert( f->file != NULL );
 
-			if( f->file->changesmd5sum != NULL )
+			if( f->file->checksumsfromchanges != NULL )
 				continue;
 
 			if( argc > 0 && !isarg(argc,argv,f->basename) )
 				continue;
 
-			f->file->changesmd5sum = strdup(f->expectedmd5sum);
-			if( f->file->changesmd5sum == NULL )
+			f->file->checksumsfromchanges = checksums_dup(f->expectedchecksums);
+			if( f->file->checksumsfromchanges == NULL )
 				return RET_ERROR_OOM;
 			/* copy section and priority information from the dsc */
 			if( f->file->section == NULL && file->section != NULL ) {
@@ -1775,10 +1814,10 @@ static retvalue includeallsources(const char *changesfilename, struct changes *c
 			}
 
 			fprintf(stderr,
-"Going to add '%s' with '%s' to '%s'.\n",
-					f->basename, f->expectedmd5sum,
+"Going to add '%s' to '%s'.\n",
+					f->basename,
 					changesfilename);
-			c->modified = TRUE;
+			c->modified = true;
 		}
 	}
 	if( c->modified ) {
@@ -1792,7 +1831,7 @@ static retvalue adddsc(struct changes *c, const char *dscfilename) {
 	struct fileentry *f;
 	struct dscfile *dsc;
 	char *fullfilename, *basename;
-	int i;
+	size_t i;
 
 	r = findfile(dscfilename, c, NULL, &fullfilename);
 	if( RET_WAS_ERROR(r) )
@@ -1887,29 +1926,29 @@ static retvalue adddsc(struct changes *c, const char *dscfilename) {
 	for( i =  0 ; i < dsc->filecount ; i++ ) {
 		struct fileentry *file;
 		const char *basefilename = dsc->files[i].basename;
-		const char *md5sum = dsc->files[i].expectedmd5sum;
+		const struct checksums *checksums = dsc->files[i].expectedchecksums;
 
 		file = add_fileentry(c, basefilename,
 				strlen(basefilename),
-				TRUE);
+				true);
 		if( file == NULL )
 			return RET_ERROR_OOM;
 		dsc->files[i].file = file;
 		/* make them appear in the .changes file if not there: */
-		if( file->changesmd5sum == NULL ) {
-			file->changesmd5sum = strdup(md5sum);
-			if( file->changesmd5sum == NULL )
+		if( file->checksumsfromchanges == NULL ) {
+			file->checksumsfromchanges = checksums_dup(checksums);
+			if( file->checksumsfromchanges == NULL )
 				return RET_ERROR_OOM;
 		} // TODO: otherwise warn if not the same
 	}
 
-	c->modified = TRUE;
-	r = md5sum_read(f->fullfilename, &f->realmd5sum);
+	c->modified = true;
+	r = checksums_read(f->fullfilename, &f->realchecksums);
 	if( RET_WAS_ERROR(r) ) {
 		return r;
 	}
-	f->changesmd5sum = strdup(f->realmd5sum);
-	if( f->changesmd5sum == NULL ) {
+	f->checksumsfromchanges = checksums_dup(f->realchecksums);
+	if( f->checksumsfromchanges == NULL ) {
 		return RET_ERROR_OOM;;
 	}
 	/* for a "extended" dsc with section or priority or
@@ -2089,20 +2128,20 @@ static retvalue adddeb(struct changes *c, const char *debfilename) {
 	}
 	deb->next = deb->binary->files;
 	deb->binary->files = deb;
-	deb->binary->missedinheader = FALSE;
-	c->modified = TRUE;
-	r = md5sum_read(f->fullfilename, &f->realmd5sum);
+	deb->binary->missedinheader = false;
+	c->modified = true;
+	r = checksums_read(f->fullfilename, &f->realchecksums);
 	if( RET_WAS_ERROR(r) ) {
 		return r;
 	}
-	f->changesmd5sum = strdup(f->realmd5sum);
-	if( f->changesmd5sum == NULL ) {
+	f->checksumsfromchanges = checksums_dup(f->realchecksums);
+	if( f->checksumsfromchanges == NULL ) {
 		return RET_ERROR_OOM;;
 	}
 	if( deb->shortdescription != NULL ) {
 		if( deb->binary->description == NULL ) {
 			deb->binary->description = strdup(deb->shortdescription);
-			deb->binary->missedinheader = FALSE;
+			deb->binary->missedinheader = false;
 		} else if( strcmp(deb->binary->description,
 		                  deb->shortdescription) != 0 ) {
 			fprintf(stderr,
@@ -2150,7 +2189,7 @@ static retvalue addrawfile(struct changes *c, const char *filename) {
 	retvalue r;
 	struct fileentry *f;
 	char *fullfilename, *basefilename;
-	char *md5sum;
+	struct checksums *checksums;
 
 	r = findfile(filename, c, NULL, &fullfilename);
 	if( RET_WAS_ERROR(r) )
@@ -2164,7 +2203,7 @@ static retvalue addrawfile(struct changes *c, const char *filename) {
 		free(fullfilename);
 		return RET_ERROR_OOM;
 	}
-	r = md5sum_read(fullfilename, &md5sum);
+	r = checksums_read(fullfilename, &checksums);
 	if( RET_WAS_ERROR(r) ) {
 		free(fullfilename);
 		free(basefilename);
@@ -2174,27 +2213,27 @@ static retvalue addrawfile(struct changes *c, const char *filename) {
 	if( RET_WAS_ERROR(r) ) {
 		free(fullfilename);
 		free(basefilename);
-		free(md5sum);
+		checksums_free(checksums);
 		return r;
 	}
 	if( r == RET_NOTHING ) {
 
 		assert( f != NULL );
 
-		if( f->changesmd5sum != NULL ) {
+		if( f->checksumsfromchanges != NULL ) {
 			/* already listed in .changes */
 
-			if( strcmp(f->changesmd5sum, md5sum) != 0 ) {
-				fprintf(stderr, "ERROR: '%s' already contains a file with name '%s' but different size or md5sum!\n", c->filename, basefilename);
+			if( !checksums_check(f->checksumsfromchanges, checksums, NULL) ) {
+				fprintf(stderr, "ERROR: '%s' already contains a file with name '%s' but different checksums!\n", c->filename, basefilename);
 				free(fullfilename);
 				free(basefilename);
-				free(md5sum);
+				checksums_free(checksums);
 				return RET_ERROR;
 			}
-			printf("'%s' already lists '%s' with same md5sum. Doing nothing.\n", c->filename, basefilename);
+			printf("'%s' already lists '%s' with same checksums. Doing nothing.\n", c->filename, basefilename);
 			free(fullfilename);
 			free(basefilename);
-			free(md5sum);
+			checksums_free(checksums);
 			return RET_NOTHING;
 		} else {
 			/* file already expected by some other part (e.g. a .dsc) */
@@ -2209,15 +2248,14 @@ static retvalue addrawfile(struct changes *c, const char *filename) {
 		fullfilename = NULL;
 	}
 
-	c->modified = TRUE;
-	assert( f->changesmd5sum == NULL );
-	f->changesmd5sum = md5sum;
-	md5sum = NULL;
-	if( f->realmd5sum == NULL )
-		f->realmd5sum = strdup(f->changesmd5sum);
-	if( f->realmd5sum == NULL ) {
+	c->modified = true;
+	assert( f->checksumsfromchanges == NULL );
+	f->checksumsfromchanges = checksums;
+	checksums = NULL;
+	if( f->realchecksums == NULL )
+		f->realchecksums = checksums_dup(f->checksumsfromchanges);
+	if( f->realchecksums == NULL )
 		return RET_ERROR_OOM;;
-	}
 	return RET_OK;
 }
 
@@ -2292,7 +2330,7 @@ static retvalue setdistribution(const char *changesfilename, struct changes *c,
 			CHANGES_WRITE_DISTRIBUTIONS);
 }
 
-static int execute_command(int argc, char **argv, const char *changesfilename, bool_t file_exists, bool_t create_file, struct changes *changesdata) {
+static int execute_command(int argc, char **argv, const char *changesfilename, bool file_exists, bool create_file, struct changes *changesdata) {
 	const char *command = argv[0];
 	retvalue r;
 
@@ -2311,7 +2349,7 @@ static int execute_command(int argc, char **argv, const char *changesfilename, b
 		}
 	} else if( strcasecmp(command, "updatechecksums") == 0 ) {
 		if( file_exists )
-			r = updatemd5sums(changesfilename, changesdata, argc-1, argv+1);
+			r = updatechecksums(changesfilename, changesdata, argc-1, argv+1);
 		else {
 			fprintf(stderr, "No such file '%s'!\n",
 					changesfilename);
@@ -2395,32 +2433,26 @@ int main(int argc,char *argv[]) {
 	static const struct option longopts[] = {
 		{"help", no_argument, NULL, 'h'},
 		{"create", no_argument, NULL, 'C'},
-		{"ignore", required_argument, NULL, 'i'},
 		{"searchpath", required_argument, NULL, 's'},
 		{NULL, 0, NULL, 0},
 	};
 	int c;
 	const char *changesfilename;
-	bool_t file_exists;
-	bool_t create_file = FALSE;
+	bool file_exists;
+	bool create_file = false;
 	struct strlist validkeys,keys;
 	struct strlist searchpath;
 	struct changes *changesdata;
 	retvalue r;
 
 	strlist_init(&searchpath);
-	init_ignores();
-
 
 	while( (c = getopt_long(argc,argv,"+hi:s:",longopts,NULL)) != -1 ) {
 		switch( c ) {
 			case 'h':
-				about(TRUE);
-			case 'i':
-				set_ignore(optarg,FALSE,CONFIG_OWNER_CMDLINE);
-				break;
+				about(true);
 			case 'C':
-				create_file = TRUE;
+				create_file = true;
 				break;
 			case 's':
 				r = splitpath(&searchpath, optarg);
@@ -2433,15 +2465,15 @@ int main(int argc,char *argv[]) {
 		}
 	}
 	if( argc - optind < 2 ) {
-		about(FALSE);
+		about(false);
 	}
-	signature_init(FALSE);
+	signature_init(false);
 
 	changesfilename = argv[optind];
-	if( strcmp(changesfilename,"-") != 0 && !endswith(changesfilename,".changes")
-			&& !IGNORING_(extension,
-				"first argument does not ending with '.changes'\n") )
+	if( strcmp(changesfilename,"-") != 0 && !endswith(changesfilename,".changes") ) {
+		fprintf(stderr, "first argument does not ending with '.changes'\n");
 		exit(EXIT_FAILURE);
+	}
 	file_exists = isregularfile(changesfilename);
 	if( file_exists ) {
 		char *changes;
